@@ -107,82 +107,107 @@ static VOID ApAutoChannelAtBootUp(RTMP_ADAPTER *pAd)
 {
 
     #define SINGLE_BAND 0
-    #define DUAL_BAND 1
-    struct wifi_dev *wdev;
+    #define DUAL_BAND   1
+    struct wifi_dev *pwdev;
     INT i;
     UCHAR NewChannel;
     UCHAR AutoChannel2GDone = FALSE;
     UCHAR AutoChannel5GDone = FALSE;
     BOOLEAN IsAband;
-    INT32 ret; 
+    INT32 ret;
 
-    MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s----------->\n", __FUNCTION__));
+    MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s----------------->\n", __FUNCTION__));
 
     MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("%s: AutoChannelBootup = %d, AutoChannelFlag = %d\n",
     __FUNCTION__, pAd->ApCfg.bAutoChannelAtBootup, pAd->AutoChSelCtrl.AutoChannelFlag));
     
-    if (!pAd->ApCfg.bAutoChannelAtBootup && pAd->AutoChSelCtrl.AutoChannelFlag == 0)
+    if (!pAd->ApCfg.bAutoChannelAtBootup || (pAd->AutoChSelCtrl.AutoChannelFlag == 0))
     {
-   	   return;
+       MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s<-----------------\n", __FUNCTION__));
+       return;
     }
     
     /* Now Enable RxTx*/
     RTMPEnableRxTx(pAd);
 
 
+    pAd->AutoChSelCtrl.p2GScanwdev = NULL;
+    pAd->AutoChSelCtrl.p5GScanwdev = NULL;    
+
     for (i = 0; i < pAd->ApCfg.BssidNum; i++)
     {            
-          wdev = &pAd->ApCfg.MBSSID[i].wdev;
-          if (wdev->channel == 0)
-          {
+        pwdev = &pAd->ApCfg.MBSSID[i].wdev;
+        
+        if (pwdev->channel == 0)
+        {
+            MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
+            ("%s: PhyMode: %d\n", __FUNCTION__, pwdev->PhyMode));
+            
+            if (WMODE_CAP_2G(pwdev->PhyMode) && AutoChannel2GDone == FALSE)
+            {
+                IsAband = FALSE;
+
+                pAd->AutoChSelCtrl.p2GScanwdev = pwdev;
+                
+                pAd->AutoChSelCtrl.PhyMode = pwdev->PhyMode; 
+                
+                /* Now we can receive the beacon and do the listen beacon*/
+                NewChannel = AP_AUTO_CH_SEL(pAd, pAd->ApCfg.AutoChannelAlg, IsAband);
+                
                 MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
-                ("%s: PhyMode: %d\n", __FUNCTION__, wdev->PhyMode));
-                if (WMODE_CAP_2G(wdev->PhyMode) && AutoChannel2GDone == FALSE)
+                ("%s : Auto channel selection for 2G channel =%d\n", __FUNCTION__, NewChannel));
+                
+                ret = HcUpdateChannel(pAd,NewChannel);
+                
+                if(ret < 0 )
                 {
-                        IsAband = FALSE;
-                        /* Now we can receive the beacon and do the listen beacon*/
-                        NewChannel = AP_AUTO_CH_SEL(pAd, pAd->ApCfg.AutoChannelAlg, IsAband);
-                        MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
-                        ("%s : Auto channel selection for 2G channel =%d\n", __FUNCTION__, NewChannel));
-                        ret = HcUpdateChannel(pAd,NewChannel);
-                        if(ret < 0 )
-                        {
-                               MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
-                               ("%s(): Update Channel %d faild, not support this RF\n", __FUNCTION__, NewChannel));		                 
-                        }
-		         AutoChannel2GDone = TRUE; 
-                }   
-                else if (WMODE_CAP_5G(wdev->PhyMode) && AutoChannel5GDone == FALSE)
-                {      
-                        IsAband = TRUE;
-                        /* Now we can receive the beacon and do the listen beacon*/
-                        NewChannel = AP_AUTO_CH_SEL(pAd, pAd->ApCfg.AutoChannelAlg, IsAband);
-                        MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
-                        ("%s : Auto channel selection for 5G channel = %d\n", __FUNCTION__, NewChannel));
-                        ret = HcUpdateChannel(pAd,NewChannel);
-                        if(ret < 0 )
-                        {
-                               MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
-                               ("%s(): Update Channel %d faild, not support this RF\n", __FUNCTION__, NewChannel));		                 
-                        }                        
-                        AutoChannel5GDone = TRUE;                        
-                 }
-           }
+                    MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
+                    ("%s(): Update Channel %d faild, not support this RF\n", __FUNCTION__, NewChannel));		                 
+                }
+                
+                AutoChannel2GDone = TRUE; 
+            }   
+            else if (WMODE_CAP_5G(pwdev->PhyMode) && AutoChannel5GDone == FALSE)
+            {      
+                IsAband = TRUE;
+
+                pAd->AutoChSelCtrl.p5GScanwdev = pwdev;
+
+                pAd->AutoChSelCtrl.PhyMode = pwdev->PhyMode; 
+                
+                /* Now we can receive the beacon and do the listen beacon*/
+                NewChannel = AP_AUTO_CH_SEL(pAd, pAd->ApCfg.AutoChannelAlg, IsAband);
+                
+                MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
+                ("%s : Auto channel selection for 5G channel = %d\n", __FUNCTION__, NewChannel));
+                
+                ret = HcUpdateChannel(pAd,NewChannel);
+                
+                if(ret < 0 )
+                {
+                    MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
+                    ("%s(): Update Channel %d faild, not support this RF\n", __FUNCTION__, NewChannel));		                 
+                } 
+                
+                AutoChannel5GDone = TRUE;                        
+            }
+        }
           
-           if((pAd->CommonCfg.dbdc_mode == DUAL_BAND) && AutoChannel2GDone && AutoChannel5GDone)
-           {
-                  break;
-           }
-           else if ((pAd->CommonCfg.dbdc_mode == SINGLE_BAND) && (AutoChannel2GDone || AutoChannel5GDone))
-           {
-                   break;
-           }
-     }
+        if((pAd->CommonCfg.dbdc_mode == DUAL_BAND) && AutoChannel2GDone && AutoChannel5GDone)
+        {
+            break;
+        }
+        else if ((pAd->CommonCfg.dbdc_mode == SINGLE_BAND) && (AutoChannel2GDone || AutoChannel5GDone))
+        {
+            break;
+        }
+    }
 
     pAd->ApCfg.bAutoChannelAtBootup = FALSE;
+    
     pAd->AutoChSelCtrl.AutoChannelFlag = 0;
     
-    MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s<-----------\n", __FUNCTION__));
+    MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s<-----------------\n", __FUNCTION__));
 }
 
 /*
@@ -196,13 +221,11 @@ static VOID ApChannelCheckForRfIC(RTMP_ADAPTER *pAd,UCHAR RfIC)
 {
 	UCHAR Channel = HcGetChannelByRf(pAd,RfIC);
 	UCHAR PhyMode = HcGetPhyModeByRf(pAd,RfIC);
-
 	if(!HcIsRfSupport(pAd,RfIC))
 	{
 		return;
 	}
 
-
 #ifdef DOT11_N_SUPPORT
 	/* If WMODE_CAP_N(phymode) and BW=40 check extension channel, after select channel	*/
 	N_ChannelCheck(pAd,PhyMode,Channel);
@@ -213,7 +236,10 @@ static VOID ApChannelCheckForRfIC(RTMP_ADAPTER *pAd,UCHAR RfIC)
 NDIS_STATUS APOneShotSettingInitialize(RTMP_ADAPTER *pAd)
 {
 	NDIS_STATUS Status = NDIS_STATUS_SUCCESS;
-
+#ifdef BACKGROUND_SCAN_SUPPORT	
+	UCHAR Channel = 0;
+	INT32	ret=0;
+#endif /* BACKGROUND_SCAN_SUPPORT */
 	struct wifi_dev *wdev = &pAd->ApCfg.MBSSID[MAIN_MBSSID].wdev;
 	/*AP Open*/
 	WifiSysOpen(pAd,wdev);
@@ -253,6 +279,9 @@ NDIS_STATUS APOneShotSettingInitialize(RTMP_ADAPTER *pAd)
 	RRM_CfgInit(pAd);
 #endif /* DOT11K_RRM_SUPPORT */
 
+	/*update ext_cha to hdev*/
+	HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
+
 	BuildChannelList(pAd);
 
 	if (CheckNonOccupancyChannel(pAd, wdev->channel) == FALSE) {
@@ -265,6 +294,41 @@ NDIS_STATUS APOneShotSettingInitialize(RTMP_ADAPTER *pAd)
 
         ApAutoChannelAtBootUp(pAd);
 
+#ifdef BACKGROUND_SCAN_SUPPORT	
+	if (pAd->CommonCfg.dbdc_mode == TRUE) {
+       		pAd->BgndScanCtrl.BgndScanSupport = 0;
+		pAd->BgndScanCtrl.DfsZeroWaitSupport = 0;
+	}        
+#ifdef DOT11_VHT_AC	
+	else if (pAd->CommonCfg.vht_bw == VHT_BW_160 || pAd->CommonCfg.vht_bw == VHT_BW_8080 ) {
+		pAd->BgndScanCtrl.BgndScanSupport = 0;
+		pAd->BgndScanCtrl.DfsZeroWaitSupport = 0;
+	}    
+#endif /* DOT11_VHT_AC */	                
+	else {
+                        pAd->BgndScanCtrl.BgndScanSupport = 1;
+	}
+	
+	Channel  = HcGetChannelByRf(pAd, RFIC_5GHZ);
+	MTWF_LOG(DBG_CAT_INIT, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("Current Channel is %d. DfsZeroWaitSupport=%d\n", Channel, pAd->BgndScanCtrl.DfsZeroWaitSupport));
+	if (Channel !=0 && pAd->BgndScanCtrl.DfsZeroWaitSupport && pAd->BgndScanCtrl.BgndScanSupport && RadarChannelCheck(pAd, Channel)) {
+
+		MTWF_LOG(DBG_CAT_INIT, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("Channel %d is a DFS channel. Trigger Auto Channel selection!\n", Channel));
+		pAd->BgndScanCtrl.DfsZeroWaitChannel = Channel; /* Record original channel for DFS zero wait */
+		pAd->BgndScanCtrl.SkipDfsChannel = 1; //Notify AP_AUTO_CH_SEL skip DFS channel.
+		/* Re-select a non-DFS channel. */
+		Channel = AP_AUTO_CH_SEL(pAd, ChannelAlgBusyTime, TRUE);
+		pAd->BgndScanCtrl.SkipDfsChannel = 0;//Clear.
+		MTWF_LOG(DBG_CAT_INIT, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("Get a new Channel %d for DFS zero wait (temporary) using!\n", Channel));	
+		ret = HcUpdateChannel(pAd,Channel);
+                        	if(ret < 0 )
+                        	{
+                               	MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
+                               	("%s(): Update Channel %d faild, not support this RF\n", __FUNCTION__, Channel));		                 
+                        	}  
+	}
+#endif /* BACKGROUND_SCAN_SUPPORT */
+
 #ifdef DOT11_N_SUPPORT
 #ifdef DOT11N_DRAFT3
 	/*per band checking*/
@@ -496,6 +560,8 @@ VOID APStartUpForMbss(RTMP_ADAPTER *pAd,BSS_STRUCT *pMbss)
 
 #ifdef DOT11_N_SUPPORT
 	RTMPSetPhyMode(pAd, wdev->PhyMode);
+	/*update rate info for wdev*/
+	RTMPUpdateRateInfo(wdev->PhyMode,&wdev->rate);
 
 	if (!WMODE_CAP_N(wdev->PhyMode))
 	{
@@ -519,7 +585,7 @@ VOID APStartUpForMbss(RTMP_ADAPTER *pAd,BSS_STRUCT *pMbss)
     {
         MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("Enable 20/40 BSSCoex Channel Scan(BssCoex=%d)\n",
                     pAd->CommonCfg.bBssCoexEnable));
-        APOverlappingBSSScan(pAd,wdev->PhyMode,wdev->channel);
+        APOverlappingBSSScan(pAd,wdev);
     }
 #endif /* DOT11N_DRAFT3 */
 
@@ -547,11 +613,24 @@ VOID APStartUpForMbss(RTMP_ADAPTER *pAd,BSS_STRUCT *pMbss)
 	}
 #endif /* DOT11_N_SUPPORT */
 
+	//update per wdev bw
+	wdev->bw = wdev->MaxHTPhyMode.field.BW;
+
 	WifiSysApLinkUp(pAd,wdev);
 
 	APKeyTableInit(pAd, wdev);
 
     RadarStateCheck(pAd);
+
+#if defined(MT7615)
+    /*==============================================================================*/
+    /* enable/disable SKU by profile  */
+    TxPowerSKUCtrl(pAd, pAd->CommonCfg.SKUenable);
+
+    /* enable/disable Power Percentage by profile */
+    TxPowerPercentCtrl(pAd, pAd->CommonCfg.PERCENTAGEenable);
+    /*==============================================================================*/
+#endif /* defined(MT7615) */    
 	
 	if(pMbss->wdev.channel > 14)
 		ap_phy_rrm_init_byRf(pAd,RFIC_5GHZ);
@@ -636,6 +715,10 @@ VOID APStartUp(RTMP_ADAPTER *pAd)
 	RTMPSetLED(pAd, LED_LINK_UP);
 #endif /* LED_CONTROL_SUPPORT */
 
+#if defined (CONFIG_WIFI_PKT_FWD)
+	WifiFwdSet(pAd->CommonCfg.WfFwdDisabled);
+#endif /* CONFIG_WIFI_PKT_FWD */
+
 	ApLogEvent(pAd, pAd->CurrentAddress, EVENT_RESET_ACCESS_POINT);
 	pAd->Mlme.PeriodicRound = 0;
 	pAd->Mlme.OneSecPeriodicRound = 0;
@@ -810,6 +893,10 @@ VOID APStartUpForMain(RTMP_ADAPTER *pAd)
 	RTMPSetLED(pAd, LED_LINK_UP);
 #endif /* LED_CONTROL_SUPPORT */
 
+#if defined (CONFIG_WIFI_PKT_FWD)
+	WifiFwdSet(pAd->CommonCfg.WfFwdDisabled);
+#endif /* CONFIG_WIFI_PKT_FWD */
+
 	ApLogEvent(pAd, pAd->CurrentAddress, EVENT_RESET_ACCESS_POINT);
 	pAd->Mlme.PeriodicRound = 0;
 	pAd->Mlme.OneSecPeriodicRound = 0;
@@ -1074,8 +1161,6 @@ VOID MacTableMaintenance(RTMP_ADAPTER *pAd)
 #ifdef RTMP_MAC_PCI
 	unsigned long	IrqFlags;
 #endif /* RTMP_MAC_PCI */
-	UINT fAnyStationPortSecured[HW_BEACON_MAX_NUM];
- 	UINT bss_index;
 	MAC_TABLE *pMacTable;
 #if defined(PRE_ANT_SWITCH) || defined(CFO_TRACK)
 	int lastClient=0;
@@ -1102,8 +1187,6 @@ VOID MacTableMaintenance(RTMP_ADAPTER *pAd)
 	struct wifi_dev * txop_wdev = NULL; 
 	UCHAR    sta_hit_2g_infra_case_number =0;
 
-	NdisZeroMemory(fAnyStationPortSecured, sizeof(fAnyStationPortSecured));
-
 	pMacTable = &pAd->MacTab;
 
 #ifdef MT_MAC
@@ -1147,6 +1230,9 @@ VOID MacTableMaintenance(RTMP_ADAPTER *pAd)
         for (BandIdx=0; BandIdx< DBDC_BAND_NUM; BandIdx++)
         {
 	    pAd->SCSCtrl.SCSMinRssi[BandIdx] =  0; /* (Reset)The minimum RSSI of STA */
+	    pAd->SCSCtrl.OneSecRxByteCount[BandIdx] = 0;
+	    pAd->SCSCtrl.OneSecTxByteCount[BandIdx] = 0;
+   
         }
 #endif /* SMART_CARRIER_SENSE_SUPPORT */
 
@@ -1157,6 +1243,20 @@ VOID MacTableMaintenance(RTMP_ADAPTER *pAd)
 		STA_TR_ENTRY *tr_entry = &pMacTable->tr_entry[wcid];
 		BOOLEAN bDisconnectSta = FALSE;
 		
+#ifdef SMART_CARRIER_SENSE_SUPPORT
+		if (IS_ENTRY_CLIENT(pEntry) || IS_ENTRY_APCLI(pEntry) || IS_ENTRY_REPEATER(pEntry)) {
+			BandIdx = HcGetBandByWdev(pEntry->wdev);
+
+			pAd->SCSCtrl.OneSecRxByteCount[BandIdx] += pEntry->OneSecRxBytes;
+			pAd->SCSCtrl.OneSecTxByteCount[BandIdx] += pEntry->OneSecTxBytes;
+			if (pAd->SCSCtrl.SCSEnable[BandIdx] == SCS_ENABLE) {
+				tmpRssi = RTMPMinRssi(pAd, pEntry->RssiSample.AvgRssi[0], pEntry->RssiSample.AvgRssi[1], 
+	                                                pEntry->RssiSample.AvgRssi[2], pEntry->RssiSample.AvgRssi[3]);
+				if (tmpRssi <pAd->SCSCtrl.SCSMinRssi[BandIdx] )
+					pAd->SCSCtrl.SCSMinRssi[BandIdx] = tmpRssi;
+			}
+		}	
+#endif /* SMART_CARRIER_SENSE_SUPPORT */	
 
 		pEntry->AvgTxBytes = (pEntry->AvgTxBytes == 0) ? \
 							pEntry->OneSecTxBytes : \
@@ -1178,8 +1278,15 @@ VOID MacTableMaintenance(RTMP_ADAPTER *pAd)
 			{
 				pEntry->ReptCliIdleCount++;
 
-				if ((pEntry->bReptEthCli) && (pEntry->ReptCliIdleCount >= MAC_TABLE_AGEOUT_TIME))
+				if ((pEntry->bReptEthCli) 
+					&& (pEntry->ReptCliIdleCount >= MAC_TABLE_AGEOUT_TIME)
+					&& (pEntry->bReptEthBridgeCli == FALSE)) /* Do NOT ageout br0 link. @2016/1/27 */
 				{
+					REPEATER_CLIENT_ENTRY *pReptCliEntry = NULL;
+					pReptCliEntry = &pAd->ApCfg.pRepeaterCliPool[pEntry->MatchReptCliIdx];
+					if (pReptCliEntry) {
+						pReptCliEntry->Disconnect_Sub_Reason = APCLI_DISCONNECT_SUB_REASON_MTM_REMOVE_STA;
+					}
 					MlmeEnqueue(pAd,
 								APCLI_CTRL_STATE_MACHINE,
  								APCLI_CTRL_DISCONNECT_REQ,
@@ -1464,19 +1571,6 @@ VOID MacTableMaintenance(RTMP_ADAPTER *pAd)
 
 		}
 
-#ifdef SMART_CARRIER_SENSE_SUPPORT
-                        if (pAd->SCSCtrl.SCSEnable[0] == SCS_ENABLE || pAd->SCSCtrl.SCSEnable[1] == SCS_ENABLE) {
-                                tmpRssi = RTMPMinRssi(pAd, pEntry->RssiSample.AvgRssi[0], pEntry->RssiSample.AvgRssi[1], 
-                                                pEntry->RssiSample.AvgRssi[2], pEntry->RssiSample.AvgRssi[3]);
-                      BandIdx = HcGetBandByWdev(pEntry->wdev); 
-                      					
-                      if (pAd->SCSCtrl.SCSEnable[BandIdx] == SCS_ENABLE)
-                      {
-                            if (tmpRssi <pAd->SCSCtrl.SCSMinRssi[BandIdx] )
-			        pAd->SCSCtrl.SCSMinRssi[BandIdx] = tmpRssi;
-                       }    
-		}
-#endif /* SMART_CARRIER_SENSE_SUPPORT */
 
 		if (bDisconnectSta)
 		{
@@ -1527,6 +1621,7 @@ VOID MacTableMaintenance(RTMP_ADAPTER *pAd)
 					if (pReptEntry &&
                         (pReptEntry->CliConnectState != REPT_ENTRY_DISCONNT))
 					{
+						pReptEntry->Disconnect_Sub_Reason = APCLI_DISCONNECT_SUB_REASON_MTM_REMOVE_STA;
 						apCliIdx = pReptEntry->MatchApCliIdx;
 						CliIdx = pReptEntry->MatchLinkIdx;
 						MlmeEnqueue(pAd,
@@ -1611,9 +1706,6 @@ VOID MacTableMaintenance(RTMP_ADAPTER *pAd)
 			pMacTable->fAllStationAsRalink[band_idx] = FALSE;
         }
 
-		/* Check if the port is secured */
-		if (tr_entry->PortSecured == WPA_802_1X_PORT_SECURED)
-			fAnyStationPortSecured[pEntry->func_tb_idx]++;
 #ifdef DOT11_N_SUPPORT
 #ifdef DOT11N_DRAFT3
 		if ((pEntry->BSS2040CoexistenceMgmtSupport)
@@ -1711,29 +1803,6 @@ VOID MacTableMaintenance(RTMP_ADAPTER *pAd)
 #ifdef CFO_TRACK
 #endif /* CFO_TRACK */
 
-	/* Update the state of port per MBSS */
-	for (bss_index = BSS0; bss_index < HW_BEACON_MAX_NUM; bss_index++)
-	{
-		struct wifi_dev *wdev = &pAd->ApCfg.MBSSID[bss_index].wdev;
-        if (wdev->tr_tb_idx == 0xff)
-        {
-            continue;//skip uninit tr_tb_idx.
-        }
-		if ((fAnyStationPortSecured[bss_index] > 0)
-#ifdef CONFIG_FPGA_MODE
-			|| (pAd->fpga_ctl.fpga_on & 0x1)
-#endif /* CONFIG_FPGA_MODE */
-		)
-		{
-			wdev->PortSecured = WPA_802_1X_PORT_SECURED;
-			pAd->MacTab.tr_entry[wdev->tr_tb_idx].PortSecured = WPA_802_1X_PORT_SECURED;
-		}
-		else {
-			wdev->PortSecured = WPA_802_1X_PORT_NOT_SECURED;
-			pAd->MacTab.tr_entry[wdev->tr_tb_idx].PortSecured = WPA_802_1X_PORT_NOT_SECURED;
-		}
-	}
-
 #ifdef DOT11_N_SUPPORT
 #ifdef DOT11N_DRAFT3
 	if (pAd->CommonCfg.Bss2040CoexistFlag & BSS_2040_COEXIST_INFO_NOTIFY)
@@ -2493,6 +2562,7 @@ and secondary channel setting.
 */
 INT GetBssCoexEffectedChRange(
 	IN RTMP_ADAPTER *pAd,
+	IN struct wifi_dev *wdev,
 	IN BSS_COEX_CH_RANGE *pCoexChRange,
 	IN UCHAR Channel)
 {
@@ -2516,7 +2586,7 @@ INT GetBssCoexEffectedChRange(
 			pCoexChRange->primaryCh = pAd->ChannelList[index].Channel;
 
 			/* Now check about the secondary and central channel */
-			if(pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+			if(wdev->extcha == EXTCHA_ABOVE)
 			{
 				pCoexChRange->effectChStart = pCoexChRange->primaryCh;
 				pCoexChRange->effectChEnd = pCoexChRange->primaryCh + 4;
@@ -2531,7 +2601,7 @@ INT GetBssCoexEffectedChRange(
 
 			MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE,("5.0GHz: Found CtrlCh idx(%d) from the ChList, ExtCh=%s, PriCh=[Idx:%d, CH:%d], SecCh=[Idx:%d, CH:%d], effected Ch=[CH:%d~CH:%d]!\n",
 										index,
-										((pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE) ? "ABOVE" : "BELOW"),
+										((wdev->extcha == EXTCHA_ABOVE) ? "ABOVE" : "BELOW"),
 										pCoexChRange->primaryCh, pAd->ChannelList[pCoexChRange->primaryCh].Channel,
 										pCoexChRange->secondaryCh, pAd->ChannelList[pCoexChRange->secondaryCh].Channel,
 										pAd->ChannelList[pCoexChRange->effectChStart].Channel,
@@ -2560,7 +2630,7 @@ INT GetBssCoexEffectedChRange(
 			pCoexChRange->primaryCh = index;
 
 			/* Now check about the secondary and central channel */
-			if(pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+			if(wdev->extcha == EXTCHA_ABOVE)
 			{
 				if ((index + 4) < pAd->ChannelListNum)
 				{
@@ -2583,7 +2653,7 @@ INT GetBssCoexEffectedChRange(
 				pCoexChRange->effectChEnd= (cntrCh + 5) > 0 ? (cntrCh + 5) : 0;
 				MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE,("2.4GHz: Found CtrlCh idx(%d) from the ChList, ExtCh=%s, PriCh=[Idx:%d, CH:%d], SecCh=[Idx:%d, CH:%d], effected Ch=[CH:%d~CH:%d]!\n",
 										index,
-										((pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE) ? "ABOVE" : "BELOW"),
+										((wdev->extcha == EXTCHA_ABOVE) ? "ABOVE" : "BELOW"),
 										pCoexChRange->primaryCh, pAd->ChannelList[pCoexChRange->primaryCh].Channel,
 										pCoexChRange->secondaryCh, pAd->ChannelList[pCoexChRange->secondaryCh].Channel,
 										pAd->ChannelList[pCoexChRange->effectChStart].Channel,
@@ -2601,11 +2671,13 @@ INT GetBssCoexEffectedChRange(
 }
 
 
-VOID APOverlappingBSSScan(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
+VOID APOverlappingBSSScan(RTMP_ADAPTER *pAd,struct wifi_dev *wdev)
 {
 	BOOLEAN needFallBack = FALSE;
 	INT chStartIdx, chEndIdx, index,curPriChIdx, curSecChIdx;
 	BSS_COEX_CH_RANGE  coexChRange;
+	UCHAR PhyMode = wdev->PhyMode;
+	UCHAR Channel = wdev->channel;
 
 	if(!HcIsRfSupport(pAd,RFIC_5GHZ))
 	{
@@ -2633,7 +2705,7 @@ VOID APOverlappingBSSScan(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 	if (index < pAd->ChannelListNum)
 	{
 
-		if(pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+		if(wdev->extcha== EXTCHA_ABOVE)
 		{
 			curPriChIdx = index;
 			curSecChIdx = ((index + 4) < pAd->ChannelListNum) ? (index + 4) : (pAd->ChannelListNum - 1);
@@ -2657,7 +2729,7 @@ VOID APOverlappingBSSScan(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 		return;
 	}
 
-	GetBssCoexEffectedChRange(pAd, &coexChRange, Channel);
+	GetBssCoexEffectedChRange(pAd,wdev,&coexChRange, Channel);
 
 	/* Before we do the scanning, clear the bEffectedChannel as zero for latter use. */
 	for (index = 0; index < pAd->ChannelListNum; index++)
@@ -2719,8 +2791,7 @@ VOID APOverlappingBSSScan(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 	{
 		pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth = BW_20;
 		pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = EXTCHA_NONE;
-        pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_NONE;
-
+		HcUpdateExtCha(pAd,wdev->channel,EXTCHA_NONE);
 		pAd->CommonCfg.LastBSSCoexist2040.field.BSS20WidthReq = 1;
 		pAd->CommonCfg.Bss2040CoexistFlag |= BSS_2040_COEXIST_INFO_SYNC;
 
