@@ -183,7 +183,8 @@ static USHORT update_associated_mac_entry(
 			{
 				pAd->CommonCfg.LastBSSCoexist2040.field.BSS20WidthReq = 1;
 				pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth = BW_20;
-                pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_NONE;
+                wdev->extcha = EXTCHA_NONE;
+				HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
 				pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = EXTCHA_NONE;
 				pAd->CommonCfg.Bss2040CoexistFlag |= BSS_2040_COEXIST_INFO_SYNC;
 
@@ -501,6 +502,11 @@ static USHORT update_associated_mac_entry(
 	if (IS_NO_SECURITY(&pEntry->SecConfig) || IS_CIPHER_WEP(pEntry->SecConfig.PairwiseCipher))
 		ApLogEvent(pAd, pEntry->Addr, EVENT_ASSOCIATED);
 
+	if ((pEntry->MaxHTPhyMode.field.MODE == MODE_OFDM) ||
+		(pEntry->MaxHTPhyMode.field.MODE == MODE_CCK)) {
+		pAd->MacTab.fAnyStationIsLegacy = TRUE;
+	}
+
 	APUpdateCapabilityAndErpIe(pAd);
 #ifdef DOT11_N_SUPPORT
 	APUpdateOperationMode(pAd, wdev);
@@ -1219,7 +1225,8 @@ VOID ap_cmm_peer_assoc_req_action(
     IN MLME_QUEUE_ELEM *Elem,
 	IN BOOLEAN isReassoc)
 {
-	struct wifi_dev *wdev;
+	struct wifi_dev *wdev;	
+	struct dev_rate_info *rate;
 	BSS_STRUCT *pMbss;
 	BOOLEAN bAssocSkip = FALSE;
 	CHAR rssi;
@@ -1234,6 +1241,9 @@ VOID ap_cmm_peer_assoc_req_action(
 	UCHAR MaxSupportedRate = 0;
 	UCHAR SupRateLen, PhyMode, FlgIs11bSta;
 	UCHAR i;
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)	
+	UCHAR ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
 	MAC_TABLE_ENTRY *pEntry = NULL;
 	STA_TR_ENTRY *tr_entry;
 #ifdef DBG
@@ -1303,6 +1313,7 @@ VOID ap_cmm_peer_assoc_req_action(
 #endif /* WSC_AP_SUPPORT */
 
 	PhyMode = wdev->PhyMode;
+	rate = &wdev->rate;
 
 	FlgIs11bSta = 1;
 	for(i=0; i<ie_list->SupportedRatesLen; i++)
@@ -1552,7 +1563,7 @@ SendAssocResponse:
 #endif /* WSC_AP_SUPPORT */
 
 	/* fail in ACL checking => send an Assoc-Fail resp. */
-	SupRateLen = pAd->CommonCfg.SupRateLen;
+	SupRateLen = rate->SupRateLen;
 
 	/* TODO: need to check rate in support rate element, not number */
 	if (FlgIs11bSta == 1)
@@ -1587,7 +1598,7 @@ SendAssocResponse:
 			              2,                        &Aid,
 			              1,                        &SupRateIe,
 			              1,                        &SupRateLen,
-			              SupRateLen, pAd->CommonCfg.SupRate,
+			              SupRateLen, rate->SupRate,
 			              END_OF_ARGS);
 		MiniportMMRequest(pAd, 0, pOutBuffer, FrameLen);
 		MlmeFreeMemory( (PVOID) pOutBuffer);
@@ -1620,16 +1631,16 @@ SendAssocResponse:
 	                  2,                        &Aid,
 	                  1,                        &SupRateIe,
 	                  1,                        &SupRateLen,
-	                  SupRateLen, pAd->CommonCfg.SupRate,
+	                  SupRateLen, 				rate->SupRate,
 	                  END_OF_ARGS);
 
-	if ((pAd->CommonCfg.ExtRateLen) && (PhyMode != WMODE_B) && (FlgIs11bSta == 0))
+	if ((rate->ExtRateLen) && (PhyMode != WMODE_B) && (FlgIs11bSta == 0))
 	{
 		ULONG TmpLen;
 		MakeOutgoingFrame(pOutBuffer+FrameLen,      &TmpLen,
 		                  1,                        &ExtRateIe,
-		                  1,                        &pAd->CommonCfg.ExtRateLen,
-		                  pAd->CommonCfg.ExtRateLen,    pAd->CommonCfg.ExtRate,
+		                  1,                        &rate->ExtRateLen,
+		                  rate->ExtRateLen,    		rate->ExtRate,
 		                  END_OF_ARGS);
 		FrameLen += TmpLen;
 	}
@@ -1703,6 +1714,20 @@ SendAssocResponse:
 
 		NdisMoveMemory(&HtCapabilityRsp, &pAd->CommonCfg.HtCapability, ie_list->ht_cap_len);
 
+#ifdef TXBF_SUPPORT
+        if (HcIsBfCapSupport(wdev) == FALSE)
+        {
+            UCHAR ucEBfCap;
+
+            ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+            pAd->CommonCfg.ETxBfEnCond = 0;
+            
+            mt_WrapSetETxBFCap(pAd, &HtCapabilityRsp.TxBFCap);
+
+            pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+        }
+#endif /* TXBF_SUPPORT */       
+
 		/* add HT Capability IE */
 #ifndef RT_BIG_ENDIAN
 		MakeOutgoingFrame(pOutBuffer+FrameLen,			&TmpLen,
@@ -1821,8 +1846,20 @@ SendAssocResponse:
         vht_ie_info.frame_subtype = SUBTYPE_ASSOC_RSP;
         vht_ie_info.channel = wdev->channel;
         vht_ie_info.phy_mode = wdev->PhyMode;
+        
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+        ucETxBfCap = pAd->CommonCfg.ETxBfEnCond;
+        if (HcIsBfCapSupport(wdev) == FALSE)
+        {
+            pAd->CommonCfg.ETxBfEnCond = 0;
+        }
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
 
         FrameLen += build_vht_ies(pAd, &vht_ie_info);
+
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+		pAd->CommonCfg.ETxBfEnCond = ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
 #endif /* DOT11_VHT_AC */
 	}
 #endif /* DOT11_N_SUPPORT */
@@ -2488,21 +2525,23 @@ VOID APPeerReassocReqAction(RTMP_ADAPTER *pAd, MLME_QUEUE_ELEM *Elem)
         TRUE if all parameters are OK, FALSE otherwise
     ==========================================================================
  */
-BOOLEAN PeerDisassocReqSanity(
+static BOOLEAN PeerDisassocReqSanity(
     IN PRTMP_ADAPTER pAd,
     IN VOID *Msg,
     IN ULONG MsgLen,
+    OUT PUCHAR pDA,
     OUT PUCHAR pAddr2,
     OUT	UINT16	*SeqNum,
     OUT USHORT *Reason)
 {
-    PFRAME_802_11 Fr = (PFRAME_802_11)Msg;
+	PFRAME_802_11 Fr = (PFRAME_802_11)Msg;
 
-    COPY_MAC_ADDR(pAddr2, &Fr->Hdr.Addr2);
+	COPY_MAC_ADDR(pDA, &Fr->Hdr.Addr1);
+	COPY_MAC_ADDR(pAddr2, &Fr->Hdr.Addr2);
 	*SeqNum = Fr->Hdr.Sequence;
-    NdisMoveMemory(Reason, &Fr->Octet[0], 2);
+	NdisMoveMemory(Reason, &Fr->Octet[0], 2);
 
-    return TRUE;
+	return TRUE;
 }
 
 
@@ -2516,6 +2555,7 @@ BOOLEAN PeerDisassocReqSanity(
  */
 VOID APPeerDisassocReqAction(RTMP_ADAPTER *pAd, MLME_QUEUE_ELEM *Elem)
 {
+	UCHAR Addr1[MAC_ADDR_LEN];
 	UCHAR Addr2[MAC_ADDR_LEN];
 	USHORT Reason;
 	UINT16 SeqNum;
@@ -2525,7 +2565,7 @@ VOID APPeerDisassocReqAction(RTMP_ADAPTER *pAd, MLME_QUEUE_ELEM *Elem)
 #endif /* DOT1X_SUPPORT */
 
 	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("ASSOC - 1 receive DIS-ASSOC request \n"));
-	if (! PeerDisassocReqSanity(pAd, Elem->Msg, Elem->MsgLen, Addr2, &SeqNum, &Reason))
+	if (! PeerDisassocReqSanity(pAd, Elem->Msg, Elem->MsgLen, Addr1, Addr2, &SeqNum, &Reason))
 		return;
 
 	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("ASSOC - receive DIS-ASSOC(seq-%d) request from %02x:%02x:%02x:%02x:%02x:%02x, reason=%d\n",
@@ -2538,8 +2578,19 @@ VOID APPeerDisassocReqAction(RTMP_ADAPTER *pAd, MLME_QUEUE_ELEM *Elem)
 	if (VALID_UCAST_ENTRY_WCID(pAd, Elem->Wcid))
     {
 
-#ifdef DOT1X_SUPPORT
 		wdev = &pAd->ApCfg.MBSSID[pEntry->func_tb_idx].wdev;
+		/*
+			iPhone sometimes sends disassoc frame which DA is old AP and BSSID is new AP.
+			@2016/1/26
+		*/
+		if (!MAC_ADDR_EQUAL(wdev->if_addr, Addr1)) {
+			MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
+				("ASSOC - The DA of this DIS-ASSOC request is %02x:%02x:%02x:%02x:%02x:%02x, ignore.\n", 
+				PRINT_MAC(Addr1)));
+			return;
+		}
+
+#ifdef DOT1X_SUPPORT
 		/* Notify 802.1x daemon to clear this sta info */
 		if (IS_AKM_1X_Entry(pEntry) || IS_IEEE8021X_Entry(wdev))
 			DOT1X_InternalCmdAction(pAd, pEntry, DOT1X_DISCONNECT_ENTRY);
@@ -2570,6 +2621,7 @@ VOID APPeerDisassocReqAction(RTMP_ADAPTER *pAd, MLME_QUEUE_ELEM *Elem)
 			{
 				apCliIdx = pReptEntry->MatchApCliIdx;
 				CliIdx = pReptEntry->MatchLinkIdx;
+				pReptEntry->Disconnect_Sub_Reason = APCLI_DISCONNECT_SUB_REASON_AP_PEER_DISASSOC_REQ;
 				MlmeEnqueue(pAd,
                             APCLI_CTRL_STATE_MACHINE,
                             APCLI_CTRL_DISCONNECT_REQ,
