@@ -192,7 +192,11 @@ static VOID APPeerDeauthReqAction(
 			pAd->ApCfg.aMICFailTime = pAd->ApCfg.PrevaMICFailTime;
 		}
 
-		MacTableDeleteEntry(pAd, Elem->Wcid, Addr2);
+		if (pEntry && !IS_ENTRY_CLIENT(pEntry))
+		    MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE,
+	                ("%s: receive not client de-auth ###\n", __FUNCTION__));
+		else
+		    MacTableDeleteEntry(pAd, Elem->Wcid, Addr2);
 
 		MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE,
 					("AUTH - receive DE-AUTH(seq-%d) from "
@@ -210,6 +214,7 @@ static VOID APPeerDeauthReqAction(
 			{
 				apCliIdx = pReptEntry->MatchApCliIdx;
 				CliIdx = pReptEntry->MatchLinkIdx;
+				pReptEntry->Disconnect_Sub_Reason = APCLI_DISCONNECT_SUB_REASON_AP_PEER_DEAUTH_REQ;
 				MlmeEnqueue(pAd,
 							APCLI_CTRL_STATE_MACHINE,
 							APCLI_CTRL_DISCONNECT_REQ,
@@ -541,7 +546,7 @@ SendAuth:
 
 #ifdef BAND_STEERING
 	BND_STRG_CHECK_CONNECTION_REQ(	pAd,
-										NULL, 
+										wdev, 
 										auth_info.addr2,
 										Elem->MsgType,
 										Elem->rssi_info,
