@@ -273,6 +273,8 @@ VOID APMakeBssBeacon(RTMP_ADAPTER *pAd, INT apidx)
 #ifdef DOT11K_RRM_SUPPORT
 	UINT i;
 #endif /* DOT11K_RRM_SUPPORT */
+	struct wifi_dev *wdev;
+	struct dev_rate_info *rate;
 
 	if(!BeaconTransmitRequired(pAd, apidx, pMbss))
 		return;
@@ -298,16 +300,18 @@ VOID APMakeBssBeacon(RTMP_ADAPTER *pAd, INT apidx)
 	{
 		pBeaconFrame = (UCHAR *)(tmac_info + TXWISize);
 	}
+	wdev = &pMbss->wdev;
+	PhyMode = wdev->PhyMode;
+	rate = &wdev->rate;
 
-	PhyMode = pMbss->wdev.PhyMode;
 	SsidLen = (pMbss->bHideSsid) ? 0 : pMbss->SsidLen;
 	MgtMacHeaderInit(pAd, &BcnHdr, SUBTYPE_BEACON,
 						0, BROADCAST_ADDR,
-						pMbss->wdev.if_addr,
-						pMbss->wdev.bssid);
+						wdev->if_addr,
+						wdev->bssid);
 
 	/* for update framelen to TxWI later. */
-	SupRateLen = pAd->CommonCfg.SupRateLen;
+	SupRateLen = rate->SupRateLen;
 	if (PhyMode == WMODE_B)
 		SupRateLen = 4;
 
@@ -321,19 +325,19 @@ VOID APMakeBssBeacon(RTMP_ADAPTER *pAd, INT apidx)
 					SsidLen,                      pMbss->Ssid,
 					1,                               &SupRateIe,
 					1,                               &SupRateLen,
-					SupRateLen,                pAd->CommonCfg.SupRate,
+					SupRateLen,                		 rate->SupRate,
 					1,                               &DsIe,
 					1,                               &DsLen,
-					1,                               &pMbss->wdev.channel,
+					1,                               &wdev->channel,
 					END_OF_ARGS);
 
-	if ((pAd->CommonCfg.ExtRateLen) && (PhyMode != WMODE_B))
+	if ((rate->ExtRateLen) && (PhyMode != WMODE_B))
 	{
 		ULONG TmpLen;
 		MakeOutgoingFrame(pBeaconFrame+FrameLen,         &TmpLen,
 						1,                               &ExtRateIe,
-						1,                               &pAd->CommonCfg.ExtRateLen,
-						pAd->CommonCfg.ExtRateLen,           pAd->CommonCfg.ExtRate,
+						1,                               &rate->ExtRateLen,
+						rate->ExtRateLen,           	 rate->ExtRate,
 						END_OF_ARGS);
 		FrameLen += TmpLen;
 	}
@@ -356,7 +360,7 @@ VOID APMakeBssBeacon(RTMP_ADAPTER *pAd, INT apidx)
 			BuildBeaconChList(pAd, TmpFrame, &TmpLen2);
 #else
 			{
-				UCHAR MaxTxPower = GetCuntryMaxTxPwr(pAd,PhyMode, pMbss->wdev.channel);
+				UCHAR MaxTxPower = GetCuntryMaxTxPwr(pAd,PhyMode, wdev->channel);
 				MakeOutgoingFrame(TmpFrame+TmpLen2,     &TmpLen,
 									1,                 	&pAd->ChannelList[0].Channel,
 									1,                 	&pAd->ChannelListNum,
@@ -409,7 +413,7 @@ VOID APMakeBssBeacon(RTMP_ADAPTER *pAd, INT apidx)
 	if (IS_RRM_ENABLE(pAd, apidx))
 	{
 		InsertTpcReportIE(pAd, pBeaconFrame+FrameLen, &FrameLen,
-			RTMP_GetTxPwr(pAd, pAd->CommonCfg.MlmeTransmit,pMbss->wdev.channel), 0);
+			RTMP_GetTxPwr(pAd, wdev->rate.MlmeTransmit,wdev->channel), 0);
 		RRM_InsertRRMEnCapIE(pAd, pBeaconFrame+FrameLen, &FrameLen, apidx);
 	}
 #endif /* DOT11K_RRM_SUPPORT */
@@ -518,14 +522,17 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 	UINT  i;
 	HTTRANSMIT_SETTING	BeaconTransmit = {.word = 0};   /* MGMT frame PHY rate setting when operatin at Ht rate. */
 	struct wifi_dev *wdev;
+	struct dev_rate_info *rate;
 	UCHAR tx_hw_hdr_len;
-        UINT8 TXWISize;
-        UCHAR DsLen = 1, SsidLen;
-        UCHAR PhyMode;
+    UINT8 TXWISize;
+    UCHAR DsLen = 1, SsidLen;
+    UCHAR PhyMode;
 #ifdef AP_QLOAD_SUPPORT
 	QLOAD_CTRL *pQload;
 #endif /*AP_QLOAD_SUPPORT*/
-
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+    UCHAR ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */
 
 	if (!pAd)
 		return;
@@ -536,6 +543,7 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 	pComCfg = &pAd->CommonCfg;
 	pMbss = &pAd->ApCfg.MBSSID[apidx];
 	wdev = &pMbss->wdev;
+	rate = &wdev->rate;
 
 	if (!pMbss || !pMbss->bcn_buf.BeaconPkt || !wdev)
 		return;
@@ -611,15 +619,15 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 				return;
 		}
 
-        PhyMode = pMbss->wdev.PhyMode;
+        PhyMode = wdev->PhyMode;
         SsidLen = (pMbss->bHideSsid) ? 0 : pMbss->SsidLen;
         MgtMacHeaderInit(pAd, &BcnHdr, SUBTYPE_BEACON,
                             0, BROADCAST_ADDR,
-                            pMbss->wdev.if_addr,
-                            pMbss->wdev.bssid);
+                            wdev->if_addr,
+                            wdev->bssid);
 
         /* for update framelen to TxWI later. */
-        SupRateLen = pAd->CommonCfg.SupRateLen;
+        SupRateLen = rate->SupRateLen;
         if (PhyMode == WMODE_B)
             SupRateLen = 4;
 
@@ -633,19 +641,19 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
                         SsidLen,                      pMbss->Ssid,
                         1,                               &SupRateIe,
                         1,                               &SupRateLen,
-                        SupRateLen,                pAd->CommonCfg.SupRate,
+                        SupRateLen,                		 rate->SupRate,
                         1,                               &DsIe,
                         1,                               &DsLen,
                         1,                               &wdev->channel,
                         END_OF_ARGS);
 
-        if ((pAd->CommonCfg.ExtRateLen) && (PhyMode != WMODE_B))
+        if ((rate->ExtRateLen) && (PhyMode != WMODE_B))
         {
             ULONG TmpLen;
             MakeOutgoingFrame(pBeaconFrame+FrameLen,         &TmpLen,
                             1,                               &ExtRateIe,
-                            1,                               &pAd->CommonCfg.ExtRateLen,
-                            pAd->CommonCfg.ExtRateLen,           pAd->CommonCfg.ExtRate,
+                            1,                               &rate->ExtRateLen,
+                            rate->ExtRateLen,           	rate->ExtRate,
                             END_OF_ARGS);
             FrameLen += TmpLen;
         }
@@ -721,7 +729,7 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
         if (IS_RRM_ENABLE(pAd, apidx))
         {
             InsertTpcReportIE(pAd, pBeaconFrame+FrameLen, &FrameLen,
-                RTMP_GetTxPwr(pAd, pAd->CommonCfg.MlmeTransmit,pMbss->wdev.channel), 0);
+                RTMP_GetTxPwr(pAd, wdev->rate.MlmeTransmit,wdev->channel), 0);
             RRM_InsertRRMEnCapIE(pAd, pBeaconFrame+FrameLen, &FrameLen, apidx);
         }
 #endif /* DOT11K_RRM_SUPPORT */
@@ -893,7 +901,7 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 		extern UCHAR		OSEN_IE[];
 		extern UCHAR		OSEN_IELEN;
 
-		if ((pMbss->HotSpotCtrl.HotSpotEnable == 0) && (pMbss->HotSpotCtrl.bASANEnable == 1) && (pMbss->wdev.AuthMode == Ndis802_11AuthModeWPA2))
+		if ((pMbss->HotSpotCtrl.HotSpotEnable == 0) && (pMbss->HotSpotCtrl.bASANEnable == 1) && (wdev->AuthMode == Ndis802_11AuthModeWPA2))
 		{
 			RSNIe = IE_WPA;
 			MakeOutgoingFrame(pBeaconFrame+FrameLen,        &TmpLen,
@@ -938,9 +946,9 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 #ifdef WSC_AP_SUPPORT
     if ((pMbss->WscControl.WscConfMode != WSC_DISABLE) &&
 #ifdef DOT1X_SUPPORT
-        (!IS_IEEE8021X_Entry(&pMbss->wdev)) &&
+        (!IS_IEEE8021X_Entry(wdev)) &&
 #endif /* DOT1X_SUPPORT */
-        (pMbss->wdev.WepStatus == Ndis802_11WEPEnabled))
+        (wdev->WepStatus == Ndis802_11WEPEnabled))
     {
         ULONG TempLen = 0;
         UCHAR PROVISION_SERVICE_IE[7] = {0xDD, 0x05, 0x00, 0x50, 0xF2, 0x05, 0x00};
@@ -953,7 +961,7 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 
 
 	/* Update ERP */
-    if ((pComCfg->ExtRateLen) && (PhyMode != WMODE_B))
+    if ((rate->ExtRateLen) && (PhyMode != WMODE_B))
     {
         /* fill ERP IE */
         ptr = (UCHAR *)pBeaconFrame + FrameLen; /* pTxD->DataByteCnt; */
@@ -1064,6 +1072,20 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 		NdisMoveMemory(&HtCapabilityTmp, &pComCfg->HtCapability, HtLen);
 		HtCapabilityTmp.HtCapInfo.ChannelWidth = pComCfg->AddHTInfo.AddHtInfo.RecomWidth;
 
+#ifdef TXBF_SUPPORT
+        if (HcIsBfCapSupport(wdev) == FALSE)
+        {
+            UCHAR ucEBfCap;
+
+            ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+            pAd->CommonCfg.ETxBfEnCond = 0;
+            
+            mt_WrapSetETxBFCap(pAd, &HtCapabilityTmp.TxBFCap);
+
+            pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+        }
+#endif /* TXBF_SUPPORT */		
+
 		MakeOutgoingFrame(pBeaconFrame+FrameLen,         &TmpLen,
 								  1,                                &HtCapIe,
 								  1,                                &HtLen,
@@ -1075,6 +1097,21 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 #else
 		NdisMoveMemory(&HtCapabilityTmp, &pComCfg->HtCapability, HtLen);
 		HtCapabilityTmp.HtCapInfo.ChannelWidth = pComCfg->AddHTInfo.AddHtInfo.RecomWidth;
+
+#ifdef TXBF_SUPPORT
+        if (HcIsBfCapSupport(wdev) == FALSE)
+        {
+            UCHAR ucEBfCap;
+
+            ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+            pAd->CommonCfg.ETxBfEnCond = 0;
+            
+            mt_WrapSetETxBFCap(pAd, &HtCapabilityTmp.TxBFCap);
+
+            pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+        }
+#endif /* TXBF_SUPPORT */
+
 		*(USHORT *)(&HtCapabilityTmp.HtCapInfo) = SWAP16(*(USHORT *)(&HtCapabilityTmp.HtCapInfo));
 #ifdef UNALIGNMENT_SUPPORT
 		{
@@ -1188,7 +1225,19 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
         vht_ie_info.channel = wdev->channel;
         vht_ie_info.phy_mode = PhyMode;
 
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+        ucETxBfCap = pAd->CommonCfg.ETxBfEnCond;
+        if (HcIsBfCapSupport(wdev) == FALSE)
+        {
+            pAd->CommonCfg.ETxBfEnCond = 0;
+        }
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
+
         FrameLen += build_vht_ies(pAd, &vht_ie_info);
+
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+		pAd->CommonCfg.ETxBfEnCond = ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
 #endif /* DOT11_VHT_AC */
 	}
 #endif /* DOT11_N_SUPPORT */
@@ -1209,7 +1258,7 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 #ifdef DOT11N_DRAFT3
 		/* P802.11n_D1.10, HT Information Exchange Support */
 		if (WMODE_CAP_N(PhyMode) && (wdev->channel <= 14) &&
-			(pMbss->wdev.DesiredHtPhyInfo.bHtEnable) &&
+			(wdev->DesiredHtPhyInfo.bHtEnable) &&
 			(pComCfg->bBssCoexEnable == TRUE)
 		)
 		{
@@ -1325,7 +1374,7 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 #endif /* WFA_VHT_PF */
 
 	/* add WMM IE here */
-	if (pMbss->wdev.bWmmCapable)
+	if (wdev->bWmmCapable)
 	{
 		ULONG TmpLen;
 		UCHAR i;
@@ -1335,7 +1384,7 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 		WmeParmIe[8] = pAd->ApCfg.BssEdcaParm.EdcaUpdateCount & 0x0f;
 
 #ifdef UAPSD_SUPPORT
-        UAPSD_MR_IE_FILL(WmeParmIe[8], &pMbss->wdev.UapsdInfo);
+        UAPSD_MR_IE_FILL(WmeParmIe[8], &wdev->UapsdInfo);
 #endif /* UAPSD_SUPPORT */
 
 		NdisMoveMemory(AIFSN, pAd->ApCfg.BssEdcaParm.Aifsn, sizeof(AIFSN));
@@ -1465,6 +1514,21 @@ VOID APUpdateBeaconFrame(RTMP_ADAPTER *pAd, INT apidx)
 						  END_OF_ARGS);
 #else
 			NdisMoveMemory(&HtCapabilityTmp, &pComCfg->HtCapability, HtLen);
+
+#ifdef TXBF_SUPPORT
+            if (HcIsBfCapSupport(wdev) == FALSE)
+            {
+                UCHAR ucEBfCap;
+
+                ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+                pAd->CommonCfg.ETxBfEnCond = 0;
+            
+                mt_WrapSetETxBFCap(pAd, &HtCapabilityTmp.TxBFCap);
+
+                pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+            }
+#endif /* TXBF_SUPPORT */
+
 			*(USHORT *)(&HtCapabilityTmp.HtCapInfo) = SWAP16(*(USHORT *)(&HtCapabilityTmp.HtCapInfo));
 #ifdef UNALIGNMENT_SUPPORT
 		{
@@ -1645,6 +1709,7 @@ VOID APUpdateAllBeaconFrame(RTMP_ADAPTER *pAd)
 	BOOLEAN FlgQloadIsAlarmIssued = FALSE;
 #endif /* DOT11N_DRAFT3 */
 #endif /* DOT11_N_SUPPORT */
+	struct wifi_dev *wdev = &pAd->ApCfg.MBSSID[0].wdev;
 
 	if (pAd->ApCfg.DtimCount == 0)
 		pAd->ApCfg.DtimCount = pAd->ApCfg.DtimPeriod - 1;
@@ -1683,7 +1748,7 @@ VOID APUpdateAllBeaconFrame(RTMP_ADAPTER *pAd)
 		else
 		{
 			pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth = pAd->CommonCfg.RegTransmitSetting.field.BW;
-			pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = pAd->CommonCfg.RegTransmitSetting.field.EXTCHA;
+			pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = wdev->extcha;
 		}
 		MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE,
 				("\tNow RecomWidth=%d, ExtChanOffset=%d, prevBW=%d, prevExtOffset=%d\n",
