@@ -83,6 +83,7 @@ VOID APDetectOverlappingExec(
 #ifdef DOT11_N_SUPPORT
 	PRTMP_ADAPTER	pAd = (RTMP_ADAPTER *)FunctionContext;
 	BOOLEAN bSupport2G = HcIsRfSupport(pAd,RFIC_24GHZ);
+	UCHAR Channel = HcGetChannelByRf(pAd,RFIC_24GHZ);
 
 	if (DetectOverlappingPeriodicRound == 0)
 	{
@@ -90,7 +91,7 @@ VOID APDetectOverlappingExec(
 		if ( bSupport2G && (pAd->CommonCfg.HtCapability.HtCapInfo.ChannelWidth == BW_40))
 		{
 			pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth = 1;
-			pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = pAd->CommonCfg.RegTransmitSetting.field.EXTCHA;
+			pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = HcGetExtCha(pAd,Channel);
 		}
 	}
 	else
@@ -110,7 +111,6 @@ VOID APDetectOverlappingExec(
 #endif /* DOT11_N_SUPPORT */
 }
 
-
 /*
     ==========================================================================
     Description:
