@@ -231,14 +231,32 @@ VOID CliLinkMapInit(RTMP_ADAPTER *pAd)
     MBSS_TO_CLI_LINK_MAP_T  *pMbssToCliLinkMap;
     struct wifi_dev *cli_link_wdev = &pAd->ApCfg.ApCliTab[0].wdev;//default bind to apcli0
     struct wifi_dev *mbss_link_wdev;
+	int		apcli_idx;
 
     NdisAcquireSpinLock(&pAd->ApCfg.CliLinkMapLock);
     for (MbssIdx = 0; MbssIdx < HW_BEACON_MAX_NUM; MbssIdx++)
     {
         mbss_link_wdev = &pAd->ApCfg.MBSSID[MbssIdx].wdev;
         pMbssToCliLinkMap = &pAd->ApCfg.MbssToCliLinkMap[MbssIdx];
-        pMbssToCliLinkMap->mbss_wdev = mbss_link_wdev;
-        pMbssToCliLinkMap->cli_link_wdev = cli_link_wdev;
+		if (pAd->CommonCfg.dbdc_mode == TRUE) {
+			for (apcli_idx=0; apcli_idx < pAd->ApCfg.ApCliNum; apcli_idx++) {
+				cli_link_wdev = &pAd->ApCfg.ApCliTab[apcli_idx].wdev;
+				if (mbss_link_wdev->channel <= 14) { //2.4G
+					if (cli_link_wdev->channel <= 14 ) { //2.4G
+						pMbssToCliLinkMap->mbss_wdev = mbss_link_wdev;
+						pMbssToCliLinkMap->cli_link_wdev = cli_link_wdev;
+					}
+				} else { //5G
+					if (cli_link_wdev->channel > 14 ) { //5G
+						pMbssToCliLinkMap->mbss_wdev = mbss_link_wdev;
+						pMbssToCliLinkMap->cli_link_wdev = cli_link_wdev;
+					}
+				}
+			}
+		} else {
+			pMbssToCliLinkMap->mbss_wdev = mbss_link_wdev;
+			pMbssToCliLinkMap->cli_link_wdev = cli_link_wdev;
+		}
     }
     NdisReleaseSpinLock(&pAd->ApCfg.CliLinkMapLock);
 }
@@ -468,10 +486,6 @@ BOOLEAN RTMPQueryLookupRepeaterCliEntryMT(
 		return TRUE;
 	}
 }
-#if defined (CONFIG_WIFI_PKT_FWD)
-EXPORT_SYMBOL(RTMPQueryLookupRepeaterCliEntryMT);
-#endif /* CONFIG_WIFI_PKT_FWD */
-
 
 UINT32 ReptTxPktCheckHandler(
     RTMP_ADAPTER *pAd,
@@ -625,6 +639,8 @@ VOID RTMPInsertRepeaterEntry(
 	pReptCliEntry->AuthCurrState = APCLI_AUTH_REQ_IDLE;
 	pReptCliEntry->AssocCurrState = APCLI_ASSOC_IDLE;
 	pReptCliEntry->CliConnectState = REPT_ENTRY_DISCONNT;
+	pReptCliEntry->LinkDownReason = APCLI_LINKDOWN_NONE;
+	pReptCliEntry->Disconnect_Sub_Reason = APCLI_DISCONNECT_SUB_REASON_NONE;
 	pReptCliEntry->CliValid = FALSE;
 	pReptCliEntry->bEthCli = FALSE;
 	pReptCliEntry->MacTabWCID = 0xFF;
@@ -1148,6 +1164,7 @@ VOID InsertIgnoreAsRepeaterEntryTable(
 		{
 			NdisZeroMemory(pEntry->MacAddr, MAC_ADDR_LEN);
 			COPY_MAC_ADDR(pEntry->MacAddr, pAddr);
+			pEntry->entry_idx = idx;
 			pEntry->bInsert = TRUE;
 			break;
 		}
@@ -1270,6 +1287,7 @@ VOID RepeaterLinkMonitor(RTMP_ADAPTER *pAd)
 					} else {
 						if (!VALID_UCAST_ENTRY_WCID(pAd, Wcid))
 							continue;
+						pReptCliEntry->Disconnect_Sub_Reason = APCLI_DISCONNECT_SUB_REASON_REPTLM_TRIGGER_TOO_LONG;
 						NdisZeroMemory(&msg, sizeof(APCLI_CTRL_MSG_STRUCT));
 						msg.BssIdx = pReptCliEntry->MatchApCliIdx;
 						msg.CliIdx = CliIdx;
