@@ -136,6 +136,7 @@ INT APKeyTableInit (
     struct _SECURITY_CONFIG *pSecConfig = NULL;
     ASIC_SEC_INFO Info = {0};
     USHORT Wcid = 0;
+    STA_TR_ENTRY *tr_entry = NULL;
 
     if (wdev == NULL)
         return 0;
@@ -150,13 +151,16 @@ INT APKeyTableInit (
     pSecConfig = &wdev->SecConfig;
 
     RTMPInitTimer(pAd, &pSecConfig->GroupRekeyTimer, GET_TIMER_FUNCTION(GroupRekeyExec), pAd,  TRUE);
-    wdev->PortSecured = WPA_802_1X_PORT_NOT_SECURED;
     if (IS_AKM_WPA_CAPABILITY(pSecConfig->AKMMap))
         pSecConfig->GroupKeyId = 1;
 
     /* Get a specific WCID to record this MBSS key attribute */
     GET_GroupKey_WCID(wdev, Wcid);
 
+    tr_entry = &pAd->MacTab.tr_entry[Wcid];
+    wdev->PortSecured = WPA_802_1X_PORT_NOT_SECURED;
+    tr_entry->PortSecured = WPA_802_1X_PORT_NOT_SECURED;
+
     /* Set key material to Asic */
     os_zero_mem(&Info, sizeof(ASIC_SEC_INFO));
     Info.Operation = SEC_ASIC_ADD_GROUP_KEY;
@@ -406,7 +410,7 @@ VOID WPAGroupRekeyAction (
     }
 }
 
-
+static PCHAR portsecured[]={"NONE", "PORT_SECURED", "NOT_SECURED"};
 INT Show_APSecurityInfo_Proc (
     IN RTMP_ADAPTER *pAd, 
     IN RTMP_STRING *arg)
@@ -415,52 +419,58 @@ INT Show_APSecurityInfo_Proc (
 	USHORT Wcid;
 	struct _SECURITY_CONFIG *pSecConfig = NULL;
     struct wifi_dev *wdev = NULL;
+    	STA_TR_ENTRY *tr_entry = NULL;
 
 	if (!pAd)
 		return FALSE; 
 
 	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("Security Infomation: AP\n"));
-	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("BSS\tWCID\tAuthMode\tPairwiseCipher\tGroupCipher\tGroupKeyId\n"));
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("BSS\tWCID\tAuthMode\tPairwiseCipher\tGroupCipher\tGroupKeyId\tPortSecured\n"));
 
 	for (idx = 0; idx < pAd->ApCfg.BssidNum; idx++)
 	{
 		pSecConfig = &pAd->ApCfg.MBSSID[idx].wdev.SecConfig;
-        wdev = &pAd->ApCfg.MBSSID[idx].wdev;
+		wdev = &pAd->ApCfg.MBSSID[idx].wdev;
 		GET_GroupKey_WCID(wdev, Wcid);
-
-		MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, (" %d\t%d\t%s\t\t%s\t\t%s\t\t%d\n",
+		tr_entry = &pAd->MacTab.tr_entry[Wcid];
+		
+		MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, (" %d\t%d\t%s\t\t%s\t\t%s\t\t%d\t\t%s\n",
 							idx,
 							Wcid,
 							GetAuthModeStr(GET_SEC_AKM(pSecConfig)),
 							GetEncryModeStr(GET_PAIRWISE_CIPHER(pSecConfig)),
 							GetEncryModeStr(GET_GROUP_CIPHER(pSecConfig)),
-							pSecConfig->GroupKeyId));
+							pSecConfig->GroupKeyId,
+							portsecured[tr_entry->PortSecured]));
+
 	}
 	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
 
 #ifdef APCLI_SUPPORT
 	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("Security Infomation: AP Client\n"));
-	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("BSS\tWCID\tAuthMode\tPairwiseCipher\n"));
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("BSS\tWCID\tAuthMode\tPairwiseCipher\tPortSecured\n"));
 
 	for (idx = 0; idx < MAX_APCLI_NUM; idx++)
 	{
 		PAPCLI_STRUCT  pApCliEntry = &pAd->ApCfg.ApCliTab[idx];
 		pSecConfig = &pApCliEntry->wdev.SecConfig;
 		wdev = &pApCliEntry->wdev;
+		tr_entry = &pAd->MacTab.tr_entry[wdev->bss_info_argument.ucBcMcWlanIdx];
 		if (pApCliEntry->Enable == TRUE)
 		{
-			MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, (" %d\t%d\t%s\t\t%s\n",
+			MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, (" %d\t%d\t%s\t\t%s\t\t%s\n",
 						idx,
 						wdev->bss_info_argument.ucBcMcWlanIdx,
 						GetAuthModeStr(GET_SEC_AKM(pSecConfig)),
-						GetEncryModeStr(GET_PAIRWISE_CIPHER(pSecConfig))));
+						GetEncryModeStr(GET_PAIRWISE_CIPHER(pSecConfig)),
+						portsecured[tr_entry->PortSecured]));
 		}
 	}
 	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
 #endif
 
 	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("Security Infomation: STA\n"));
-	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("BSS\t\tAID\tWCID\tAuthMode\tPairwiseCipher\n"));
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("BSS\t\t\tAID\tWCID\tAuthMode\tPairwiseCipher\tPortSecured\n"));
 
 	for (idx=0; idx<MAX_LEN_OF_MAC_TABLE; idx++)
 	{
@@ -468,18 +478,165 @@ INT Show_APSecurityInfo_Proc (
 		pSecConfig = &pEntry->SecConfig;
 		if (pEntry && IS_ENTRY_CLIENT(pEntry) && pEntry->Sst == SST_ASSOC)
 		{
-			MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%02X:%02X:%02X:%02X:%02X:%02X\t%d\t%d\t\t%s\t\t%s\n",
+			tr_entry = &pAd->MacTab.tr_entry[pEntry->tr_tb_idx];
+			MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%02X:%02X:%02X:%02X:%02X:%02X\t%d\t%d\t%s\t\t%s\t\t%s\n",
 						PRINT_MAC(pEntry->Addr),
 						pEntry->Aid,
 						pEntry->wcid,
 						GetAuthModeStr(GET_SEC_AKM(pSecConfig)),
-						GetEncryModeStr(GET_PAIRWISE_CIPHER(pSecConfig))));
+						GetEncryModeStr(GET_PAIRWISE_CIPHER(pSecConfig)),
+						portsecured[tr_entry->PortSecured]));
+			if ((!IS_AKM_OPEN(GET_SEC_AKM(pSecConfig))) && tr_entry->PortSecured == 1)
+			{
+				INT32 i,key_len=32;
+				if (IS_CIPHER_TKIP(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_TKIP_TK;
+				}
+				else if (IS_CIPHER_CCMP128(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_CCMP128_TK;
+				}
+				else if (IS_CIPHER_CCMP256(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_CCMP256_TK;
+				}
+				else if (IS_CIPHER_GCMP128(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_GCMP128_TK;
+				}
+				else if (IS_CIPHER_GCMP256(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_GCMP256_TK;
+				}
+				MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("  PTK:"));
+#define RED(_text)  "\033[1;31m"_text"\033[0m"
+				for (i=0;i<64;i++) {
+					if ((i>=32) && (i<(key_len+32))) {
+						MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, (RED("%02x"),pSecConfig->PTK[i]));
+					} else {
+					MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%02x",pSecConfig->PTK[i]));
+					}
+				}
+				MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			}
 		}
 	}
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+
+#ifdef APCLI_SUPPORT
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("Security Infomation: ApCli/Repeater\n"));
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("BSS\t\t\tAID\tWCID\tAuthMode\tPairwiseCipher\tPortSecured\n"));
+	
+	for (idx=0; idx<MAX_LEN_OF_MAC_TABLE; idx++)
+	{
+		PMAC_TABLE_ENTRY pEntry = &pAd->MacTab.Content[idx];
+		pSecConfig = &pEntry->SecConfig;
+		if (pEntry && (IS_ENTRY_REPEATER(pEntry) || IS_ENTRY_APCLI(pEntry)) && pEntry->Sst == SST_ASSOC)
+		{
+			tr_entry = &pAd->MacTab.tr_entry[pEntry->tr_tb_idx];
+			MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%02X:%02X:%02X:%02X:%02X:%02X\t%d\t%d\t%s\t\t%s\t\t%s\n",
+					 PRINT_MAC(pEntry->Addr),
+					 pEntry->Aid,
+					 pEntry->wcid,
+					 GetAuthModeStr(GET_SEC_AKM(pSecConfig)),
+					 GetEncryModeStr(GET_PAIRWISE_CIPHER(pSecConfig)),
+					 portsecured[tr_entry->PortSecured]));
+			if ((!IS_AKM_OPEN(GET_SEC_AKM(pSecConfig))) && tr_entry->PortSecured == 1)
+			{
+				INT32 i,key_len=32;
+				if (IS_CIPHER_TKIP(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_TKIP_TK;
+				}
+				else if (IS_CIPHER_CCMP128(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_CCMP128_TK;
+				}
+				else if (IS_CIPHER_CCMP256(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_CCMP256_TK;
+				}
+				else if (IS_CIPHER_GCMP128(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_GCMP128_TK;
+				}
+				else if (IS_CIPHER_GCMP256(pSecConfig->PairwiseCipher))
+				{
+					key_len = LEN_GCMP256_TK;
+				}
+				MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("  PTK:"));
+#define RED(_text)  "\033[1;31m"_text"\033[0m"
+				for (i=0;i<64;i++) {
+					if ((i>=32) && (i<(key_len+32))) {
+						MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, (RED("%02x"),pSecConfig->PTK[i]));
+					} else {
+						MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%02x",pSecConfig->PTK[i]));
+					}
+				}
+				MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			}
+		}
+	}
+#endif
 
 	return TRUE;
 }
 
+VOID CheckBMCPortSecured(
+	IN RTMP_ADAPTER *pAd,
+	IN MAC_TABLE_ENTRY *pEntry,
+	IN BOOLEAN isConnect)
+{
+	UINT32 bss_index = pEntry->func_tb_idx;
+	UINT32 wcid;
+	UCHAR PortSecured = WPA_802_1X_PORT_NOT_SECURED;
+	MAC_TABLE_ENTRY *pMacEntry = NULL;
+	STA_TR_ENTRY *tr_entry = NULL;
+	struct wifi_dev *wdev = &pAd->ApCfg.MBSSID[bss_index].wdev;
+	
+	if (wdev->tr_tb_idx == 0xff)
+		return;//skip uninit tr_tb_idx.
+	if (isConnect) {
+		PortSecured = WPA_802_1X_PORT_SECURED;
+	}
+	else {
+		RETURN_IF_PAD_NULL(pAd);
+		for (wcid = 1; VALID_UCAST_ENTRY_WCID(pAd, wcid); wcid++) {
+			pMacEntry = &pAd->MacTab.Content[wcid];
+			tr_entry = &pAd->MacTab.tr_entry[wcid];
+
+			/*if (wcid == pEntry->wcid)
+				continue;*/
+
+			if (((pMacEntry)
+				&& (IS_ENTRY_CLIENT(pMacEntry))
+				&& (pMacEntry->Sst == SST_ASSOC)
+				&& (pMacEntry->func_tb_idx == bss_index)
+				&& (tr_entry->PortSecured == WPA_802_1X_PORT_SECURED))
+#ifdef CONFIG_FPGA_MODE
+					|| (pAd->fpga_ctl.fpga_on & 0x1)
+#endif /* CONFIG_FPGA_MODE */
+				) {
+				PortSecured = WPA_802_1X_PORT_SECURED;
+				break;
+			}
+		}
+	}
+	if (wdev->PortSecured != PortSecured) {
+		tr_entry = &pAd->MacTab.tr_entry[wdev->bss_info_argument.ucBcMcWlanIdx];
+		tr_entry->PortSecured = PortSecured;
+		wdev->PortSecured = PortSecured;
+		MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
+				("%s: bss_index = %d, wcid = %d, PortSecured = %d\n",
+					__FUNCTION__,
+					bss_index,
+					wdev->bss_info_argument.ucBcMcWlanIdx,
+					PortSecured)); 
+		
+	}
+}
+
 #ifdef DOT1X_SUPPORT
 /*
     ========================================================================
