@@ -54,10 +54,14 @@ VOID APPeerProbeReqAction(
 	UCHAR apidx = 0, PhyMode, SupRateLen;
 	BSS_STRUCT *mbss;
 	struct wifi_dev *wdev;
+	struct dev_rate_info *rate;
 #ifdef BAND_STEERING
 	BOOLEAN bBndStrgCheck = TRUE;
 	BOOLEAN bAllowStaConnectInHt = FALSE;
 #endif /* BAND_STEERING */
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+    UCHAR ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */
 	QLOAD_CTRL *pQloadCtrl = HcGetQloadCtrl(pAd);
 	CHAR rsne_idx = 0;
 	struct _SECURITY_CONFIG *pSecConfig = NULL;
@@ -86,6 +90,7 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 	{
 		mbss = &pAd->ApCfg.MBSSID[apidx];
 		wdev = &mbss->wdev;
+		rate = &wdev->rate;
 
 		if ((wdev->if_dev == NULL) || ((wdev->if_dev != NULL) &&
 			!(RTMP_OS_NETDEV_STATE_RUNNING(wdev->if_dev))))
@@ -93,6 +98,9 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 			/* the interface is down, so we can not send probe response */
 			continue;
 		}
+		if (Elem->Channel != wdev->channel) {
+			continue;
+		}
 
 		PhyMode = wdev->PhyMode;
 
@@ -119,7 +127,7 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 		if (ProbeReqParam.IsHtSupport && WMODE_CAP_N(wdev->PhyMode))
 			bAllowStaConnectInHt = TRUE;
 		BND_STRG_CHECK_CONNECTION_REQ(	pAd,
-											NULL, 
+											wdev, 
 											ProbeReqParam.Addr2,
 											Elem->MsgType,
 											Elem->rssi_info,
@@ -138,7 +146,7 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 							wdev->if_addr, wdev->bssid);
 
 		{
-		SupRateLen = pAd->CommonCfg.SupRateLen;
+		SupRateLen = rate->SupRateLen;
 		if (PhyMode == WMODE_B)
 			SupRateLen = 4;
 
@@ -152,22 +160,22 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 						  mbss->SsidLen,     mbss->Ssid,
 						  1,                          &SupRateIe,
 						  1,                          &SupRateLen,
-						  SupRateLen,                 pAd->CommonCfg.SupRate,
+						  SupRateLen,                 rate->SupRate,
 						  1,                          &DsIe,
 						  1,                          &DsLen,
 						  1,                          &wdev->channel,
 						  END_OF_ARGS);
 		}
 
-		if ((pAd->CommonCfg.ExtRateLen) && (PhyMode != WMODE_B))
+		if ((rate->ExtRateLen) && (PhyMode != WMODE_B))
 		{
 			MakeOutgoingFrame(pOutBuffer+FrameLen,      &TmpLen,
 							  1,                        &ErpIe,
 							  1,                        &ErpIeLen,
 							  1,                        &pAd->ApCfg.ErpIeContent,
 							  1,                        &ExtRateIe,
-							  1,                        &pAd->CommonCfg.ExtRateLen,
-							  pAd->CommonCfg.ExtRateLen,    pAd->CommonCfg.ExtRate,
+							  1,                        &rate->ExtRateLen,
+							  rate->ExtRateLen,    		rate->ExtRate,
 							  END_OF_ARGS);
 			FrameLen += TmpLen;
 		}
@@ -191,6 +199,21 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 			/*New extension channel offset IE is included in Beacon, Probe Rsp or channel Switch Announcement Frame */
 #ifndef RT_BIG_ENDIAN
 			NdisMoveMemory(&HtCapabilityTmp, &pAd->CommonCfg.HtCapability, HtLen);
+
+#ifdef TXBF_SUPPORT
+            if (HcIsBfCapSupport(wdev) == FALSE)
+            {
+                UCHAR ucEBfCap;
+
+                ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+                pAd->CommonCfg.ETxBfEnCond = 0;
+            
+                mt_WrapSetETxBFCap(pAd, &HtCapabilityTmp.TxBFCap);
+
+                pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+            }
+#endif /* TXBF_SUPPORT */ 
+
 			HtCapabilityTmp.HtCapInfo.ChannelWidth = pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth;
 
 			MakeOutgoingFrame(pOutBuffer + FrameLen,            &TmpLen,
@@ -203,6 +226,21 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 							  END_OF_ARGS);
 #else
 			NdisMoveMemory(&HtCapabilityTmp, &pAd->CommonCfg.HtCapability, HtLen);
+
+#ifdef TXBF_SUPPORT
+            if (HcIsBfCapSupport(wdev) == FALSE)
+            {
+                UCHAR ucEBfCap;
+
+                ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+                pAd->CommonCfg.ETxBfEnCond = 0;
+            
+                mt_WrapSetETxBFCap(pAd, &HtCapabilityTmp.TxBFCap);
+
+                pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+            }
+#endif /* TXBF_SUPPORT */ 
+
 			HtCapabilityTmp.HtCapInfo.ChannelWidth = pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth;
 			*(USHORT *)(&HtCapabilityTmp.HtCapInfo) = SWAP16(*(USHORT *)(&HtCapabilityTmp.HtCapInfo));
 #ifdef UNALIGNMENT_SUPPORT
@@ -388,13 +426,13 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 #endif /* AP_QLOAD_SUPPORT */
 
 		/* add WMM IE here */
-		if (mbss->wdev.bWmmCapable)
+		if (wdev->bWmmCapable)
 		{
 			UCHAR i;
 			UCHAR WmeParmIe[26] = {IE_VENDOR_SPECIFIC, 24, 0x00, 0x50, 0xf2, 0x02, 0x01, 0x01, 0, 0};
 			WmeParmIe[8] = pAd->ApCfg.BssEdcaParm.EdcaUpdateCount & 0x0f;
 #ifdef UAPSD_SUPPORT
-			UAPSD_MR_IE_FILL(WmeParmIe[8], &mbss->wdev.UapsdInfo);
+			UAPSD_MR_IE_FILL(WmeParmIe[8], &wdev->UapsdInfo);
 #endif /* UAPSD_SUPPORT */
 			for (i=QID_AC_BE; i<=QID_AC_VO; i++)
 			{
@@ -482,7 +520,7 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 		if (IS_RRM_ENABLE(pAd, apidx))
 		{
 			InsertTpcReportIE(pAd, pOutBuffer+FrameLen, &FrameLen,
-			RTMP_GetTxPwr(pAd, pAd->CommonCfg.MlmeTransmit,wdev->channel), 0);
+			RTMP_GetTxPwr(pAd, wdev->rate.MlmeTransmit,wdev->channel), 0);
 			RRM_InsertRRMEnCapIE(pAd, pOutBuffer+FrameLen, &FrameLen, apidx);
 		}
 
@@ -725,8 +763,23 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
 							  HtLen,          					&pAd->CommonCfg.HtCapability,
 							  END_OF_ARGS);
 #else
-				NdisMoveMemory(&HtCapabilityTmp, &pAd->CommonCfg.HtCapability, HtLen);
-				*(USHORT *)(&HtCapabilityTmp.HtCapInfo) = SWAP16(*(USHORT *)(&HtCapabilityTmp.HtCapInfo));
+			NdisMoveMemory(&HtCapabilityTmp, &pAd->CommonCfg.HtCapability, HtLen);
+
+#ifdef TXBF_SUPPORT
+            if (HcIsBfCapSupport(wdev) == FALSE)
+            {
+                UCHAR ucEBfCap;
+
+                ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+                pAd->CommonCfg.ETxBfEnCond = 0;
+            
+                mt_WrapSetETxBFCap(pAd, &HtCapabilityTmp.TxBFCap);
+
+                pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+            }
+#endif /* TXBF_SUPPORT */
+
+			*(USHORT *)(&HtCapabilityTmp.HtCapInfo) = SWAP16(*(USHORT *)(&HtCapabilityTmp.HtCapInfo));
 #ifdef UNALIGNMENT_SUPPORT
 			{
 				EXT_HT_CAP_INFO extHtCapInfo;
@@ -779,7 +832,19 @@ MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s():shiang! PeerProbeReqSan
             vht_ie_info.channel = wdev->channel;
             vht_ie_info.phy_mode = wdev->PhyMode;
 
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+        ucETxBfCap = pAd->CommonCfg.ETxBfEnCond;
+        if (HcIsBfCapSupport(wdev) == FALSE)
+        {
+            pAd->CommonCfg.ETxBfEnCond = 0;
+        }
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
+
             FrameLen += build_vht_ies(pAd, &vht_ie_info);
+
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+		pAd->CommonCfg.ETxBfEnCond = ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
 #endif /* DOT11_VHT_AC */
 
 		}
@@ -1063,7 +1128,7 @@ VOID APPeerBeaconAction(
 
                 SupportRate(ie_list->SupRate, ie_list->SupRateLen, ie_list->ExtRate, ie_list->ExtRateLen, &pRates, &RatesLen, &MaxSupportedRate);
 
-                if ((ie_list->Erp & 0x01) || (RatesLen <= 4))
+		if ((ie_list->Erp & 0x01) || (RatesLen <= 4))
 			LegacyBssExist = TRUE;
 		else
 			LegacyBssExist = FALSE;
@@ -1094,7 +1159,7 @@ VOID APPeerBeaconAction(
 			{
 				pAd->ApCfg.ApCliTab[pEntry->func_tb_idx].ApCliRcvBeaconTime = pAd->Mlme.Now32;
 
-				if (pAd->CommonCfg.BBPCurrentBW == BW_40)
+				if ((pEntry->wdev->bw == BW_40) && (ie_list->vht_cap_len == 0))
 				{
 					/* Check if root-ap change BW to 20 */
 					if ((ie_list->AddHtInfo.AddHtInfo.ExtChanOffset == EXTCHA_NONE) &&
@@ -1116,7 +1181,8 @@ VOID APPeerBeaconAction(
 							{
 								pReptEntry = &pAd->ApCfg.pRepeaterCliPool[CliIdx];
 
-								if ((pReptEntry->CliEnable) && (pReptEntry->CliValid))
+								if ((pReptEntry->CliEnable) && (pReptEntry->CliValid) && 
+									(pReptEntry->MatchApCliIdx == ifIndex))
 								{
 									pEntry = &pAd->MacTab.Content[pReptEntry->MacTabWCID];
 									if (pEntry)
@@ -1149,7 +1215,8 @@ VOID APPeerBeaconAction(
 							{
 								pReptEntry = &pAd->ApCfg.pRepeaterCliPool[CliIdx];
 
-								if ((pReptEntry->CliEnable) && (pReptEntry->CliValid))
+								if ((pReptEntry->CliEnable) && (pReptEntry->CliValid) &&
+									(pReptEntry->MatchApCliIdx == ifIndex))
 								{
 									pEntry = &pAd->MacTab.Content[pReptEntry->MacTabWCID];
 									if (pEntry)
