@@ -215,6 +215,10 @@ MAC_TABLE_ENTRY *MacTableInsertWDSEntry(
 	HTTRANSMIT_SETTING HTPhyMode;
 	RT_802_11_WDS_ENTRY *wds_entry;
 	struct wifi_dev *wdev;
+	struct dev_rate_info *rate;
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)	
+	UCHAR ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */
 
 	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF,("%s(): WdsTabIdx = %d, Addr %02x-%02x-%02x-%02x-%02x-%02x\n", __FUNCTION__, 
 		WdsTabIdx, PRINT_MAC(pAddr)));
@@ -228,6 +232,7 @@ MAC_TABLE_ENTRY *MacTableInsertWDSEntry(
 
 	wds_entry = &pAd->WdsTab.WdsEntry[WdsTabIdx];
 	wdev = &wds_entry->wdev;
+	rate = &wdev->rate;
 
 	/* allocate one WDS entry */
 	do
@@ -316,6 +321,20 @@ MAC_TABLE_ENTRY *MacTableInsertWDSEntry(
 				pEntry->MmpsMode = MMPS_DISABLE;
 				NdisMoveMemory(&pEntry->HTCapability, &pAd->CommonCfg.HtCapability, sizeof(HT_CAPABILITY_IE));
 
+#ifdef TXBF_SUPPORT
+                if (HcIsBfCapSupport(wdev) == FALSE)
+                {
+                    UCHAR ucEBfCap;
+
+                    ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+                    pAd->CommonCfg.ETxBfEnCond = 0;
+            
+                    mt_WrapSetETxBFCap(pAd, &pEntry->HTCapability.TxBFCap);
+
+                    pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+                }
+#endif /* TXBF_SUPPORT */				
+
 				set_sta_ht_cap(pAd, pEntry, &pEntry->HTCapability);
 
 				CLIENT_STATUS_SET_FLAG(pEntry, fCLIENT_STATUS_WMM_CAPABLE);
@@ -324,8 +343,20 @@ MAC_TABLE_ENTRY *MacTableInsertWDSEntry(
 				{
 					VHT_CAP_IE vht_cap;
 
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+                    ucETxBfCap = pAd->CommonCfg.ETxBfEnCond;
+                    if (HcIsBfCapSupport(wdev) == FALSE)
+                    {
+                        pAd->CommonCfg.ETxBfEnCond = 0;
+                    }
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */                   
+                    
 					build_vht_cap_ie(pAd, (UCHAR *)&vht_cap);
 
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+					pAd->CommonCfg.ETxBfEnCond = ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
+
 					vht_mode_adjust(pAd, pEntry, &vht_cap, NULL);
 					dot11_vht_mcs_to_internal_mcs(pAd, &vht_cap, &pEntry->MaxHTPhyMode);
 
@@ -360,10 +391,10 @@ MAC_TABLE_ENTRY *MacTableInsertWDSEntry(
 			RTMPSetSupportMCS(pAd,
 						OPMODE_AP,
 						pEntry,
-						pAd->CommonCfg.SupRate,
-						pAd->CommonCfg.SupRateLen,
-						pAd->CommonCfg.ExtRate,
-						pAd->CommonCfg.ExtRateLen,
+						rate->SupRate,
+						rate->SupRateLen,
+						rate->ExtRate,
+						rate->ExtRateLen,
 #ifdef DOT11_VHT_AC
 						sizeof(VHT_CAP_IE),
 						&pEntry->vht_cap_ie,
@@ -391,7 +422,8 @@ MAC_TABLE_ENTRY *MacTableInsertWDSEntry(
 			pEntry->func_tb_idx = WdsTabIdx;
 			pEntry->wdev = wdev;
 			COPY_MAC_ADDR(&wdev->bssid[0], &pEntry->Addr[0]);
-
+			//update per wdev bw
+			wdev->bw = wdev->MaxHTPhyMode.field.BW;
 			WifiSysWdsLinkUp(pAd,wdev,pEntry->wcid);
 
 			AsicUpdateWdsEncryption(pAd, pEntry->wcid);
@@ -876,6 +908,7 @@ VOID WdsPeerBeaconProc(
 	VHT_CAP_IE *pVhtCap = NULL;
 	UCHAR VhtCapLen = 0;
 #endif /* DOT11_VHT_AC */
+	struct dev_rate_info *rate;
 
 	RETURN_IF_PAD_NULL(pAd);
 
@@ -1021,14 +1054,15 @@ VOID WdsPeerBeaconProc(
 #endif /* DOT11_N_SUPPORT */
 
 		pWdsEntry->wdev.PhyMode = WdsPhyOpModeToSuppPhyMode(pAd, pWdsEntry) & PeerPhyMode;
-		
+		rate = &pWdsEntry->wdev.rate;
+
 		RTMPSetSupportMCS(pAd,
 					OPMODE_AP,
 					pEntry,
-					pAd->CommonCfg.SupRate,
-					pAd->CommonCfg.SupRateLen,
-					pAd->CommonCfg.ExtRate,
-					pAd->CommonCfg.ExtRateLen,
+					rate->SupRate,
+					rate->SupRateLen,
+					rate->ExtRate,
+					rate->ExtRateLen,
 #ifdef DOT11_VHT_AC
 					sizeof(VHT_CAP_IE),
 					&pEntry->vht_cap_ie,
@@ -1413,6 +1447,8 @@ VOID WDS_Init(RTMP_ADAPTER *pAd, RTMP_OS_NETDEV_OP_HOOK *pNetDevOps)
 		wdev->channel = pAd->ApCfg.MBSSID[MAIN_MBSSID].wdev.channel;
 		wdev->allow_data_tx = TRUE;	// let tx_pkt_allowed() to check it!
 		wdev->PortSecured = WPA_802_1X_PORT_SECURED;
+		/*update rate info for wdev*/
+		RTMPUpdateRateInfo(wdev->PhyMode,&wdev->rate);
 		NdisMoveMemory(&wdev->if_addr[0], &pNetDevOps->devAddr[0], MAC_ADDR_LEN);
         os_move_mem(wdev->bss_info_argument.Bssid,wdev->if_addr,MAC_ADDR_LEN);//TODO: Carter, check flow with Linker
 
@@ -1453,7 +1489,6 @@ VOID WDS_Remove(RTMP_ADAPTER *pAd)
 	for(index = 0; index < MAX_WDS_ENTRY; index++)
 	{
 		wdev = &pAd->WdsTab.WdsEntry[index].wdev;
-		HcReleaseRadioForWdev(pAd,wdev);
 		if (wdev->if_dev)
 		{
 			RtmpOSNetDevProtect(1);
@@ -1558,7 +1593,7 @@ UCHAR WdsPhyOpModeToSuppPhyMode(PRTMP_ADAPTER pAd, PRT_802_11_WDS_ENTRY pWdsEntr
 			else
 				pWdsEntry->PhyOpMode = Legacy_CCK;
 		}
-		MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("ERROR!!! No matched PhyMode can be found, aline to default 0x%x\n", 
+		MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("!!! WARNING !!! No matched PhyMode can be found, align to default 0x%x\n", 
 			ApPhyMode));
 	}
 
