@@ -227,6 +227,9 @@ VOID ApCliCtrlStateMachineInit(
 	for (i = 0; i < MAX_APCLI_NUM; i++)
 	{
 		pAd->ApCfg.ApCliTab[i].CtrlCurrState = APCLI_CTRL_DISCONNECTED;
+		pAd->ApCfg.ApCliTab[i].LinkDownReason = APCLI_LINKDOWN_NONE;
+		pAd->ApCfg.ApCliTab[i].Disconnect_Sub_Reason = APCLI_DISCONNECT_SUB_REASON_NONE;
+
 #ifdef APCLI_CONNECTION_TRIAL
 		pApCliEntry = &pAd->ApCfg.ApCliTab[i];
 		/* timer init */
@@ -343,8 +346,8 @@ static VOID ApCliTrialConnectTimeoutAction(PRTMP_ADAPTER pAd, MLME_QUEUE_ELEM *p
 	pApCliEntry = &pAd->ApCfg.ApCliTab[ifIndex];
 	pCurrState = &pApCliEntry->CtrlCurrState;
 
-	if (pApCliEntry->TrialCh !=
-			HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1))) {
+	if ((pAd->CommonCfg.BBPCurrentBW != pAd->hw_cfg.bbp_bw) ||
+		(pApCliEntry->TrialCh != HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1)))) {
 		/* switch channel to original channel */
 		ch = HcGetCentralChByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1));
 		MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
@@ -382,8 +385,13 @@ static VOID ApCliTrialConnectTimeoutAction(PRTMP_ADAPTER pAd, MLME_QUEUE_ELEM *p
 			pApCliEntry->NewRootApRetryCnt = 0;//cleanup retry count
 			pApCliEntry->Enable = FALSE;
 		}
-		else
+		else {
+			/* trial connection probe fail                                               */
+			/* change apcli sync state machine to idle state to reset sync state machine */
+			PULONG pSync_CurrState = &pAd->ApCfg.ApCliTab[ifIndex].SyncCurrState;
+			*pSync_CurrState = APCLI_SYNC_IDLE;
 			*pCurrState = APCLI_CTRL_DISCONNECTED;//Disconnected State will bring the next probe req, auth req.
+		}
 	}
 
 	return;
@@ -405,8 +413,8 @@ static VOID ApCliTrialPhase2TimeoutAction(PRTMP_ADAPTER pAd, MLME_QUEUE_ELEM *pE
 			 to issue Assoc Req to new root AP\n",
 			 __func__, pApCliEntry->TrialCh));
 
-	if (pApCliEntry->TrialCh !=
-		HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1))) {
+	if ((BW_20 != HcGetBw(pAd,&pApCliEntry->wdev))||
+		(pApCliEntry->TrialCh != HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1)))) {
 		/* TBD disable beacon? */
 		ApCliTrialConnectionBeaconControl(pAd,FALSE);
 		MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
@@ -471,8 +479,8 @@ static VOID ApCliTrialConnectRetryTimeoutAction(PRTMP_ADAPTER pAd, MLME_QUEUE_EL
 			pApCliEntry->Enable = FALSE;
 		}
 		
-		if (pApCliEntry->TrialCh !=
-				HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1))) {
+		if ((pAd->CommonCfg.BBPCurrentBW != pAd->hw_cfg.bbp_bw) ||
+			(pApCliEntry->TrialCh != HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1)))) {
 			ch = HcGetCentralChByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1));
 			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
 				 ("%s[%d]Change bw %s to %s,ch %d to %d(%d)\n\r",
@@ -503,8 +511,8 @@ static VOID ApCliTrialConnectRetryTimeoutAction(PRTMP_ADAPTER pAd, MLME_QUEUE_EL
 
 		//MacTableDeleteEntry(pAd, pApCliEntry->MacTabWCID, APCLI_ROOT_BSSID_GET(pAd, pApCliEntry->MacTabWCID));
 		MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("ApCli_SYNC - %s, jump back to origin channel to wait for User's operation!\n", __func__));
-		if (pApCliEntry->TrialCh !=
-		HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1))) {
+		if ((pAd->CommonCfg.BBPCurrentBW != pAd->hw_cfg.bbp_bw) ||
+			(pApCliEntry->TrialCh != HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1)))) {
 			ch = HcGetCentralChByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1));
 			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
 				 ("%s[%d]Change bw %s to %s,ch %d to %d(%d)\n\r",
@@ -556,8 +564,8 @@ static VOID ApCliTrialConnectRetryTimeoutAction(PRTMP_ADAPTER pAd, MLME_QUEUE_EL
 			pApCliEntry->Enable = FALSE;
 			ApCliLinkDown(pAd, ifIndex);
 		}
-		if (pApCliEntry->TrialCh !=
-				HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1))) {
+		if ((pAd->CommonCfg.BBPCurrentBW != pAd->hw_cfg.bbp_bw) ||
+			(pApCliEntry->TrialCh != HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1)))) {
 			ch = HcGetCentralChByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1));
 			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
 				 ("%s[%d]Change bw %s to %s,ch %d to %d(%d)\n\r",
@@ -600,6 +608,7 @@ static VOID ApCliCtrlJoinReqAction(
 	PWSC_CTRL	pWpsCtrl = &pAd->ApCfg.ApCliTab[ifIndex].WscControl;
 #endif /* WSC_AP_SUPPORT */
 	struct wifi_dev *wdev = NULL;
+	PULONG pLinkDownReason = &pAd->ApCfg.ApCliTab[ifIndex].LinkDownReason;
 
 	wdev = &pAd->ApCfg.ApCliTab[ifIndex].wdev;
 
@@ -620,12 +629,12 @@ static VOID ApCliCtrlJoinReqAction(
 	}
 
 #ifdef WSC_AP_SUPPORT
-    if ((pWpsCtrl->WscConfMode != WSC_DISABLE) &&
+	if ((pWpsCtrl->WscConfMode != WSC_DISABLE) &&
 		(pWpsCtrl->bWscTrigger == TRUE))
-    {
-    	ULONG bss_idx = 0;
-        NdisZeroMemory(JoinReq.Ssid, MAX_LEN_OF_SSID);
-        JoinReq.SsidLen = pAd->ApCfg.ApCliTab[ifIndex].WscControl.WscSsid.SsidLength;
+	{
+		ULONG bss_idx = 0;
+		NdisZeroMemory(JoinReq.Ssid, MAX_LEN_OF_SSID);
+		JoinReq.SsidLen = pAd->ApCfg.ApCliTab[ifIndex].WscControl.WscSsid.SsidLength;
 		NdisMoveMemory(JoinReq.Ssid, pAd->ApCfg.ApCliTab[ifIndex].WscControl.WscSsid.Ssid, JoinReq.SsidLen);
 		if (pWpsCtrl->WscMode == 1) /* PIN */
 		{
@@ -640,25 +649,15 @@ static VOID ApCliCtrlJoinReqAction(
 				INT old_conf_mode = pWpsCtrl->WscConfMode;
 				ADD_HTINFO	RootApHtInfo, ApHtInfo;
 				UCHAR channel = wdev->channel, RootApChannel = pAd->ScanTab.BssEntry[bss_idx].Channel;
-				UCHAR RootApCentralChannel = pAd->ScanTab.BssEntry[bss_idx].CentralChannel;
 				ApHtInfo = pAd->CommonCfg.AddHTInfo.AddHtInfo;
 				RootApHtInfo = pAd->ScanTab.BssEntry[bss_idx].AddHtInfo.AddHtInfo;
 
-				if ((RootApChannel != channel) ||
-					((RootApCentralChannel != RootApChannel) &&
-					 (pAd->CommonCfg.HtCapability.HtCapInfo.ChannelWidth  == BW_40) &&
-					 (ApHtInfo.ExtChanOffset != RootApHtInfo.ExtChanOffset)))
+				if (RootApChannel != channel)
 				{
 					RTMP_STRING ChStr[5] = {0};
-					if (pAd->CommonCfg.HtCapability.HtCapInfo.ChannelWidth  == BW_40)
-					{
-						if (RootApHtInfo.ExtChanOffset == EXTCHA_ABOVE)
-							Set_HtExtcha_Proc(pAd, "1");
-						else
-							Set_HtExtcha_Proc(pAd, "0");
-					}
-					snprintf(ChStr, sizeof(ChStr), "%d", pAd->ScanTab.BssEntry[bss_idx].Channel);
+					snprintf(ChStr, sizeof(ChStr), "%d", RootApChannel);
 					Set_Channel_Proc(pAd, ChStr);
+					wdev->channel = RootApChannel;
 					/*
 						ApStop will call WscStop, we need to reset WscConfMode, WscMode & bWscTrigger here.
 					*/
@@ -706,6 +705,7 @@ static VOID ApCliCtrlJoinReqAction(
 				pApCliEntry->wdev.channel = pApCliEntry->wdev.CentralChannel;
 				pApCliEntry->wdev.bw = HT_BW_20;
 				pApCliEntry->wdev.extcha = EXTCHA_NONE;
+				HcUpdateExtCha(pAd,pApCliEntry->wdev.channel,pApCliEntry->wdev.extcha);
 #endif /* CONFIG_MULTI_CHANNEL */
 			AsicSwitchChannel(pAd, pApCliEntry->MlmeAux.Channel, FALSE);
 			AsicLockChannel(pAd, pApCliEntry->MlmeAux.Channel);
@@ -721,6 +721,7 @@ static VOID ApCliCtrlJoinReqAction(
 		__FUNCTION__, JoinReq.Ssid, PRINT_MAC(JoinReq.Bssid)));
 
 	*pCurrState = APCLI_CTRL_PROBE;
+	*pLinkDownReason = APCLI_LINKDOWN_NONE;
 
 	MlmeEnqueue(pAd, APCLI_SYNC_STATE_MACHINE, APCLI_MT2_MLME_PROBE_REQ,
 		sizeof(APCLI_MLME_JOIN_REQ_STRUCT), &JoinReq, ifIndex);
@@ -1478,6 +1479,7 @@ static VOID ApCliCtrlDeAssocRspAction(
 	UCHAR CliIdx = 0xFF;
 	REPEATER_CLIENT_ENTRY *pReptEntry = NULL;
 #endif /* MAC_REPEATER_SUPPORT */
+	PULONG pLinkDownReason = NULL;
 
 	if ((ifIndex >= MAX_APCLI_NUM)
 #ifdef MAC_REPEATER_SUPPORT
@@ -1493,11 +1495,14 @@ static VOID ApCliCtrlDeAssocRspAction(
 		pReptEntry = &pAd->ApCfg.pRepeaterCliPool[CliIdx];
 		ifIndex = pReptEntry->wdev->func_idx;
 		pCurrState = &pReptEntry->CtrlCurrState;
+		pLinkDownReason = &pReptEntry->LinkDownReason;
 	}
 	else
 #endif /* MAC_REPEATER_SUPPORT */
-	pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].CtrlCurrState;
-
+	{
+		pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].CtrlCurrState;
+		pLinkDownReason = &pAd->ApCfg.ApCliTab[ifIndex].LinkDownReason;
+	}
 	pApCliEntry = &pAd->ApCfg.ApCliTab[ifIndex];
 	if (Status == MLME_SUCCESS)
 	{
@@ -1518,8 +1523,10 @@ static VOID ApCliCtrlDeAssocRspAction(
 	ifIndex = (USHORT)(Elem->Priv);
 #endif /* MAC_REPEATER_SUPPORT */
 
-	if (bValid)
+	if (bValid) {
+		*pLinkDownReason = APCLI_LINKDOWN_PEER_DEASSOC_RSP;
 		ApCliLinkDown(pAd, ifIndex);
+	}
 
 	*pCurrState = APCLI_CTRL_DISCONNECTED;
 
@@ -1641,6 +1648,7 @@ static VOID ApCliCtrlDisconnectReqAction(
 	UCHAR CliIdx = 0xFF;
     REPEATER_CLIENT_ENTRY *pReptEntry = NULL;
 #endif /* MAC_REPEATER_SUPPORT */
+	PULONG pLinkDownReason = NULL;
 
 	MTWF_LOG(DBG_CAT_CLIENT, CATCLIENT_APCLI, DBG_LVL_TRACE, ("(%s) MLME Request disconnect.\n", __FUNCTION__));
 
@@ -1658,11 +1666,14 @@ static VOID ApCliCtrlDisconnectReqAction(
 		pReptEntry = &pAd->ApCfg.pRepeaterCliPool[CliIdx];
 		ifIndex = pReptEntry->wdev->func_idx;
 		pCurrState = &pReptEntry->CtrlCurrState;
+		pLinkDownReason = &pReptEntry->LinkDownReason;
 	}
 	else
 #endif /* MAC_REPEATER_SUPPORT */
-	pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].CtrlCurrState;
-
+	{
+		pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].CtrlCurrState;
+		pLinkDownReason = &pAd->ApCfg.ApCliTab[ifIndex].LinkDownReason;
+	}
 	pApCliEntry = &pAd->ApCfg.ApCliTab[ifIndex];
 
 #ifdef MAC_REPEATER_SUPPORT
@@ -1677,9 +1688,10 @@ static VOID ApCliCtrlDisconnectReqAction(
 #endif /* MAC_REPEATER_SUPPORT */
 
 	MTWF_LOG(DBG_CAT_CLIENT, CATCLIENT_APCLI, DBG_LVL_ERROR, ("(%s) 2. Before do ApCliLinkDown.\n", __FUNCTION__));
-	if (bValid)
+	if (bValid) {
+		*pLinkDownReason = APCLI_LINKDOWN_DISCONNECT_REQ;
 		ApCliLinkDown(pAd, ifIndex);
-
+	}
 	/* set the apcli interface be invalid. */
 #ifdef MAC_REPEATER_SUPPORT
 	if (CliIdx == 0xFF)
@@ -1721,6 +1733,7 @@ static VOID ApCliCtrlPeerDeAssocReqAction(
 #ifdef APCLI_AUTO_CONNECT_SUPPORT
 	USHORT apcli_ifIndex = (USHORT)(Elem->Priv);
 #endif
+	PULONG pLinkDownReason = NULL;
 
 	MTWF_LOG(DBG_CAT_CLIENT, CATCLIENT_APCLI, DBG_LVL_TRACE, ("(%s) Peer DeAssoc Req.\n", __FUNCTION__));
 
@@ -1738,11 +1751,14 @@ static VOID ApCliCtrlPeerDeAssocReqAction(
 		pReptEntry = &pAd->ApCfg.pRepeaterCliPool[CliIdx];
 		ifIndex = pReptEntry->wdev->func_idx;
 		pCurrState = &pReptEntry->CtrlCurrState;
+		pLinkDownReason = &pReptEntry->LinkDownReason;
 	}
 	else
 #endif /* MAC_REPEATER_SUPPORT */
+	{
         pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].CtrlCurrState;
-
+		pLinkDownReason = &pAd->ApCfg.ApCliTab[ifIndex].LinkDownReason;
+	}
     pApCliEntry = &pAd->ApCfg.ApCliTab[ifIndex];
 
 #ifdef MAC_REPEATER_SUPPORT
@@ -1756,8 +1772,10 @@ static VOID ApCliCtrlPeerDeAssocReqAction(
                 pReptEntry = &pAd->ApCfg.pRepeaterCliPool[index];
                 if ((pReptEntry->CliEnable) && (pReptEntry->wdev == &pApCliEntry->wdev))
                 {
-                    if (pReptEntry->CliValid)
+                    if (pReptEntry->CliValid) {
+						pReptEntry->LinkDownReason = APCLI_LINKDOWN_PEER_DEASSOC_REQ;
                         ApCliLinkDown(pAd, (REPT_MLME_START_IDX + index));
+					}
                 }
             }
         }
@@ -1775,8 +1793,10 @@ static VOID ApCliCtrlPeerDeAssocReqAction(
 	ifIndex = (USHORT)(Elem->Priv);
 #endif /* MAC_REPEATER_SUPPORT */
 
-	if (bValid)
+	if (bValid) {
+		*pLinkDownReason = APCLI_LINKDOWN_PEER_DEASSOC_REQ;
 		ApCliLinkDown(pAd, ifIndex);
+	}
 
 #ifdef APCLI_AUTO_CONNECT_SUPPORT
 	if ((pAd->ApCfg.ApCliAutoConnectRunning[apcli_ifIndex] == TRUE)
@@ -1829,6 +1849,7 @@ static VOID ApCliCtrlDeAssocAction(
 	UCHAR CliIdx = 0xFF;
     REPEATER_CLIENT_ENTRY *pReptEntry = NULL;
 #endif /* MAC_REPEATER_SUPPORT */
+	PULONG pLinkDownReason = NULL;
 
 	MTWF_LOG(DBG_CAT_CLIENT, CATCLIENT_APCLI, DBG_LVL_TRACE, ("(%s) MLME Request Disconnect.\n", __FUNCTION__));
 
@@ -1846,11 +1867,14 @@ static VOID ApCliCtrlDeAssocAction(
 		pReptEntry = &pAd->ApCfg.pRepeaterCliPool[CliIdx];
 		ifIndex = pReptEntry->wdev->func_idx;
 		pCurrState = &pReptEntry->CtrlCurrState;
+		pLinkDownReason = &pReptEntry->LinkDownReason;
 	}
 	else
 #endif /* MAC_REPEATER_SUPPORT */
-	pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].CtrlCurrState;
-
+	{
+		pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].CtrlCurrState;
+		pLinkDownReason = &pAd->ApCfg.ApCliTab[ifIndex].LinkDownReason;
+	}
 	pApCliEntry = &pAd->ApCfg.ApCliTab[ifIndex];
 
 	DisassocParmFill(pAd, &DisassocReq, pAd->ApCfg.ApCliTab[ifIndex].MlmeAux.Bssid, REASON_DISASSOC_STA_LEAVING);
@@ -1870,8 +1894,10 @@ static VOID ApCliCtrlDeAssocAction(
 		sizeof(MLME_DISASSOC_REQ_STRUCT), &DisassocReq, ifIndex);
     RTMP_MLME_HANDLER(pAd);
 
-	if (bValid)
+	if (bValid) {
+		*pLinkDownReason = APCLI_LINKDOWN_DEASSOC_REQ;
 		ApCliLinkDown(pAd, ifIndex);
+	}
 
 	/* set the apcli interface be invalid. */
 #ifdef MAC_REPEATER_SUPPORT
@@ -1907,6 +1933,7 @@ static VOID ApCliCtrlDeAuthAction(
 	USHORT ifIndex = (USHORT)(Elem->Priv);
 	PULONG pCurrState = NULL;
 	BOOLEAN bValid = FALSE;
+	PULONG pLinkDownReason = NULL;
 #ifdef MAC_REPEATER_SUPPORT
 	UCHAR CliIdx = 0xFF;
 	REPEATER_CLIENT_ENTRY *pReptEntry = NULL;
@@ -1928,10 +1955,14 @@ static VOID ApCliCtrlDeAuthAction(
 		pReptEntry = &pAd->ApCfg.pRepeaterCliPool[CliIdx];
 		ifIndex = pReptEntry->MatchApCliIdx;
 		pCurrState = &pReptEntry->CtrlCurrState;
+		pLinkDownReason = &pReptEntry->LinkDownReason;
 	}
 	else
 #endif /* MAC_REPEATER_SUPPORT */
-	pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].CtrlCurrState;
+	{
+		pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].CtrlCurrState;
+		pLinkDownReason = &pAd->ApCfg.ApCliTab[ifIndex].LinkDownReason;
+	}
 
 	pApCliEntry = &pAd->ApCfg.ApCliTab[ifIndex];
 
@@ -1958,8 +1989,10 @@ static VOID ApCliCtrlDeAuthAction(
 				  ifIndex);
     RTMP_MLME_HANDLER(pAd);
 
-	if (bValid)
+	if (bValid) {
+		*pLinkDownReason = APCLI_LINKDOWN_DEAUTH_REQ;
 		ApCliLinkDown(pAd, ifIndex);
+	}
 
 	/* set the apcli interface be invalid. */
 #ifdef MAC_REPEATER_SUPPORT
@@ -2177,8 +2210,8 @@ static VOID ApCliCtrlTrialConnectAction(
 		MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("(%s) channel TrialConnectTimer\n", __func__));
 		RTMPCancelTimer(&(pApCliEntry->TrialConnectTimer), &Cancelled);
 
-		if (pApCliEntry->TrialCh !=
-			HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1))) {
+		if ((BW_20 != HcGetBw(pAd,&pApCliEntry->wdev))||
+			(pApCliEntry->TrialCh != HcGetChannelByRf(pAd,(HcGetBandByWdev(&pApCliEntry->wdev)+1)))) {
 			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("(%s) Jump to CH:%d\n", __func__, pApCliEntry->TrialCh));
 			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("(%s) pAd->CommonCfg.CentralChannel:%d\n", __func__, pAd->CommonCfg.CentralChannel));
 			
