@@ -144,6 +144,7 @@ static VOID ApCliMlmeProbeReqAction(
 	PULONG pCurrState = &pAd->ApCfg.ApCliTab[ifIndex].SyncCurrState;
 	APCLI_STRUCT *pApCliEntry = NULL;
 	struct wifi_dev *wdev = &pAd->ApCfg.ApCliTab[ifIndex].wdev;
+	struct dev_rate_info *rate = &wdev->rate;
 
 	MTWF_LOG(DBG_CAT_CLIENT, CATCLIENT_APCLI, DBG_LVL_TRACE, ("ApCli SYNC - ApCliMlmeProbeReqAction(Ssid %s)\n", Info->Ssid));
 
@@ -176,12 +177,12 @@ static VOID ApCliMlmeProbeReqAction(
 	/*Assign Channel for APCLI*/
 	pApCliEntry->MlmeAux.Channel = wdev->channel;
 #endif /* APCLI_CONNECTION_TRIAL */
-	pApCliEntry->MlmeAux.SupRateLen = pAd->CommonCfg.SupRateLen;
-	NdisMoveMemory(pApCliEntry->MlmeAux.SupRate, pAd->CommonCfg.SupRate, pAd->CommonCfg.SupRateLen);
+	pApCliEntry->MlmeAux.SupRateLen = rate->SupRateLen;
+	NdisMoveMemory(pApCliEntry->MlmeAux.SupRate, rate->SupRate, rate->SupRateLen);
 
 	/* Prepare the default value for extended rate */
-	pApCliEntry->MlmeAux.ExtRateLen = pAd->CommonCfg.ExtRateLen;
-	NdisMoveMemory(pApCliEntry->MlmeAux.ExtRate, pAd->CommonCfg.ExtRate, pAd->CommonCfg.ExtRateLen);
+	pApCliEntry->MlmeAux.ExtRateLen = rate->ExtRateLen;
+	NdisMoveMemory(pApCliEntry->MlmeAux.ExtRate, rate->ExtRate, rate->ExtRateLen);
 #endif /* RT_CFG80211_P2P_CONCURRENT_DEVICE */
 
 	RTMPSetTimer(&(pApCliEntry->MlmeAux.ProbeTimer), PROBE_TIMEOUT);
@@ -739,6 +740,9 @@ static VOID ApCliEnqueueProbeRequest(
 #ifdef WSC_AP_SUPPORT	
 	BOOLEAN bHasWscIe = FALSE;
 #endif /* WSC_AP_SUPPORT */
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+    UCHAR ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */
 
 	MTWF_LOG(DBG_CAT_CLIENT, CATCLIENT_APCLI, DBG_LVL_TRACE, ("force out a ProbeRequest ...\n"));
 
@@ -796,7 +800,16 @@ static VOID ApCliEnqueueProbeRequest(
         vht_ie_info.channel = pApCliEntry->wdev.channel;
         vht_ie_info.phy_mode = pApCliEntry->wdev.PhyMode;
 
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+        ucETxBfCap = pAd->CommonCfg.ETxBfEnCond;
+        if (HcIsBfCapSupport(&pApCliEntry->wdev) == FALSE)
+        {
+            pAd->CommonCfg.ETxBfEnCond = 0;
+        }
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
+
         FrameLen += build_vht_ies(pAd, &vht_ie_info);
+
 #endif /* DOT11_VHT_AC */
 
 #ifdef WSC_AP_SUPPORT
