@@ -860,7 +860,7 @@ VOID PeerPublicAction(RTMP_ADAPTER *pAd, MLME_QUEUE_ELEM *Elem)
 						}
 						MTWF_LOG(DBG_CAT_PROTO, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("\n"));
 
-						retVal = GetBssCoexEffectedChRange(pAd, &coexChRange,wdev->channel);
+						retVal = GetBssCoexEffectedChRange(pAd,wdev,&coexChRange,wdev->channel);
 						if (retVal == TRUE)
 						{
 							ptr = pIntolerantReport->ChList;
@@ -893,7 +893,7 @@ VOID PeerPublicAction(RTMP_ADAPTER *pAd, MLME_QUEUE_ELEM *Elem)
 
 					if (bNeedFallBack)
 					{
-                        struct wifi_dev *wdev = NULL;
+                        struct wifi_dev *twdev = NULL;
 						int apidx;
 
 						NdisMoveMemory((PUCHAR)&pAd->CommonCfg.LastBSSCoexist2040,
@@ -943,12 +943,13 @@ VOID PeerPublicAction(RTMP_ADAPTER *pAd, MLME_QUEUE_ELEM *Elem)
                             {
                                 pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth = BW_20;
                                 pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = EXTCHA_NONE;
-                                pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_NONE;
+								wdev->extcha = EXTCHA_NONE;
+								HcUpdateExtCha(pAd,wdev->channel,EXTCHA_NONE);
                             }
                             else
                             {
                                 pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth = pAd->CommonCfg.RegTransmitSetting.field.BW;
-                                pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = pAd->CommonCfg.RegTransmitSetting.field.EXTCHA;
+                                pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = wdev->extcha;
                             }
                             MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE,
                                     ("\tNow RecomWidth=%d, ExtChanOffset=%d\n",
@@ -958,12 +959,12 @@ VOID PeerPublicAction(RTMP_ADAPTER *pAd, MLME_QUEUE_ELEM *Elem)
                         }
 
                         apidx = pAd->MacTab.Content[Elem->Wcid].apidx;
-                        wdev = &pAd->ApCfg.MBSSID[apidx].wdev;
-                        if (wdev)
+                        twdev = &pAd->ApCfg.MBSSID[apidx].wdev;
+                        if (twdev)
                         {
                             MTWF_LOG(DBG_CAT_PROTO, DBG_SUBCAT_ALL, DBG_LVL_TRACE,
                                     ("%s, ACTION_BSS_2040_COEXIST Update Beacon for idx:%d\n", __FUNCTION__, apidx));
-                            UpdateBeaconHandler(pAd, wdev, IE_CHANGE);
+                            UpdateBeaconHandler(pAd, twdev, IE_CHANGE);
                         }
 					}
 #ifdef APCLI_SUPPORT	
