@@ -697,6 +697,7 @@ BOOLEAN BARecSessionAdd(
 	/* Start fill in parameters.*/
 	if (pBAEntry != NULL)
 	{
+		NdisAcquireSpinLock(&pAd->BATabLock);
 		pBAEntry->REC_BA_Status = Recipient_HandleRes;
 		pBAEntry->BAWinSize = BAWinSize;
 		pBAEntry->Wcid = pEntry->wcid;
@@ -707,6 +708,7 @@ BOOLEAN BARecSessionAdd(
 		pBAEntry->LastIndSeq = RESET_RCV_SEQ; /*pFrame->BaStartSeq.field.StartSeq;*/
 
 		MTWF_LOG(DBG_CAT_PROTO, CATPROTO_BA, DBG_LVL_OFF, ("Start Seq = %08x\n",  pFrame->BaStartSeq.field.StartSeq));
+		NdisReleaseSpinLock(&pAd->BATabLock);
 
 		if (pEntry->RXBAbitmap & (1<<TID))
 			RTMPCancelTimer(&pBAEntry->RECBATimer, &Cancelled);
@@ -821,7 +823,7 @@ VOID BATableFreeOriEntry(RTMP_ADAPTER *pAd, ULONG Idx)
 		{
 			pAd->BATable.numDoneOriginator -= 1;
 		 	pEntry->TXBAbitmap &= (~(1<<(pBAEntry->TID) ));
-			MTWF_LOG(DBG_CAT_PROTO, CATPROTO_BA, DBG_LVL_TRACE, ("BATableFreeOriEntry numAsOriginator= %ld\n", pAd->BATable.numAsRecipient));
+			MTWF_LOG(DBG_CAT_PROTO, CATPROTO_BA, DBG_LVL_TRACE, ("BATableFreeOriEntry numAsOriginator= %ld\n", pAd->BATable.numDoneOriginator));
 			/* Erase Bitmap flag.*/
 		}
 
@@ -846,7 +848,7 @@ VOID BATableFreeRecEntry(RTMP_ADAPTER *pAd, ULONG Idx)
 	if ((Idx == 0) || (Idx >= MAX_LEN_OF_BA_REC_TABLE))
 		return;
 
-	pBAEntry =&pAd->BATable.BARecEntry[Idx];
+	pBAEntry = &pAd->BATable.BARecEntry[Idx];
 
 	NdisAcquireSpinLock(&pAd->BATabLock);
 	if (pBAEntry->REC_BA_Status != Recipient_NONE)
@@ -856,7 +858,12 @@ VOID BATableFreeRecEntry(RTMP_ADAPTER *pAd, ULONG Idx)
 
 		ASSERT(pAd->BATable.numAsRecipient != 0);
 
-		pAd->BATable.numAsRecipient -= 1;
+		if (pAd->BATable.numAsRecipient > 0) {
+			pAd->BATable.numAsRecipient -= 1;
+		} else {
+			MTWF_LOG(DBG_CAT_PROTO, CATPROTO_BA, DBG_LVL_OFF, ("Idx = %lu, REC_BA_Status = %d, Wcid(pBAEntry) = %d,\
+				Wcid(pEntry) = %d, Tid = %d\n", Idx, pBAEntry->REC_BA_Status, pBAEntry->Wcid, pEntry->wcid, pBAEntry->TID));
+		}
 
 		pBAEntry->REC_BA_Status = Recipient_NONE;
 	}
