@@ -273,6 +273,13 @@ INT bcn_buf_deinit(RTMP_ADAPTER *pAd, BCN_BUF_STRUC *bcn_info)
         RTMP_SEM_LOCK(&bcn_info->BcnContentLock);
         RTMPFreeNdisPacket(pAd, bcn_info->BeaconPkt);
         bcn_info->BeaconPkt = NULL;
+
+		if (bcn_info->tim_buf.TimPkt)
+		{
+			RTMPFreeNdisPacket(pAd, bcn_info->tim_buf.TimPkt);
+			bcn_info->tim_buf.TimPkt = NULL;
+		}
+
         RTMP_SEM_UNLOCK(&bcn_info->BcnContentLock);
     }
 
@@ -691,24 +698,37 @@ VOID MakeChSwitchAnnounceIEandExtend(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, U
             ptr += 2;
             NdisZeroMemory(&wb_info, sizeof(WIDE_BW_CH_SWITCH_ELEMENT));
 
-			switch(pComCfg->vht_bw){
-			case VHT_BW_2040:
-                wb_info.new_ch_width = 0;
-			break;
-			case VHT_BW_80:
-                wb_info.new_ch_width = 1;
-				wb_info.center_freq_1 = vht_cent_ch_freq(wdev->channel, pComCfg->vht_bw);
-			break;
-			case VHT_BW_160:
-                wb_info.new_ch_width = 2;
-				wb_info.center_freq_1 = vht_cent_ch_freq(wdev->channel, pComCfg->vht_bw);
-			break;
-			case VHT_BW_8080:
-                wb_info.new_ch_width = 3;
-				wb_info.center_freq_1 = vht_cent_ch_freq(wdev->channel, pComCfg->vht_bw);
-                wb_info.center_freq_2 = vht_cent_ch_freq(pAd->CommonCfg.vht_cent_ch2, pComCfg->vht_bw);
-			break;
-			}
+	    switch(pComCfg->vht_bw){
+		    case VHT_BW_2040:
+			    wb_info.new_ch_width = 0;
+			    break;
+		    case VHT_BW_80:
+			    wb_info.new_ch_width = 1;
+			    wb_info.center_freq_1 = vht_cent_ch_freq(wdev->channel, pComCfg->vht_bw);
+			    wb_info.center_freq_2 = 0;
+			    break;
+		    case VHT_BW_160:
+#ifdef DOT11_VHT_R2 
+			    wb_info.new_ch_width = 1;
+			    wb_info.center_freq_1 = (vht_cent_ch_freq(wdev->channel, pComCfg->vht_bw) - 8);
+			    wb_info.center_freq_2 = vht_cent_ch_freq(wdev->channel, pComCfg->vht_bw);
+#else
+			    wb_info.new_ch_width = 2;
+			    wb_info.center_freq_1 = vht_cent_ch_freq(wdev->channel, pComCfg->vht_bw);
+#endif /* DOT11_VHT_R2 */
+			    break;
+		    case VHT_BW_8080:
+#ifdef DOT11_VHT_R2 
+			    wb_info.new_ch_width = 1;
+			    wb_info.center_freq_1 = vht_cent_ch_freq(wdev->channel, pComCfg->vht_bw);
+			    wb_info.center_freq_2 = vht_cent_ch_freq(pAd->CommonCfg.vht_cent_ch2, pComCfg->vht_bw);
+#else
+			    wb_info.new_ch_width = 3;
+			    wb_info.center_freq_1 = vht_cent_ch_freq(wdev->channel, pComCfg->vht_bw);
+			    wb_info.center_freq_2 = vht_cent_ch_freq(pAd->CommonCfg.vht_cent_ch2, pComCfg->vht_bw);
+#endif /* DOT11_VHT_R2 */
+			    break;
+	    }
 
             NdisMoveMemory(ptr, &wb_info, sizeof(WIDE_BW_CH_SWITCH_ELEMENT));
             wb_len = sizeof(WIDE_BW_CH_SWITCH_ELEMENT);
@@ -739,7 +759,7 @@ static VOID ExtChannelCheck(RTMP_ADAPTER *pAd,struct wifi_dev *wdev)
 {
     /* NOTE: 2015-Aug-10
 
-        RegTransmitSetting.field.EXTCHA will be updated in HcBbpSetBwByChannel,
+        EXTCHA will be updated in HcBbpSetBwByChannel,
         but when 20/40 Coexist happened in protocol stage,
         we shall consider the station which connected already with BW_40.
         so we cannot switch HW to BW_20 directly, since we don't disconnect other station.
@@ -752,7 +772,7 @@ static VOID ExtChannelCheck(RTMP_ADAPTER *pAd,struct wifi_dev *wdev)
 	/* sanity check for extention channel */
 	N_ChannelCheck(pAd,wdev->PhyMode,wdev->channel);
 	pAd->CommonCfg.AddHTInfo.ControlChan = wdev->channel;
-    if ((pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_NONE) &&
+    if ((wdev->extcha == EXTCHA_NONE) &&
         (pAd->CommonCfg.LastBSSCoexist2040.field.BSS20WidthReq ||
         pAd->CommonCfg.LastBSSCoexist2040.field.Intolerant40))
     {
@@ -765,7 +785,7 @@ static VOID ExtChannelCheck(RTMP_ADAPTER *pAd,struct wifi_dev *wdev)
         else
             pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth = 0;    	 
     }
-    pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = pAd->CommonCfg.RegTransmitSetting.field.EXTCHA;
+    pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = wdev->extcha;
 }
 
 VOID MakeHTIe(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLen, UCHAR *pBeaconFrame)
@@ -792,6 +812,20 @@ VOID MakeHTIe(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLen, UCHAR
     NdisMoveMemory(&HtCapabilityTmp, &pComCfg->HtCapability, HtLen);
     HtCapabilityTmp.HtCapInfo.ChannelWidth = pComCfg->AddHTInfo.AddHtInfo.RecomWidth;
 
+#ifdef TXBF_SUPPORT
+    if (HcIsBfCapSupport(wdev) == FALSE)
+    {
+        UCHAR ucEBfCap;
+
+        ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+        pAd->CommonCfg.ETxBfEnCond = 0;
+            
+        mt_WrapSetETxBFCap(pAd, &HtCapabilityTmp.TxBFCap);
+
+        pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+    }
+#endif /* TXBF_SUPPORT */    
+
     MakeOutgoingFrame(pBeaconFrame+FrameLen,         &TmpLen,
                                 1,                                &HtCapIe,
                                 1,                                &HtLen,
@@ -803,6 +837,21 @@ VOID MakeHTIe(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLen, UCHAR
 #else
     NdisMoveMemory(&HtCapabilityTmp, &pComCfg->HtCapability, HtLen);
     HtCapabilityTmp.HtCapInfo.ChannelWidth = pComCfg->AddHTInfo.AddHtInfo.RecomWidth;
+
+#ifdef TXBF_SUPPORT
+    if (HcIsBfCapSupport(wdev) == FALSE)
+    {
+        UCHAR ucEBfCap;
+
+        ucEBfCap = pAd->CommonCfg.ETxBfEnCond;
+        pAd->CommonCfg.ETxBfEnCond = 0;
+            
+        mt_WrapSetETxBFCap(pAd, &HtCapabilityTmp.TxBFCap);
+
+        pAd->CommonCfg.ETxBfEnCond = ucEBfCap;
+    }
+#endif /* TXBF_SUPPORT */
+
     *(USHORT *)(&HtCapabilityTmp.HtCapInfo) = SWAP16(*(USHORT *)(&HtCapabilityTmp.HtCapInfo));
 #ifdef UNALIGNMENT_SUPPORT
     {
@@ -1093,7 +1142,7 @@ VOID MakeCountryIe(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLen, U
 
             /* prepare channel information */
 #ifdef EXT_BUILD_CHANNEL_LIST
-            BuildBeaconChList(pAd, TmpFrame, &TmpLen2);
+            BuildBeaconChList(pAd, wdev, TmpFrame, &TmpLen2);
 #else
             {
                 UCHAR MaxTxPower = GetCuntryMaxTxPwr(pAd, wdev->PhyMode,wdev->channel);
@@ -1206,13 +1255,13 @@ VOID MakeExtSuppRateIe(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLe
     ULONG TmpLen;
     UCHAR PhyMode = wdev->PhyMode;
 
-    if ((pAd->CommonCfg.ExtRateLen) && (PhyMode != WMODE_B))
+    if ((wdev->rate.ExtRateLen) && (PhyMode != WMODE_B))
     {
         TmpLen = 0;
         MakeOutgoingFrame(pBeaconFrame+FrameLen,         &TmpLen,
                         1,                               &ExtRateIe,
-                        1,                               &pAd->CommonCfg.ExtRateLen,
-                        pAd->CommonCfg.ExtRateLen,       pAd->CommonCfg.ExtRate,
+                        1,                               &wdev->rate.ExtRateLen,
+                        wdev->rate.ExtRateLen,       wdev->rate.ExtRate,
                         END_OF_ARGS);
         FrameLen += TmpLen;
 
@@ -1278,9 +1327,9 @@ VOID MakePwrConstraintIe(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrame
 
 VOID ComposeBcnPktTail(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLen, UCHAR *pBeaconFrame)
 {
-    COMMON_CONFIG *pComCfg = &pAd->CommonCfg;
     ULONG FrameLen = *pFrameLen;
 #ifdef CONFIG_AP_SUPPORT
+    COMMON_CONFIG *pComCfg = &pAd->CommonCfg;
     BSS_STRUCT *pMbss = NULL;
     UCHAR apidx = 0;
     //BOOLEAN HotSpotEnable = FALSE;
@@ -1289,6 +1338,9 @@ VOID ComposeBcnPktTail(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLe
 #ifdef AP_QLOAD_SUPPORT
 	QLOAD_CTRL *pQloadCtrl = HcGetQloadCtrl(pAd);
 #endif /*AP_QLOAD_SUPPORT*/
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+    UCHAR ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */
 
 #ifdef CONFIG_AP_SUPPORT
     if (wdev->wdev_type == WDEV_TYPE_AP)
@@ -1326,7 +1378,7 @@ VOID ComposeBcnPktTail(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLe
     if (IS_RRM_ENABLE(pAd, apidx))
     {
         InsertTpcReportIE(pAd, pBeaconFrame+FrameLen, &FrameLen,
-        RTMP_GetTxPwr(pAd, pAd->CommonCfg.MlmeTransmit,wdev->channel), 0);
+        RTMP_GetTxPwr(pAd, wdev->rate.MlmeTransmit,wdev->channel), 0);
         RRM_InsertRRMEnCapIE(pAd, pBeaconFrame+FrameLen, &FrameLen, apidx);
     }
 #endif /* DOT11K_RRM_SUPPORT */
@@ -1352,7 +1404,7 @@ VOID ComposeBcnPktTail(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLe
 #endif /* DOT11R_FT_SUPPORT */
 
     /* Update ERP */
-    if ((pComCfg->ExtRateLen) && (PhyMode != WMODE_B))
+    if ((wdev->rate.ExtRateLen) && (PhyMode != WMODE_B))
     {
         MakeErpIE(pAd, wdev, &FrameLen, pBeaconFrame);
     }
@@ -1395,7 +1447,19 @@ VOID ComposeBcnPktTail(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLe
         vht_ie_info.channel = wdev->channel;
         vht_ie_info.phy_mode = PhyMode;
 
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+        ucETxBfCap = pAd->CommonCfg.ETxBfEnCond;
+        if (HcIsBfCapSupport(wdev) == FALSE)
+        {
+            pAd->CommonCfg.ETxBfEnCond = 0;
+        }
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
+
         FrameLen += build_vht_ies(pAd, &vht_ie_info);
+
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+		pAd->CommonCfg.ETxBfEnCond = ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
 #endif /* DOT11_VHT_AC */
     }
 #endif /* DOT11_N_SUPPORT */
@@ -1464,55 +1528,13 @@ VOID ComposeBcnPktTail(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLe
 
     *pFrameLen = FrameLen;
 }
-
-VOID MakeSupRateIe(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, ULONG *pFrameLen, UCHAR *pBeaconFrame)
-{
-    UCHAR SupRateLen = 0;
-    UCHAR PhyMode = wdev->PhyMode;
-    UCHAR SupRate[MAX_LEN_OF_SUPPORTED_RATES] = {0}, *pSupRate;
-    ULONG TmpLen = 0;
-    ULONG FrameLen = *pFrameLen;
-    UCHAR DsLen = 1;
-
-    pSupRate = &SupRate[0];
-    SupRateLen = pAd->CommonCfg.SupRateLen;
-
-    if (PhyMode == WMODE_B)
-        SupRateLen = 4;
-
-        pSupRate = pAd->CommonCfg.SupRate;
-
-    TmpLen = 0;
-    MakeOutgoingFrame(pBeaconFrame+FrameLen,    &TmpLen,
-                    1,                          &SupRateIe,
-                    1,                          &SupRateLen,
-                    SupRateLen,                 pSupRate,
-                    1,                          &DsIe,
-                    1,                          &DsLen,
-                    1,                          &wdev->channel,
-                    END_OF_ARGS);
-
-    FrameLen += TmpLen;
-
-    if ((pAd->CommonCfg.ExtRateLen) && (PhyMode != WMODE_B))
-    {
-        MakeOutgoingFrame(pBeaconFrame+FrameLen,         &TmpLen,
-                        1,                               &ExtRateIe,
-                        1,                               &pAd->CommonCfg.ExtRateLen,
-                        pAd->CommonCfg.ExtRateLen,       pAd->CommonCfg.ExtRate,
-                        END_OF_ARGS);
-        FrameLen += TmpLen;
-    }
-    *pFrameLen = FrameLen;
-}
-
 VOID updateBeaconRoutineCase(RTMP_ADAPTER *pAd, BOOLEAN UpdateAfterTim)
 {
     INT     i;
-    struct wifi_dev *wdev = NULL;
-
+	 struct wifi_dev *wdev ;
 #ifdef CONFIG_AP_SUPPORT
     BOOLEAN FlgQloadIsAlarmIssued = FALSE;
+	wdev = &pAd->ApCfg.MBSSID[0].wdev;
 
     if (pAd->ApCfg.DtimCount == 0)
         pAd->ApCfg.DtimCount = pAd->ApCfg.DtimPeriod - 1;
@@ -1545,12 +1567,13 @@ VOID updateBeaconRoutineCase(RTMP_ADAPTER *pAd, BOOLEAN UpdateAfterTim)
         {
             pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth = BW_20;
             pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = EXTCHA_NONE;
-            pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_NONE;
+            wdev->extcha = EXTCHA_NONE;
+			HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
         }
         else
         {
             pAd->CommonCfg.AddHTInfo.AddHtInfo.RecomWidth = pAd->CommonCfg.RegTransmitSetting.field.BW;
-            pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = pAd->CommonCfg.RegTransmitSetting.field.EXTCHA;
+            pAd->CommonCfg.AddHTInfo.AddHtInfo.ExtChanOffset = wdev->extcha;
         }
         MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE,
                 ("\tNow RecomWidth=%d, ExtChanOffset=%d, prevBW=%d, prevExtOffset=%d\n",
@@ -1823,7 +1846,7 @@ ULONG ComposeBcnPktHead(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, UCHAR *pBeacon
       if wdev is AP, SupRateLen is global setting,
       shall check each's wdev setting to update SupportedRate.
     */
-    SupRateLen = pAd->CommonCfg.SupRateLen;
+    SupRateLen = wdev->rate.SupRateLen;
     if (PhyMode == WMODE_B)
         SupRateLen = 4;
 
@@ -1831,7 +1854,7 @@ ULONG ComposeBcnPktHead(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, UCHAR *pBeacon
     MakeOutgoingFrame(pBeaconFrame+FrameLen,      &TmpLen,
                     1,                              &SupRateIe,
                     1,                              &SupRateLen,
-                    SupRateLen,                     pAd->CommonCfg.SupRate,
+                    SupRateLen,                     wdev->rate.SupRate,
                     END_OF_ARGS);
 
     FrameLen += TmpLen;
@@ -1848,3 +1871,207 @@ ULONG ComposeBcnPktHead(RTMP_ADAPTER *pAd, struct wifi_dev *wdev, UCHAR *pBeacon
     return FrameLen;
 }
 
+#ifdef MT7615
+#ifdef CONFIG_AP_SUPPORT
+UINT32 nobcn = 0;
+#define CR4_HEART_BEAT_STS 0x80200
+VOID BcnCheck(RTMP_ADAPTER *pAd)
+{
+	if ((pAd->Mlme.PeriodicRound % 50) == 0) {
+		UINT32 mac_val;
+		UINT32 bcn_cnt = 0;
+		UINT32 recoverext=0;
+		UINT32 Index;
+		BOOLEAN bcnactive = FALSE;
+		struct wifi_dev *wdev;
+
+#ifdef ERR_RECOVERY
+		if(IsErrRecoveryInIdleStat(pAd) == FALSE)
+			return;
+#endif
+#ifdef DOT11_N_SUPPORT
+#ifdef DOT11N_DRAFT3
+		if (pAd->CommonCfg.bOverlapScanning)
+			return;
+#endif
+#endif
+
+#ifdef CONFIG_ATE
+		if (ATE_ON(pAd))
+			return;
+#endif
+
+#ifdef MT_DFS_SUPPORT
+		if (pAd->Dot11_H.RDMode == RD_SILENCE_MODE)
+			return;
+#endif
+
+		if (!(RTMP_TEST_FLAG(pAd, fRTMP_ADAPTER_SYSEM_READY)))
+			return;
+
+		if (RTMP_TEST_FLAG(pAd, (fRTMP_ADAPTER_RADIO_OFF | 
+					fRTMP_ADAPTER_HALT_IN_PROGRESS |
+					fRTMP_ADAPTER_BSS_SCAN_IN_PROGRESS)))
+			return;
+
+		for (Index = 0; Index < WDEV_NUM_MAX; Index++) {
+			wdev = pAd->wdev_list[Index];
+			if (wdev == NULL)
+				continue;
+			if ((HcGetBandByWdev(wdev) == DBDC_BAND0) 
+				&& (wdev->bss_info_argument.Active == TRUE)
+				&& (wdev->bcn_buf.bBcnSntReq)) {
+				bcnactive = TRUE;
+				break;
+			}
+		}
+		if (bcnactive == FALSE)
+			return;
+
+		MAC_IO_READ32(pAd, MIB_M0SDR0, &mac_val);
+		bcn_cnt = (mac_val & 0xffff);
+
+			
+		if (bcn_cnt == 0) {
+			nobcn++;
+			if (nobcn > 4) {
+				if (nobcn % 6 == 0) //6*5=30s
+					MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
+						("%s: no bcn still occur within %d sec!!\n",__FUNCTION__, nobcn * 5));
+				if (nobcn == 5)
+					MtCmdFwLog2Host(pAd, 0, 0);
+				return;
+			}
+		}
+		else if (nobcn != 0) {
+			recoverext = 1;
+			nobcn = 0;
+		}
+		else {
+			nobcn = 0;
+			return;
+		}
+
+		if ((nobcn != 0 || recoverext == 1) && DebugLevel >= DBG_LVL_ERROR) {
+			UINT32 mac_val;
+			UINT32 addr;
+			UINT32 idx;
+			if (recoverext == 1)
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, 
+					("%s: bcn recover!!\ndebug info dump as below:\n\n",
+					__FUNCTION__));
+			else
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, 
+					("%s: no bcn occurs within %d sec!!\ndebug info dump as below:\n\n",
+					__FUNCTION__, nobcn * 5));
+			dump_dmac_mib_info(pAd, "0");
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			show_trinfo_proc(pAd,"");
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			ShowPLEInfo(pAd, NULL);
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			dump_dmac_pse_info(pAd);
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			show_tpinfo_proc(pAd, NULL);
+
+			MtCmdFwLog2Host(pAd, 0, 2);
+			MtCmdFwLog2Host(pAd, 1, 2);
+			MtCmdFwLog2Host(pAd, 1, 155);
+			MtCmdFwLog2Host(pAd, 1, 156);
+			MtCmdFwLog2Host(pAd, 1, 39);
+			MtCmdFwLog2Host(pAd, 0, 4);
+			MtCmdFwLog2Host(pAd, 0, 11);
+
+			MAC_IO_READ32(pAd, ARB_BFCR, &mac_val);
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("ARB_BFCR(0x820f3190)=0x%08x\n", mac_val));
+
+			MAC_IO_READ32(pAd, ARB_SCR, &mac_val);
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("ARB_SCR(0x820f3080)=0x%08x\n", mac_val));
+
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("dump 0x820f3100~0x820f3154:\n"));
+			for (addr = ARB_TQSW0; addr <= ARB_TQPM1; addr = addr + 4) {
+				if ((addr & 0xf) == 0 && (addr != ARB_TQSW0))
+					MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+				MAC_IO_READ32(pAd, addr, &mac_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("addr 0x%05x=0x%08x ", addr, mac_val));
+			}
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			MAC_IO_READ32(pAd, ARB_BFCR, &mac_val);
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("ARB_BFCR(0x820f3190)=0x%08x\n", mac_val));
+			MAC_IO_READ32(pAd, CR4_HEART_BEAT_STS, &mac_val);
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("CR4 heart beat status (0x80200)=0x%08x\n", mac_val));
+			MtCmdFwLog2Host(pAd, 1, 0);
+
+			MAC_IO_READ32(pAd, RO_BAND0_PHYCTRL_STS, &mac_val);
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("RO_BAND0_PHYCTRL_STS(0x82070230)= 0x%08x\n", mac_val));
+			// 0x82070618~0x8207065c
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("dump 0x82070618~0x8207065c:\n"));
+			for (addr = PHY_BAND0_PHYMUX_6; addr <= PHY_BAND0_PHYMUX_23; addr = addr + 4) {
+				if ((addr & 0xf) == 8 && (addr != PHY_BAND0_PHYMUX_6))
+					MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+				MAC_IO_READ32(pAd, addr, &mac_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("addr 0x%05x=0x%08x ", addr, mac_val));
+			}
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			// 0x8207227c~0x82072294
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("dump 0x8207227c~0x82072294:\n"));
+			for (addr = RO_BAND0_RXTD_DEBUG0; addr <= RO_BAND0_RXTD_DEBUG6; addr = addr + 4) {
+				if (addr == RO_BAND0_RXTD_DEBUG4)
+					MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+				MAC_IO_READ32(pAd, addr, &mac_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("addr 0x%05x=0x%08x ", addr, mac_val));
+			}
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			// 0x820721a0~0x820721b8
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("dump 0x820721a0~0x820721b8:\n"));
+			for (addr = RO_BAND0_AGC_DEBUG_0; addr <= RO_BAND0_AGC_DEBUG_6; addr = addr + 4) {
+				if (addr == RO_BAND0_AGC_DEBUG_4)
+					MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+				MAC_IO_READ32(pAd, addr, &mac_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("addr 0x%05x=0x%08x ", addr, mac_val));
+			}
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+
+			/* 
+				
+				Write 0x82070614[22:20]=0
+				Write 0x82070614[22:20]=5
+			
+				// read the following registers by 10 times
+				0X8207_020C
+				0x8207_0210
+				0x8207_0214
+				0x8207_021C
+				0x8207_0220
+				// End of Loop
+				
+			*/
+			MAC_IO_READ32(pAd, PHY_BAND0_PHYMUX_5, &mac_val);
+			mac_val &= ~(BITS(20,22));
+			MAC_IO_WRITE32(pAd, PHY_BAND0_PHYMUX_5, mac_val);
+			MAC_IO_READ32(pAd, PHY_BAND0_PHYMUX_5, &mac_val);
+			mac_val &= ~(BITS(20,22));
+			mac_val |= (BIT(20) | BIT(22));
+			MAC_IO_WRITE32(pAd, PHY_BAND0_PHYMUX_5, mac_val);
+			for (idx = 0; idx < 10; idx++) {
+				MAC_IO_READ32(pAd, RO_BAND0_PHYCTRL_STS0, &mac_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("loop %d addr(0x8207020c)=0x%08x ",idx, mac_val));
+				MAC_IO_READ32(pAd, RO_BAND0_PHYCTRL_STS1, &mac_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("addr(0x82070210)=0x%08x ", mac_val));
+				MAC_IO_READ32(pAd, RO_BAND0_PHYCTRL_STS2, &mac_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("addr(0x82070214)=0x%08x ", mac_val));
+				MAC_IO_READ32(pAd, RO_BAND0_PHYCTRL_STS4, &mac_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("addr(0x8207021c)=0x%08x ", mac_val));
+				MAC_IO_READ32(pAd, RO_BAND0_PHYCTRL_STS5, &mac_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("addr(0x82070220)=0x%08x\n", mac_val));
+				
+			}
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n"));
+			if (recoverext == 1)
+				MtCmdFwLog2Host(pAd, 0, 0);
+		}
+	}
+}
+#endif
+#endif
+
