@@ -247,7 +247,9 @@ VOID BackgroundScanStateMachineInit(
 	StateMachineSetAction(Sm, BGND_SCAN_IDLE, BGND_SCAN_REQ, (STATE_MACHINE_FUNC)BackgroundScanStartAction);
 	StateMachineSetAction(Sm, BGND_SCAN_LISTEN, BGND_SCAN_TIMEOUT, (STATE_MACHINE_FUNC)BackgroundScanTimeoutAction);
 	StateMachineSetAction(Sm, BGND_SCAN_LISTEN, BGND_SCAN_CNCL, (STATE_MACHINE_FUNC)BackgroundScanCancelAction);
-
+	StateMachineSetAction(Sm, BGND_SCAN_IDLE, BGND_SWITCH_CHANNEL, (STATE_MACHINE_FUNC)BackgroundSwitchChannelAction);	
+	StateMachineSetAction(Sm, BGND_SCAN_IDLE, BGND_RDD_REQ, (STATE_MACHINE_FUNC)DfsZeroWaitStartAction);
+	StateMachineSetAction(Sm, BGND_RDD_DETEC, BGND_RDD_TIMEOUT, (STATE_MACHINE_FUNC)DfsZeroWaitStopAction);
 }
 
 VOID BackgroundScanInit (
@@ -281,7 +283,8 @@ VOID BackgroundScanInit (
 	/* Decide RxPath&TxStream for background */
 	pAd->BgndScanCtrl.RxPath = 0xc;
 	pAd->BgndScanCtrl.TxStream = 0x2;
-	pAd->BgndScanCtrl.ScanDuration = 500;
+	pAd->BgndScanCtrl.ScanDuration = 200;
+	pAd->BgndScanCtrl.DfsZeroWaitDuration = 60000; /* 1 min */
 	
 				if (pAd->CommonCfg.dbdc_mode == TRUE) {
                         pAd->BgndScanCtrl.BgndScanSupport = 0;
@@ -302,6 +305,7 @@ VOID BackgroundScanInit (
 		//Copy channel list for background.
 		BuildBgndScanChList(pAd);
 		RTMPInitTimer(pAd, &pAd->BgndScanCtrl.BgndScanTimer, GET_TIMER_FUNCTION(BackgroundScanTimeout), pAd, FALSE);
+		RTMPInitTimer(pAd, &pAd->BgndScanCtrl.DfsZeroWaitTimer, GET_TIMER_FUNCTION(DfsZeroWaitTimeout), pAd, FALSE);
 		
 		//ToDo: Related CR initialization.
 		MAC_IO_WRITE32(pAd, MIB_M1SCR, 0x7ef3ffff); /* 0x820fd200 Enable Band1 PSCCA time */
@@ -314,6 +318,11 @@ VOID BackgroundScanInit (
 		/* Enabel PSCCA time count & Enable EDCCA time count */
 		
 		//MtCmdMultipleMacRegAccessWrite
+		pAd->BgndScanCtrl.BgndScanAllow = FALSE;
+		pAd->BgndScanCtrl.BgndScanInterval = DefaultBgndScanInterval;
+		pAd->BgndScanCtrl.BgndScanIntervalCount = 0;
+		pAd->BgndScanCtrl.SkipDfsChannel = FALSE;
+		pAd->BgndScanCtrl.DfsZeroWaitSupport = TRUE;
 	}
 	else
 	{
@@ -327,6 +336,7 @@ VOID BackgroundScanDeInit(
 {
     BOOLEAN Cancelled;
     RTMPReleaseTimer(&pAd->BgndScanCtrl.BgndScanTimer, &Cancelled);	   
+    RTMPReleaseTimer(&pAd->BgndScanCtrl.DfsZeroWaitTimer, &Cancelled);	 
 }
 
 VOID BackgroundScanStart(
@@ -374,7 +384,6 @@ VOID BackgroundScanTimeout(
 	RTMP_MLME_HANDLER(pAd);
 }
 
-
 VOID BackgroundScanTimeoutAction(
 	RTMP_ADAPTER *pAd,
 	MLME_QUEUE_ELEM *Elem)
@@ -418,9 +427,11 @@ VOID BackgroundScanNextChannel(
 	MT_BGND_SCAN_CFG BgndScanCfg;
 	RTMP_REG_PAIR Reg[5];
 	MT_BGND_SCAN_NOTIFY BgScNotify;
-	//INT	i;
+	INT	i;
 //	UINT32	pccatime, sccatime, edtime, band0txtime, mdrdy;
 	UCHAR	BestChannel=0;
+	UCHAR	RxStreamNums =0;
+	UCHAR	RxPath =0;
 	MT_SWITCH_CHANNEL_CFG *CurrentSwChCfg;
 
 	/* Restore switch channel configuration */
@@ -429,12 +440,23 @@ VOID BackgroundScanNextChannel(
 	if (pAd->BgndScanCtrl.ScanChannel == 0)
 	{		
 		pAd->BgndScanCtrl.BgndScanStatMachine.CurrState = BGND_SCAN_IDLE;//Scan Stop
+		/* RxStream to RxPath */
+		RxStreamNums = CurrentSwChCfg->RxStream; 
+		if (RxStreamNums > 4) {
+            			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR,
+                            		("%s():illegal RxStreamNums(%d) \n",
+                            		__FUNCTION__, RxStreamNums));
+            			RxStreamNums = 4;
+        		}
+
+        		for (i = 0; i < RxStreamNums; i++)
+            		RxPath |= 1 << i;	
 		BgndScanCfg.BandIdx=0;
 		BgndScanCfg.Bw = CurrentSwChCfg->Bw;
 		BgndScanCfg.ControlChannel= CurrentSwChCfg->ControlChannel;
 		BgndScanCfg.CentralChannel = CurrentSwChCfg->CentralChannel;
 		BgndScanCfg.Reason = CH_SWITCH_BACKGROUND_SCAN_STOP;
-		BgndScanCfg.RxPath =CurrentSwChCfg->RxStream; /* return to 4 Rx */
+		BgndScanCfg.RxPath =RxPath; /* return to 4 Rx */
 		BgndScanCfg.TxStream = CurrentSwChCfg->TxStream;
 		MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("%s Stop Scan Bandidx=%d, BW=%d, CtrlCh=%d, CenCh=%d, Reason=%d, RxPath=%d\n", 
 			__FUNCTION__, BgndScanCfg.BandIdx, BgndScanCfg.Bw, BgndScanCfg.ControlChannel,
@@ -450,7 +472,11 @@ VOID BackgroundScanNextChannel(
 		MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("Dump Channel Info \n"));
 		GenerateGroupChannelList(pAd);
 		BestChannel = BgndSelectBestChannel(pAd);
+		pAd->BgndScanCtrl.BestChannel = BestChannel;
 		MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("Best Channel=%d", BestChannel));
+		if (BestChannel != CurrentSwChCfg->ControlChannel)
+			MlmeEnqueue(pAd, BGND_SCAN_STATE_MACHINE, BGND_SWITCH_CHANNEL, 0, NULL, 0); 
+		
 		
 	}
 	else if (pAd->BgndScanCtrl.ScanChannel == pAd->BgndScanCtrl.FirstChannel)
@@ -526,7 +552,170 @@ VOID BackgroundScanNextChannel(
 	}
 }
 
+VOID BackgroundSwitchChannelAction(
+                IN RTMP_ADAPTER *pAd,
+                IN MLME_QUEUE_ELEM *Elem)
+{
+        UCHAR	BestChannel[5] = {0};
+ 	pAd->BgndScanCtrl.BgndScanStatMachine.CurrState = BGND_CS_ANN; 	
+        sprintf(BestChannel, "%d", pAd->BgndScanCtrl.BestChannel);
+        printk("Switch to channel to %d \n", pAd->BgndScanCtrl.BestChannel);
+        Set_Channel_Proc(pAd, BestChannel);
+	pAd->BgndScanCtrl.BgndScanStatMachine.CurrState = BGND_SCAN_IDLE; 
+}
+       
+VOID BackgroundChannelSwitchAnnouncementAction(
+                IN RTMP_ADAPTER *pAd,
+                IN MLME_QUEUE_ELEM *Elem)
+{
+	printk("Trigger Channel Switch Announcemnet IE \n");
+	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("Trigger Channel Switch Announcemnet IE \n"));
+	pAd->BgndScanCtrl.BgndScanStatMachine.CurrState = BGND_CS_ANN; //Channel switch annoncement.
+//	HcUpdateCsaCntByChannel(pAd, pAd->BgndScanCtrl.BestChannel);
+//	pAd->BgndScanCtrl.BgndScanStatMachine.CurrState = BGND_SCAN_IDLE; //For temporary
+}
 
+VOID DfsZeroWaitTimeout( 
+	IN PVOID SystemSpecific1, 
+	IN PVOID FunctionContext, 
+	IN PVOID SystemSpecific2, 
+	IN PVOID SystemSpecific3)
+{
+	PRTMP_ADAPTER pAd = (PRTMP_ADAPTER)FunctionContext;
+	
+	MlmeEnqueue(pAd, BGND_SCAN_STATE_MACHINE, BGND_RDD_TIMEOUT, 0, NULL, 0);
+	RTMP_MLME_HANDLER(pAd);
+}
+
+VOID DfsZeroWaitStart(
+	IN PRTMP_ADAPTER pAd,
+	IN BOOLEAN	DfsZeroWaitEnable)
+{
+
+	if (DfsZeroWaitEnable) 	
+		MlmeEnqueue(pAd, BGND_SCAN_STATE_MACHINE, BGND_RDD_REQ, 0, NULL, 0); 
+	else
+		MlmeEnqueue(pAd, BGND_SCAN_STATE_MACHINE, BGND_RDD_CNCL, 0, NULL, 0); /* So far not thing to do. */
+	
+	RTMP_MLME_HANDLER(pAd);
+}
+
+
+VOID DfsZeroWaitStartAction(
+                IN RTMP_ADAPTER *pAd,
+                IN MLME_QUEUE_ELEM *Elem)
+{
+	MT_BGND_SCAN_CFG BgndScanCfg;
+	UCHAR		channel_idx;
+	MT_BGND_SCAN_NOTIFY BgScNotify;
+	MT_SWITCH_CHANNEL_CFG *CurrentSwChCfg;
+
+	pAd->BgndScanCtrl.BgndScanStatMachine.CurrState = BGND_RDD_DETEC;
+
+	CurrentSwChCfg = &(pAd->BgndScanCtrl.CurrentSwChCfg[0]);	
+	
+	BgScNotify.NotifyFunc =  (0x2 << 5 | 0xf);
+	BgScNotify.BgndScanStatus = 1;//start
+	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("DfsZeroWaitStartAction Notify NotifyFunc=%x, Status=%d", BgScNotify.NotifyFunc, BgScNotify.BgndScanStatus));
+	MtCmdBgndScanNotify(pAd, BgScNotify);
+	
+	for (channel_idx = 0; channel_idx < pAd->BgndScanCtrl.ChannelListNum; channel_idx++) {
+		if (pAd->BgndScanCtrl.DfsZeroWaitChannel == pAd->BgndScanCtrl.BgndScanChList[channel_idx].Channel) {
+			BgndScanCfg.CentralChannel = pAd->BgndScanCtrl.BgndScanChList[channel_idx].CenChannel;
+			break;
+		}		
+	}
+	
+	BgndScanCfg.ControlChannel = pAd->BgndScanCtrl.DfsZeroWaitChannel;
+	BgndScanCfg.BandIdx=1;
+	BgndScanCfg.Bw = pAd->BgndScanCtrl.ScanBW;
+	BgndScanCfg.Reason = CH_SWITCH_BACKGROUND_SCAN_START;
+	BgndScanCfg.RxPath =0x0C; /* Distribute 2 Rx for background scan */
+	BgndScanCfg.TxStream = 2;
+	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("%s Start DFS Zero Wait Bandidx=%d,	BW=%d, CtrlCh=%d, CenCh=%d, Reason=%d, RxPath=%d\n", 
+		__FUNCTION__, BgndScanCfg.BandIdx, BgndScanCfg.Bw, BgndScanCfg.ControlChannel,
+		BgndScanCfg.CentralChannel, BgndScanCfg.Reason, BgndScanCfg.RxPath));
+	MtCmdBgndScan(pAd, BgndScanCfg);
+
+	//Fill band0 BgndScanCfg
+	BgndScanCfg.BandIdx=0;
+	BgndScanCfg.Bw = CurrentSwChCfg->Bw /*pAd->BgndScanCtrl.ScanBW*/;
+	BgndScanCfg.ControlChannel= CurrentSwChCfg->ControlChannel;
+	BgndScanCfg.CentralChannel = CurrentSwChCfg->CentralChannel;
+	BgndScanCfg.Reason = CH_SWITCH_BACKGROUND_SCAN_START;
+	BgndScanCfg.RxPath =0x03; /* Keep 2 Rx for original service */
+	BgndScanCfg.TxStream = 2;
+	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("%s Start DFS Zero Wait Bandidx=%d,	BW=%d, CtrlCh=%d, CenCh=%d, Reason=%d, RxPath=%d\n", 
+		__FUNCTION__, BgndScanCfg.BandIdx, BgndScanCfg.Bw, BgndScanCfg.ControlChannel,
+		BgndScanCfg.CentralChannel, BgndScanCfg.Reason, BgndScanCfg.RxPath));
+	MtCmdBgndScan(pAd, BgndScanCfg);
+
+	/* TODO: Trigger band1 DFS API */
+
+	RTMPSetTimer(&pAd->BgndScanCtrl.DfsZeroWaitTimer, pAd->BgndScanCtrl.DfsZeroWaitDuration);
+
+
+}
+
+VOID DfsZeroWaitStopAction(
+                IN RTMP_ADAPTER *pAd,
+                IN MLME_QUEUE_ELEM *Elem)
+{
+	MT_BGND_SCAN_CFG BgndScanCfg;
+
+	MT_BGND_SCAN_NOTIFY BgScNotify;
+	MT_SWITCH_CHANNEL_CFG *CurrentSwChCfg;
+	UCHAR	RxStreamNums =0;
+	UCHAR	RxPath =0;
+	UCHAR	i;
+	UCHAR	DfsChannel[5] = {0};
+	BOOL	Cancelled;
+
+	RTMPCancelTimer(&pAd->BgndScanCtrl.DfsZeroWaitTimer, &Cancelled);	
+	pAd->BgndScanCtrl.BgndScanStatMachine.CurrState = BGND_SCAN_IDLE;
+	CurrentSwChCfg = &(pAd->BgndScanCtrl.CurrentSwChCfg[0]);
+
+	/* TODO: Turn off band1 DFS API */
+	
+	
+	/* RxStream to RxPath */
+	RxStreamNums = CurrentSwChCfg->RxStream; 
+	if (RxStreamNums > 4) {
+		MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR,
+            		("%s():illegal RxStreamNums(%d) \n",
+            		__FUNCTION__, RxStreamNums));
+		RxStreamNums = 4;
+	}
+
+	for (i = 0; i < RxStreamNums; i++)
+		RxPath |= 1 << i;	
+
+	BgndScanCfg.BandIdx=0;
+	BgndScanCfg.Bw = CurrentSwChCfg->Bw;
+	BgndScanCfg.ControlChannel= CurrentSwChCfg->ControlChannel;
+	BgndScanCfg.CentralChannel = CurrentSwChCfg->CentralChannel;
+	BgndScanCfg.Reason = CH_SWITCH_BACKGROUND_SCAN_STOP;
+	BgndScanCfg.RxPath =RxPath; /* return to 4 Rx */
+	BgndScanCfg.TxStream = CurrentSwChCfg->TxStream;
+	
+	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("%s Stop Scan Bandidx=%d, BW=%d, CtrlCh=%d, CenCh=%d, Reason=%d, RxPath=%d\n", 
+		__FUNCTION__, BgndScanCfg.BandIdx, BgndScanCfg.Bw, BgndScanCfg.ControlChannel,
+		BgndScanCfg.CentralChannel, BgndScanCfg.Reason, BgndScanCfg.RxPath));
+	MtCmdBgndScan(pAd, BgndScanCfg);
+	//Notify RA background scan stop
+	BgScNotify.NotifyFunc =  (BgndScanCfg.TxStream << 5 | 0xf);
+	BgScNotify.BgndScanStatus = 0;//stop
+	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("Background scan Notify NotifyFunc=%x, Status=%d", BgScNotify.NotifyFunc, BgScNotify.BgndScanStatus));
+	MtCmdBgndScanNotify(pAd, BgScNotify);
+
+	if (pAd->BgndScanCtrl.RadarDetected !=1) {
+		/* DFS channel is safe for using */
+		sprintf(DfsChannel, "%d", pAd->BgndScanCtrl.DfsZeroWaitChannel);
+        		printk("Switch to channel to %d \n", pAd->BgndScanCtrl.DfsZeroWaitChannel);
+		Set_Channel_Proc(pAd, DfsChannel);
+	}	
+		
+}
 VOID BackgroundScanTest(
 	IN PRTMP_ADAPTER pAd,
 	IN MT_BGND_SCAN_CFG BgndScanCfg)
@@ -535,3 +724,56 @@ VOID BackgroundScanTest(
 	
 	MtCmdBgndScan(pAd, BgndScanCfg);	
 }
+
+VOID ChannelQualityDetection(
+                IN PRTMP_ADAPTER pAd)
+{
+        UINT32 ChBusyTime=0;
+        UCHAR BandIdx=0;
+        UINT32      lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9, lv10, CrValue;
+        UCHAR Noisy=0;
+        //Phase 1: No traffic stat
+        //Check IPI & Channel Busy Time
+
+	
+        ChBusyTime = pAd->OneSecMibBucket.ChannelBusyTime[BandIdx];
+
+        
+        /* 1. Check Open enviroment via IPI (Band0) */
+        HW_IO_READ32(pAd, 0x12250, &lv0);
+        HW_IO_READ32(pAd, 0x12254, &lv1);
+        HW_IO_READ32(pAd, 0x12258, &lv2);
+        HW_IO_READ32(pAd, 0x1225c, &lv3);
+        HW_IO_READ32(pAd, 0x12260, &lv4);
+        HW_IO_READ32(pAd, 0x12264, &lv5);
+        HW_IO_READ32(pAd, 0x12268, &lv6);
+        HW_IO_READ32(pAd, 0x1226c, &lv7);
+        HW_IO_READ32(pAd, 0x12270, &lv8);
+        HW_IO_READ32(pAd, 0x12274, &lv9);
+        HW_IO_READ32(pAd, 0x12278, &lv10);
+
+	if (lv0+lv1+lv2+lv3+lv4+lv5+lv6+lv7+lv8+lv9+lv10 !=0)	
+		Noisy=((lv9+lv10)*100/(lv0+lv1+lv2+lv3+lv4+lv5+lv6+lv7+lv8+lv9+lv10));
+	pAd->BgndScanCtrl.Noisy = Noisy;
+        //MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_OFF,
+        //				("%s Band0:lv0~5 %d, %d, %d, %d, %d, %d  lv6~10 %d, %d, %d, %d, %d Noisy=%d, BusyTime=%d\n",
+        //				__FUNCTION__, lv0, lv1, lv2, lv3, lv4, lv5, lv6, lv7, lv8, lv9, lv10, Noisy, ChBusyTime));
+        HW_IO_READ32(pAd, PHY_RXTD_12, &CrValue);
+        CrValue |= (1 << 29);
+        HW_IO_WRITE32(pAd, PHY_RXTD_12, CrValue);// Cleaer
+        HW_IO_WRITE32(pAd, PHY_RXTD_12, CrValue);// Trigger again
+
+	if (pAd->BgndScanCtrl.BgndScanAllow == FALSE) {
+		pAd->BgndScanCtrl.BgndScanIntervalCount++;
+		if (pAd->BgndScanCtrl.BgndScanIntervalCount >= pAd->BgndScanCtrl.BgndScanInterval)
+			pAd->BgndScanCtrl.BgndScanAllow = TRUE;
+	}	
+	
+	if ((ChBusyTime > 450000 || Noisy > 50 )&& pAd->BgndScanCtrl.BgndScanAllow == TRUE) {
+		BackgroundScanStart(pAd, TRUE);
+		pAd->BgndScanCtrl.BgndScanAllow = FALSE;
+		pAd->BgndScanCtrl.BgndScanIntervalCount = 0;
+	}	
+
+}
+
