@@ -72,7 +72,7 @@ UINT GenerateWpsPinCode(
 	}
 #endif /* CONFIG_AP_SUPPORT */
 
-	iPin = macAddr[3] * 256 * 256 + macAddr[4] * 256 + macAddr[5];
+	iPin = macAddr[0] * 256 * 256 + macAddr[4] * 256 + macAddr[5];
 
 	iPin = iPin % 10000000;
 
@@ -916,14 +916,11 @@ INT RTMP_COM_IoctlHandle(
 
 
 #ifdef RTMP_PCI_SUPPORT
+
 		case CMD_RTPRIV_IOCTL_PCI_SUSPEND:
-			RTMP_SET_FLAG(pAd, fRTMP_ADAPTER_HALT_IN_PROGRESS);
-			RTMP_SET_FLAG(pAd, fRTMP_ADAPTER_RADIO_OFF);
 			break;
 
 		case CMD_RTPRIV_IOCTL_PCI_RESUME:
-			RTMP_CLEAR_FLAG(pAd, fRTMP_ADAPTER_HALT_IN_PROGRESS);
-			RTMP_CLEAR_FLAG(pAd, fRTMP_ADAPTER_RADIO_OFF);
 			break;
 
 		case CMD_RTPRIV_IOCTL_PCI_CSR_SET:
@@ -979,6 +976,9 @@ INT RTMP_COM_IoctlHandle(
                             ("mt_wifi_open return fail!\n"));
                     return NDIS_STATUS_FAILURE;
                 }
+#ifdef WSC_INCLUDED
+				WscUUIDInit(pAd, pObj->ioctl_if, FALSE);
+#endif /* WSC_INCLUDED */
             }
             else
             {
@@ -1985,7 +1985,7 @@ INT set_cr4_query(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
 
 	option = simple_strtoul(arg, 0, 16);
 
-	MtCmdCr4Query(pAd, option);
+	MtCmdCr4Query(pAd, option, 0, 0);
 
 	return TRUE;
 }
@@ -2734,6 +2734,54 @@ error:
 	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("iwpriv ra0 set tpc=h_en-\"value\"_h_th-\"value\"_l_en-\"value\"_l_th-\"value\"_REC_T-\"value\"_RFOFF_en-\"value\"_RFOFF_th-\"value\"\n"));
 	return TRUE;
 }
+
+INT set_thermal_protection_admin_ctrl_duty_proc(
+	IN PRTMP_ADAPTER	pAd,
+	IN RTMP_STRING		*arg)
+{
+	UINT32 u4Lv0Duty, u4Lv1Duty, u4Lv2Duty, u4Lv3Duty;
+	INT32 i4Ret = 0;
+
+	i4Ret = sscanf (arg, "%d-%d-%d-%d", &u4Lv0Duty, &u4Lv1Duty, &u4Lv2Duty, &u4Lv3Duty);
+
+	if (i4Ret != 4)
+	{
+		MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF,
+			("iwpriv ra0 set tpc_duty=Lv0Duty-Lv1Duty-Lv2Duty-Lv3Duty\n"));
+		return FALSE;
+	}
+	else
+	{
+		if (u4Lv0Duty > 100)
+		{
+			MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF,
+				("Parameters error! Lv0Duty > 100\n"));
+			return FALSE;
+		}
+		if (u4Lv1Duty > u4Lv0Duty)
+		{
+			MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF,
+				("Parameters error! Lv1Duty > Lv0Duty"));
+			return FALSE;
+		}
+		if (u4Lv2Duty > u4Lv1Duty)
+		{
+			MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF,
+				("Parameters error! Lv2Duty > Lv1Duty"));
+			return FALSE;
+		}
+		if (u4Lv3Duty > u4Lv2Duty)
+		{
+			MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF,
+				("Parameters error! Lv3Duty > Lv2Duty"));
+			return FALSE;
+		}
+
+		AsicThermalProtectAdmitDuty(pAd, u4Lv0Duty, u4Lv1Duty, u4Lv2Duty, u4Lv3Duty);
+	}
+
+	return TRUE;
+}
 #endif /* THERMAL_PROTECT_SUPPORT */
 
 #ifdef MT_MAC
@@ -2940,6 +2988,7 @@ INT Set_SCSCfg_Proc(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
     UINT32 SCSTrafficThreshold = 0;
     UINT32 OfdmPdSupport = 0;
     INT32   Recv=0;
+    UINT32 CckUpBond = 0, CckLowBond = 0, OfdmUpBond = 0, OfdmLowBond = 0, CckFixedBond = 0, OfdmFixedBond = 0;
     struct wifi_dev *wdev;
 
 
@@ -2952,15 +3001,21 @@ INT Set_SCSCfg_Proc(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
 
 
 
-    Recv = sscanf(arg, "%d-%d-%d-%d", &(SCSMinRssiTolerance), &(SCSThTolerance), &(SCSTrafficThreshold), &(OfdmPdSupport));
-    if (Recv != 4){
+    Recv = sscanf(arg, "%d-%d-%d-%d-%d-%d-%d-%d-%d-%d", &(SCSMinRssiTolerance), &(SCSThTolerance), &(SCSTrafficThreshold), &(OfdmPdSupport), &(CckUpBond), &(CckLowBond), &(OfdmUpBond), &(OfdmLowBond), &(CckFixedBond), &(OfdmFixedBond));
+    if (Recv != 10){
         MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("Format Error!\n"));
-        MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("iwpriv ra0 set SCSCfg=[MinRssiTolerance]-[ThTolerance]-[TrafficThreshold]-[OfdmSupport]"));
+        MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("iwpriv ra0 set SCSCfg=[MinRssiTolerance]-[ThTolerance]-[TrafficThreshold]-[OfdmSupport]-[CckUpBoundary]-[CckLowBoundary]-[OfdmUpBoundary]-[OfdmLowBoundary]-[FixedCckBoundary]-[FixedOfdmBoundary]"));
     } else {
         pAd->SCSCtrl.SCSMinRssiTolerance[BandIdx] = SCSMinRssiTolerance;
         pAd->SCSCtrl.SCSThTolerance[BandIdx] = SCSThTolerance;
         pAd->SCSCtrl.SCSTrafficThreshold[BandIdx] = SCSTrafficThreshold;
         pAd->SCSCtrl.OfdmPdSupport[BandIdx] = (UCHAR) OfdmPdSupport;
+        pAd->SCSCtrl.CckFalseCcaUpBond[BandIdx] = (UINT16)CckUpBond;
+        pAd->SCSCtrl.CckFalseCcaLowBond[BandIdx] = (UINT16)CckLowBond;
+        pAd->SCSCtrl.OfdmFalseCcaUpBond[BandIdx] = (UINT16)OfdmUpBond;
+        pAd->SCSCtrl.OfdmFalseCcaLowBond[BandIdx] = (UINT16)OfdmLowBond;	
+        pAd->SCSCtrl.CckFixedRssiBond[BandIdx] = (INT32)CckFixedBond;	
+        pAd->SCSCtrl.OfdmFixedRssiBond[BandIdx] = (INT32)OfdmFixedBond;
     }
     //MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s()  BandIdx=%d, SCSMinRssiTolerance=%d
     //    , SCSThTolerance=%d, SCSTrafficThreshold=%d\n", __FUNCTION__,
@@ -2970,14 +3025,11 @@ INT Set_SCSCfg_Proc(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
 }
 #endif /* SMART_CARRIER_SENSE_SUPPORT */
 
-INT SetTxPowerFeatureCtrl(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
+INT SetSKUCtrl(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
 {
-	UINT8	i;
+    UINT8	i;
 	CHAR	*value = 0;
 	UCHAR 	TxPowerSKUEn = 0;
-	UCHAR 	TxPowerPercentEn = 0;
-	UCHAR   MUTxPower = 0;
-	MT_TxPower_CTRL MtTxPowerCtrl;
 	INT     status = TRUE;
 	
 	for (i=0, value = rstrtok(arg,":"); value; value = rstrtok(NULL,":"), i++)
@@ -2987,12 +3039,6 @@ INT SetTxPowerFeatureCtrl(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
 			case 0:
 				TxPowerSKUEn = simple_strtol(value, 0, 10);
 				break;
-			case 1:
-				TxPowerPercentEn = simple_strtol(value, 0, 10);
-				break;
-			case 2:
-				MUTxPower = simple_strtol(value, 0, 10);
-				break;
 			default:
 			{
 				status = FALSE;
@@ -3002,15 +3048,106 @@ INT SetTxPowerFeatureCtrl(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
 		}
 	}
 
-	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s:  TxPowerSKUEn = %d, TxPowerPercentEn = %d \n", __FUNCTION__, TxPowerSKUEn, TxPowerPercentEn));
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s: TxPowerSKUEn = %d \n", __FUNCTION__, TxPowerSKUEn));
+	
+    return TxPowerSKUCtrl(pAd, TxPowerSKUEn); 
+}
 
-	MtTxPowerCtrl.TxPowerSKUEn = TxPowerSKUEn;
-	MtTxPowerCtrl.TxPowerPercentEn = TxPowerPercentEn;
-	MtTxPowerCtrl.MUTxPower = MUTxPower;
+INT SetPercentageCtrl(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
+{
+    UINT8	i;
+	CHAR	*value = 0;
+	UCHAR 	TxPowerPercenEn = 0;
+	INT     status = TRUE;
 	
-	MtCmdSetTxPowerFeatureCrtl(pAd, MtTxPowerCtrl);
+	for (i=0, value = rstrtok(arg,":"); value; value = rstrtok(NULL,":"), i++)
+	{
+		switch (i)
+		{
+			case 0:
+				TxPowerPercenEn = simple_strtol(value, 0, 10);
+				break;
+			default:
+			{
+				status = FALSE;
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("%s: set wrong parameters\n", __FUNCTION__));
+				break;
+			}	
+		}
+	}
+
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s: TxPowerPercenEn = %d \n", __FUNCTION__, TxPowerPercenEn));
 	
-    return TRUE;
+    return TxPowerPercentCtrl(pAd, TxPowerPercenEn);
+}
+
+INT SetPowerUpperBoundCtrl(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
+{
+    UINT8	i;
+	CHAR	*value = 0;
+	UCHAR 	PwrUppBound = 0;
+	INT     status = TRUE;
+	
+	for (i=0, value = rstrtok(arg,":"); value; value = rstrtok(NULL,":"), i++)
+	{
+		switch (i)
+		{
+			case 0:
+				PwrUppBound = simple_strtol(value, 0, 10);
+				break;
+			default:
+			{
+				status = FALSE;
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("%s: set wrong parameters\n", __FUNCTION__));
+				break;
+			}	
+		}
+	}
+
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s: PwrUppBound = %d \n", __FUNCTION__, PwrUppBound));
+	
+    return TxPowerBoundCtrl(pAd, PwrUppBound);
+}
+
+INT SetRfTxAnt(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
+{
+    UINT8	i;
+	CHAR	*value = 0;
+    UCHAR   TxAntCtrlEn = 0;
+	UCHAR 	WIFI_En[4] = {0,0,0,0};
+	INT     status = TRUE;
+	
+	for (i=0, value = rstrtok(arg,":"); value; value = rstrtok(NULL,":"), i++)
+	{
+		switch (i)
+		{
+            case 0:
+                TxAntCtrlEn = simple_strtol(value, 0, 10);
+                break;
+            case 1:
+				WIFI_En[0] = simple_strtol(value, 0, 10);
+				break;
+            case 2:
+				WIFI_En[1] = simple_strtol(value, 0, 10);
+				break;
+            case 3:
+				WIFI_En[2] = simple_strtol(value, 0, 10);
+				break;    
+            case 4:
+				WIFI_En[3] = simple_strtol(value, 0, 10);
+				break;
+			default:
+			{
+				status = FALSE;
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("%s: set wrong parameters\n", __FUNCTION__));
+				break;
+			}	
+		}
+	}
+
+	MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s: TxAntCtrlEn = %d, WIFI_En(0:1:2:3) = %d:%d:%d:%d \n", __FUNCTION__, TxAntCtrlEn, WIFI_En[0], WIFI_En[1], WIFI_En[2], WIFI_En[3]));
+	
+    return TxPowerRfTxAnt(pAd, TxAntCtrlEn, WIFI_En[0], WIFI_En[1], WIFI_En[2], WIFI_En[3]);
 }
 
 INT set_hnat_register(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
@@ -3054,3 +3191,20 @@ INT set_hnat_register(RTMP_ADAPTER *pAd, RTMP_STRING *arg)
     return TRUE;
 }
 
+INT Set_MibBucket_Proc (RTMP_ADAPTER *pAd, RTMP_STRING *arg)
+{
+    UCHAR  MibBucketEnable;
+    UCHAR       concurrent_bands = HcGetAmountOfBand(pAd);
+    UCHAR i=0;
+
+	
+
+	
+    MibBucketEnable = simple_strtol(arg, 0, 10);
+
+    //MTWF_LOG(DBG_CAT_CFG, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s()  BandIdx=%d, MibBucket Enable=%d \n", __FUNCTION__, BandIdx, MibBucketEnable));
+	for(i=0;i<concurrent_bands;i++) 
+    		pAd->OneSecMibBucket.Enabled[i] = MibBucketEnable;
+    return TRUE;
+}
+
