@@ -475,6 +475,17 @@ VOID DfsChannelSwitchTimeoutAction(
     //MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("Timeout Channel Switch\n"));
     APStop(pAd);
     APStartUp(pAd);
+
+#ifdef MT_DFS_SUPPORT
+        if (pAd->CommonCfg.dbdc_mode)
+        {
+                MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("Dot11HCntDownTimeoutAction. SetDfsTxStart\n"));
+
+                MtCmdSetDfsTxStart(pAd, BAND1);
+        }
+        else
+                MtCmdSetDfsTxStart(pAd, BAND0);
+#endif	
 }
 
 VOID DfsCacNormalStart(
