@@ -735,6 +735,7 @@ BOOLEAN PeerBeaconAndProbeRspSanity(
 		case IE_VHT_OP:
 			if (pEid->Len == sizeof(VHT_OP_IE)) {
 				NdisMoveMemory(&ie_list->vht_op_ie, &pEid->Octet[0], sizeof(VHT_OP_IE));
+				//hex_dump ("recv. vht op", (UCHAR *)&ie_list->vht_op_ie, sizeof(VHT_OP_IE));
 				ie_list->vht_op_len = pEid->Len;
 			}
 			break;
@@ -788,7 +789,7 @@ BOOLEAN PeerBeaconAndProbeRspSanity(
 #ifdef CONFIG_MULTI_CHANNEL
 	if (((Sanity & 0x4) == 0))
 #else /* CONFIG_MULTI_CHANNEL */
-	if ((pAd->LatchRfRegs.Channel > 14) && ((Sanity & 0x4) == 0))
+	if ((LatchRfChannel > 14) && ((Sanity & 0x4) == 0))
 #endif /* !CONFIG_MULTI_CHANNEL */
 	{
 		if (CtrlChannel != 0)
