@@ -795,8 +795,14 @@ INT RtmpChipOpsEepromHook(RTMP_ADAPTER *pAd, INT infType,INT forceMode)
 #ifdef CONFIG_RT_SECOND_CARD
 			if ( pAd->dev_idx == 1 )
 				pAd->flash_offset = CONFIG_RT_SECOND_IF_RF_OFFSET;
-#endif /* CONFIG_RT_FIRST_CARD */
-			MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("NVM is FLASH mode\n"));
+#endif /* CONFIG_RT_SECOND_CARD */
+#ifdef CONFIG_RT_THIRD_CARD
+			if ( pAd->dev_idx == 2 )
+				pAd->flash_offset = CONFIG_RT_THIRD_IF_RF_OFFSET;
+#endif /* CONFIG_RT_THIRD_CARD */
+
+			MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_OFF, 
+				("NVM is FLASH mode. dev_idx [%d] FLASH OFFSET [0x%X]\n",pAd->dev_idx,pAd->flash_offset));
 			return 0;
 		}
 #endif /* RTMP_FLASH_SUPPORT */
