@@ -299,12 +299,28 @@ INT NfcBuildOOBDevPasswdTLV(
 	for (idx = 0; idx < 192; idx++)
 		pWscCtrl->RegData.EnrolleeRandom[idx] = RandomByte(pAd);
 
+    NdisZeroMemory(pWscCtrl->RegData.Pke, sizeof(pWscCtrl->RegData.Pke));
 	RT_DH_PublicKey_Generate (
         WPS_DH_G_VALUE, sizeof(WPS_DH_G_VALUE),
 	    WPS_DH_P_VALUE, sizeof(WPS_DH_P_VALUE),
 	    pWscCtrl->RegData.EnrolleeRandom, sizeof(pWscCtrl->RegData.EnrolleeRandom),
 	    pWscCtrl->RegData.Pke, (UINT *) &DH_Len);
 
+    /* Need to prefix zero padding */
+    if((DH_Len != sizeof(pWscCtrl->RegData.Pke)) &&
+        (DH_Len < sizeof(pWscCtrl->RegData.Pke)))
+    {
+        UCHAR TempKey[192];
+        INT DiffCnt;
+        DiffCnt = sizeof(pWscCtrl->RegData.Pke) - DH_Len;
+
+        NdisFillMemory(&TempKey, DiffCnt, 0);
+        NdisCopyMemory(&TempKey[DiffCnt], pWscCtrl->RegData.Pke, DH_Len);
+        NdisCopyMemory(pWscCtrl->RegData.Pke, TempKey, sizeof(TempKey));
+        DH_Len += DiffCnt;
+        MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("%s: Do zero padding!\n", __FUNCTION__));
+    }
+
 	/* For Handover case, We may as Registrar 
 		So keep the same public key for Registrar */
 	RTMPMoveMemory(pWscCtrl->RegData.Pkr, pWscCtrl->RegData.Pke, DH_Len);
@@ -542,12 +558,28 @@ static BOOLEAN	NfcProcessPasswdTV(
 		for (idx = 0; idx < 192; idx++)
 			pWscCtrl->RegData.EnrolleeRandom[idx] = RandomByte(pAdapter);
 
+        NdisZeroMemory(pWscCtrl->RegData.Pkr, sizeof(pWscCtrl->RegData.Pkr));
 		RT_DH_PublicKey_Generate (
 			WPS_DH_G_VALUE, sizeof(WPS_DH_G_VALUE),
 			WPS_DH_P_VALUE, sizeof(WPS_DH_P_VALUE),
 			pWscCtrl->RegData.EnrolleeRandom, sizeof(pWscCtrl->RegData.EnrolleeRandom),
 			pWscCtrl->RegData.Pkr, (UINT *) &DH_Len);
 
+        /* Need to prefix zero padding */
+        if((DH_Len != sizeof(pWscCtrl->RegData.Pkr)) &&
+            (DH_Len < sizeof(pWscCtrl->RegData.Pkr)))
+        {
+            UCHAR TempKey[192];
+            INT DiffCnt;
+            DiffCnt = sizeof(pWscCtrl->RegData.Pkr) - DH_Len;
+
+            NdisFillMemory(&TempKey, DiffCnt, 0);
+            NdisCopyMemory(&TempKey[DiffCnt], pWscCtrl->RegData.Pkr, DH_Len);
+            NdisCopyMemory(pWscCtrl->RegData.Pkr, TempKey, sizeof(TempKey));
+            DH_Len += DiffCnt;
+            MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("%s: Do zero padding!\n", __FUNCTION__));
+        }
+
 		hex_dump("Pkr", pWscCtrl->RegData.Pkr, 192);
 	}
 
