@@ -885,6 +885,7 @@ VOID RRM_EnqueueLinkMeasureReq(
 	NDIS_STATUS NStatus;
 	ULONG FrameLen;
 	PMAC_TABLE_ENTRY pEntry;
+	struct wifi_dev *wdev;
 
 	if ((apidx >= pAd->ApCfg.BssidNum)
 		|| (!(VALID_UCAST_ENTRY_WCID(pAd, Aid))))
@@ -902,6 +903,7 @@ VOID RRM_EnqueueLinkMeasureReq(
 	}
 
 	pEntry = &pAd->MacTab.Content[Aid];
+	wdev = pEntry->wdev;
 
 	/* build action frame header. */
 	MgtMacHeaderInit(pAd, &ActHdr, SUBTYPE_ACTION, 0, pEntry->Addr,
@@ -921,7 +923,7 @@ VOID RRM_EnqueueLinkMeasureReq(
 	/* fill Tx Power Used field */
 	{
 		ULONG TempLen;
-		UINT8 TxPwr = RTMP_GetTxPwr(pAd, pAd->CommonCfg.MlmeTransmit, pEntry->wdev->channel);
+		UINT8 TxPwr = RTMP_GetTxPwr(pAd, wdev->rate.MlmeTransmit, wdev->channel);
 		MakeOutgoingFrame(pOutBuffer + FrameLen,	&TempLen,
 						1,							&TxPwr,
 						END_OF_ARGS);
@@ -933,7 +935,7 @@ VOID RRM_EnqueueLinkMeasureReq(
 	/* fill Max Tx Power field */
 	{
 		ULONG TempLen;
-		UINT8 MaxTxPwr = GetCuntryMaxTxPwr(pAd,pEntry->wdev->PhyMode, pEntry->wdev->channel);
+		UINT8 MaxTxPwr = GetCuntryMaxTxPwr(pAd,wdev->PhyMode, wdev->channel);
 		MakeOutgoingFrame(pOutBuffer + FrameLen,	&TempLen,
 						1,							&MaxTxPwr,
 						END_OF_ARGS);
