@@ -1653,21 +1653,7 @@ static UCHAR FillChList(
 		}
 
 #endif /* CONFIG_AP_SUPPORT */
-		for (l=0; l<MAX_NUM_OF_CHANNELS; l++)
-		{
-			if (channel == pAd->TxPower[l].Channel)
-			{
-				pAd->ChannelList[j].Power = pAd->TxPower[l].Power;
-				pAd->ChannelList[j].Power2 = pAd->TxPower[l].Power2;
-#ifdef DOT11N_SS3_SUPPORT
-					pAd->ChannelList[j].Power3 = pAd->TxPower[l].Power3;
-#endif /* DOT11N_SS3_SUPPORT */
-				break;
-			}
-		}
-		if (l == MAX_NUM_OF_CHANNELS)
-			continue;
-
+//sachin - TODO
 		pAd->ChannelList[j].Channel = pChDesp->FirstChannel + i * increment;
 		pAd->ChannelList[j].MaxTxPwr = pChDesp->MaxTxPwr;
 		pAd->ChannelList[j].DfsReq = pChDesp->DfsReq;
@@ -2092,7 +2078,6 @@ BOOLEAN N_ChannelGroupCheck(
 VOID N_ChannelCheck(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 {
 	INT idx;
-	
 	static const UCHAR wfa_ht_ch_ext[] = {
 			36, EXTCHA_ABOVE, 40, EXTCHA_BELOW,
 			44, EXTCHA_ABOVE, 48, EXTCHA_BELOW,
@@ -2116,7 +2101,7 @@ VOID N_ChannelCheck(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 			idx = 0;
 			while(wfa_ht_ch_ext[idx] != 0) {
 				if (wfa_ht_ch_ext[idx] == Channel && IsValidChannel(pAd, GetExtCh(Channel, wfa_ht_ch_ext[idx + 1]))) {
-					pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = wfa_ht_ch_ext[idx + 1];
+					 HcUpdateExtCha(pAd,Channel,wfa_ht_ch_ext[idx + 1]);
 					break;
 				}
 				idx += 2;
@@ -2124,7 +2109,7 @@ VOID N_ChannelCheck(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 			if (wfa_ht_ch_ext[idx] == 0)
 			{
 				pAd->CommonCfg.RegTransmitSetting.field.BW  = BW_20;
-				pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_NONE;
+				HcUpdateExtCha(pAd,Channel,EXTCHA_NONE);
 			}
 		}
 		else
@@ -2132,16 +2117,20 @@ VOID N_ChannelCheck(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 			do
 			{
 				UCHAR ExtCh;
-				UCHAR Dir = pAd->CommonCfg.RegTransmitSetting.field.EXTCHA;
+				UCHAR Dir = HcGetExtCha(pAd,Channel);
+				if(Dir == EXTCHA_NONE){
+					Dir = EXTCHA_ABOVE;
+				}
 				ExtCh = GetExtCh(Channel, Dir);
-				if (IsValidChannel(pAd, ExtCh))
+				if (IsValidChannel(pAd, ExtCh)){
+					HcUpdateExtCha(pAd,Channel,Dir);
 					break;
-
+				}
 				Dir = (Dir == EXTCHA_ABOVE) ? EXTCHA_BELOW : EXTCHA_ABOVE;
 				ExtCh = GetExtCh(Channel, Dir);
 				if (IsValidChannel(pAd, ExtCh))
 				{
-					pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = Dir;
+					HcUpdateExtCha(pAd,Channel,Dir);
 					break;
 				}
 				pAd->CommonCfg.RegTransmitSetting.field.BW  = BW_20;
@@ -2157,7 +2146,7 @@ VOID N_ChannelCheck(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 	if (WMODE_CAP_N(PhyMode) &&
 		(pAd->CommonCfg.RegTransmitSetting.field.BW  == BW_20))
 	{
-		pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_NONE;
+		HcUpdateExtCha(pAd,Channel,EXTCHA_NONE);
 	}
 }
 
@@ -2166,7 +2155,7 @@ UCHAR N_SetCenCh(RTMP_ADAPTER *pAd, UCHAR prim_ch)
 {
 	if (pAd->CommonCfg.RegTransmitSetting.field.BW == BW_40)
 	{
-		if (pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+		if (HcGetExtCha(pAd,prim_ch) == EXTCHA_ABOVE)
 			pAd->CommonCfg.CentralChannel = prim_ch + 2;
 		else
 		{
