@@ -1107,11 +1107,6 @@ VOID UserCfgInit(RTMP_ADAPTER *pAd)
 #endif /* DOT11_N_SUPPORT */
 
 	pAd->CommonCfg.TxRate = RATE_6;
-
-	pAd->CommonCfg.MlmeTransmit.field.MCS = MCS_RATE_6;
-	pAd->CommonCfg.MlmeTransmit.field.BW = BW_20;
-	pAd->CommonCfg.MlmeTransmit.field.MODE = MODE_OFDM;
-
 	pAd->CommonCfg.BeaconPeriod = 100;     /* in mSec*/
 
 #ifdef STREAM_MODE_SUPPORT
@@ -1497,7 +1492,7 @@ VOID UserCfgInit(RTMP_ADAPTER *pAd)
 	pAd->WOW_Cfg.nSelectedGPIO = 2;
 	pAd->WOW_Cfg.nDelay = 3; /* (3+1)*3 = 12 sec */
 	pAd->WOW_Cfg.nHoldTime = 1000;	// unit is us
-	pAd->WOW_Cfg.nWakeupInterface = WOW_WAKEUP_BY_USB; /* USB as default */
+	pAd->WOW_Cfg.nWakeupInterface = pAd->chipCap.nWakeupInterface; //WOW_WAKEUP_BY_USB; /* USB as default */
 	pAd->WOW_Cfg.bGPIOHighLow = WOW_GPIO_LOW_TO_HIGH;
 	MTWF_LOG(DBG_CAT_INIT, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("WOW Enable %d, WOWFirmware %d\n", pAd->WOW_Cfg.bEnable, pAd->WOW_Cfg.bWOWFirmware));
 #endif /* (defined(WOW_SUPPORT) && defined(RTMP_MAC_USB)) || defined(NEW_WOW_SUPPORT) || defined(MT_WOW_SUPPORT) */
@@ -1773,11 +1768,19 @@ VOID UserCfgInit(RTMP_ADAPTER *pAd)
         pAd->SCSCtrl.SCSStatus[i] = PD_BLOCKING_OFF;
         pAd->SCSCtrl.OneSecTxByteCount[i] = 0;
         pAd->SCSCtrl.OneSecRxByteCount[i] = 0;
-        pAd->SCSCtrl.PdBlkTh[i] = 0;
+	pAd->SCSCtrl.CckPdBlkTh[i] = PdBlkCckThDefault;
+	pAd->SCSCtrl.OfdmPdBlkTh[i] = PdBlkOfmdThDefault;	
 		pAd->SCSCtrl.SCSThTolerance[i] = ThTolerance;
         pAd->SCSCtrl.SCSMinRssiTolerance[i] = MinRssiTolerance;
         pAd->SCSCtrl.OfdmPdSupport[i] = TRUE;        
+	pAd->SCSCtrl.CckFalseCcaUpBond[i] = FalseCcaUpBondDefault;
+	pAd->SCSCtrl.CckFalseCcaLowBond[i] = FalseCcaLowBondDefault;
+	pAd->SCSCtrl.OfdmFalseCcaUpBond[i] = FalseCcaUpBondDefault;
+	pAd->SCSCtrl.OfdmFalseCcaLowBond[i] = FalseCcaLowBondDefault;
+	pAd->SCSCtrl.CckFixedRssiBond[i] = CckFixedRssiBondDefault;
+	pAd->SCSCtrl.OfdmFixedRssiBond[i] = OfdmFixedRssiBondDefault;	
     }    
+    //pAd->SCSCtrl.SCSEnable[0] = SCS_ENABLE;
 #endif /* SMART_CARRIER_SENSE_SUPPORT */
 
         for (i=0; i<DBDC_BAND_NUM; i++)
@@ -1787,6 +1790,8 @@ VOID UserCfgInit(RTMP_ADAPTER *pAd)
         pAd->g_mode_txop_wdev = NULL;
         pAd->G_MODE_INFRA_TXOP_RUNNING = FALSE;
 
+        pAd->MUMIMO_TxOP_Value = 0;
+
 	MTWF_LOG(DBG_CAT_INIT, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("<-- UserCfgInit\n"));
 }
 
