@@ -245,6 +245,7 @@ int mt_wifi_init(VOID *pAdSrc, RTMP_STRING *pDefaultMac, RTMP_STRING *pHostName)
 #ifdef SMART_CARRIER_SENSE_SUPPORT
         UINT32  CrValue;
 #endif /* SMART_CARRIER_SENSE_SUPPORT */
+	UCHAR	i=0;
 	RTMP_ADAPTER *pAd = (RTMP_ADAPTER *)pAdSrc;
 	NDIS_STATUS Status;
 
@@ -333,16 +334,10 @@ int mt_wifi_init(VOID *pAdSrc, RTMP_STRING *pDefaultMac, RTMP_STRING *pHostName)
 	rtmp_hif_data_init(pAd);
 
 #ifdef SMART_CARRIER_SENSE_SUPPORT
-	/* Enable band0 IPI SW control */
-        HW_IO_READ32(pAd, PHY_RXTD_12, &CrValue);
-        CrValue |= (1 << 18);
-        CrValue |= (1 << 29);
-        HW_IO_WRITE32(pAd, PHY_RXTD_12, CrValue);
-        HW_IO_WRITE32(pAd, PHY_RXTD_12, CrValue);
-        /* Enable badn0 IPI control */
-        HW_IO_READ32(pAd, PHY_BAND0_PHYMUX_5, &CrValue);
-        CrValue |= (0x5 << 12);
-        HW_IO_WRITE32(pAd, PHY_BAND0_PHYMUX_5, CrValue);  
+	/* Enable PD_BLOCKING */
+	HW_IO_READ32(pAd, PHY_MIN_PRI_PWR, &CrValue);
+	CrValue |= (0x1 << PdBlkEnabeOffset); /* Bit[19] */
+	HW_IO_WRITE32(pAd, PHY_MIN_PRI_PWR, CrValue);
 #endif /* SMART_CARRIER_SENSE_SUPPORT */
 
 #ifdef CONFIG_AP_SUPPORT
@@ -410,6 +405,13 @@ int mt_wifi_init(VOID *pAdSrc, RTMP_STRING *pDefaultMac, RTMP_STRING *pHostName)
 	BackgroundScanInit(pAd);
 #endif /* BACKGROUND_SCAN_SUPPORT */
 
+#ifdef CONFIG_AP_SUPPORT
+    AutoChSelInit(pAd);
+#endif /* CONFIG_AP_SUPPORT */
+    
+	/* Trigger MIB counter update */
+	for (i=0; i<DBDC_BAND_NUM; i++)
+		pAd->OneSecMibBucket.Enabled[i] = TRUE;
 
 	MTWF_LOG(DBG_CAT_INIT, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("<==== mt_wifi_init, Status=%x\n", Status));
 
@@ -417,9 +419,13 @@ int mt_wifi_init(VOID *pAdSrc, RTMP_STRING *pDefaultMac, RTMP_STRING *pHostName)
     mt_Trigger_Sounding_Packet(pAd,
                                TRUE,
 	                              0,
-	                              4,
+	                  BF_PROCESSING,
 	                              0,
 	                           NULL);
+
+    AsicTxBfHwEnStatusUpdate(pAd,
+                             pAd->CommonCfg.ETxBfEnCond,
+                             pAd->CommonCfg.RegTransmitSetting.field.ITxBfEn);
 #endif /* TXBF_SUPPORT */
 
 	return TRUE;
@@ -470,47 +476,6 @@ VOID RTMPDrvOpen(VOID *pAdSrc)
 
 #ifdef WSC_INCLUDED
 #ifdef CONFIG_AP_SUPPORT
-	if ((pAd->OpMode == OPMODE_AP)
-		)
-	{
-		INT index;
-		for (index = 0; index < pAd->ApCfg.BssidNum; index++)
-		{
-#ifdef HOSTAPD_SUPPORT
-			if (pAd->ApCfg.MBSSID[index].Hostapd == Hostapd_EXT)
-			{
-				MTWF_LOG(DBG_CAT_INIT, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("WPS is control by hostapd now.\n"));
-			}
-			else
-#endif /*HOSTAPD_SUPPORT*/
-			{
-				PWSC_CTRL pWscControl;
-				UCHAR zeros16[16]= {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
-
-				pWscControl = &pAd->ApCfg.MBSSID[index].WscControl;
-				MTWF_LOG(DBG_CAT_INIT, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("Generate UUID for apidx(%d)\n", index));
-				if (NdisEqualMemory(&pWscControl->Wsc_Uuid_E[0], zeros16, UUID_LEN_HEX))
-					WscGenerateUUID(pAd, &pWscControl->Wsc_Uuid_E[0], &pWscControl->Wsc_Uuid_Str[0], index, FALSE);
-				WscInit(pAd, FALSE, index);
-			}
-		}
-
-#ifdef APCLI_SUPPORT
-		for(index = 0; index < MAX_APCLI_NUM; index++)
-		{
-			PWSC_CTRL pWpsCtrl = &pAd->ApCfg.ApCliTab[index].WscControl;
-
-			pWpsCtrl->pAd = pAd;
-			NdisZeroMemory(pWpsCtrl->EntryAddr, MAC_ADDR_LEN);
-#ifdef WSC_V2_SUPPORT
-			pWpsCtrl->WscConfigMethods= 0x238C;
-#else /* WSC_V2_SUPPORT */
-			pWpsCtrl->WscConfigMethods= 0x018C;
-#endif /* !WSC_V2_SUPPORT */
-			RTMP_AP_IoctlHandle(pAd, NULL, CMD_RTPRIV_IOCTL_WSC_INIT, 0, (VOID *)&pAd->ApCfg.ApCliTab[index], index);
-		}
-#endif /* APCLI_SUPPORT */
-	}
 #endif /* CONFIG_AP_SUPPORT */
 
 
@@ -535,6 +500,14 @@ VOID RTMPDrvOpen(VOID *pAdSrc)
 #endif /* CONFIG_AP_SUPPORT */
 #endif /* VOW_SUPPORT */
 
+#ifdef BACKGROUND_SCAN_SUPPORT
+	if (IS_MT7615(pAd)) {
+		MTWF_LOG(DBG_CAT_INIT, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("Trigger DFS Zero wait procedure Support=%d, DfsZeroWaitChannel=%d", pAd->BgndScanCtrl.DfsZeroWaitSupport, pAd->BgndScanCtrl.DfsZeroWaitChannel));
+		if (pAd->BgndScanCtrl.DfsZeroWaitSupport == 1 && pAd->BgndScanCtrl.DfsZeroWaitChannel !=0)
+			DfsZeroWaitStart(pAd, TRUE);
+	}
+#endif /* BACKGROUND_SCAN_SUPPORT */
+
 }
 
 
@@ -745,6 +718,10 @@ VOID RTMPDrvClose(VOID *pAdSrc, VOID *net_dev)
     BackgroundScanDeInit(pAd); 
 #endif /* BACKGROUND_SCAN_SUPPORT */
 
+#ifdef CONFIG_AP_SUPPORT
+    AutoChSelRelease(pAd);    
+#endif/* CONFIG_AP_SUPPORT */
+
 	RTMP_CLEAR_FLAG(pAd, fRTMP_ADAPTER_START_UP);
 
 /*+++Modify by woody to solve the bulk fail+++*/
