@@ -29,7 +29,7 @@
 #ifdef SCAN_SUPPORT
 
 
-static INT scan_ch_restore(RTMP_ADAPTER *pAd, UCHAR OpMode, struct wifi_dev *pwdev)
+INT scan_ch_restore(RTMP_ADAPTER *pAd, UCHAR OpMode, struct wifi_dev *pwdev)
 {
 	INT bw, ch;
 	UCHAR BandIdx;	
@@ -236,6 +236,9 @@ static INT scan_active(RTMP_ADAPTER *pAd, UCHAR OpMode, UCHAR ScanType,struct wi
 	HEADER_802_11 Hdr80211;
 	ULONG FrameLen = 0;
 	UCHAR SsidLen = 0;
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+    UCHAR ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */	
 
 
 	if (MlmeAllocateMemory(pAd, &frm_buf) != NDIS_STATUS_SUCCESS)
@@ -260,7 +263,6 @@ static INT scan_active(RTMP_ADAPTER *pAd, UCHAR OpMode, UCHAR ScanType,struct wi
 	
 	/* There is no need to send broadcast probe request if active scan is in effect.*/
 	SsidLen = 0;
-#ifndef APCLI_CONNECTION_TRIAL
 	if ((ScanType == SCAN_ACTIVE) || (ScanType == FAST_SCAN_ACTIVE)
 #ifdef WSC_STA_SUPPORT
 		|| ((ScanType == SCAN_WSC_ACTIVE) && (OpMode == OPMODE_STA))
@@ -270,7 +272,6 @@ static INT scan_active(RTMP_ADAPTER *pAd, UCHAR OpMode, UCHAR ScanType,struct wi
 #endif /* RT_CFG80211_P2P_SUPPORT */
 		)
 		SsidLen = pAd->ScanCtrl.SsidLen;
-#endif /* APCLI_CONNECTION_TRIAL */
 
 #ifdef RT_CFG80211_P2P_SUPPORT
     if (ScanType == SCAN_P2P)
@@ -337,17 +338,17 @@ static INT scan_active(RTMP_ADAPTER *pAd, UCHAR OpMode, UCHAR ScanType,struct wi
 						  1,                        &SsidLen,
 						  SsidLen,			        pAd->ScanCtrl.Ssid,
 						  1,                        &SupRateIe,
-						  1,                        &pAd->CommonCfg.SupRateLen,
-						  pAd->CommonCfg.SupRateLen,  pAd->CommonCfg.SupRate, 
+						  1,                        &wdev->rate.SupRateLen,
+						  wdev->rate.SupRateLen,  wdev->rate.SupRate, 
 						  END_OF_ARGS);
 
-		if (pAd->CommonCfg.ExtRateLen)
+		if (wdev->rate.ExtRateLen)
 		{
 			ULONG Tmp;
 			MakeOutgoingFrame(frm_buf + FrameLen,            &Tmp,
 							  1,                                &ExtRateIe,
-							  1,                                &pAd->CommonCfg.ExtRateLen,
-							  pAd->CommonCfg.ExtRateLen,          pAd->CommonCfg.ExtRate, 
+							  1,                                &wdev->rate.ExtRateLen,
+							  wdev->rate.ExtRateLen,          wdev->rate.ExtRate, 
 							  END_OF_ARGS);
 			FrameLen += Tmp;
 		}
@@ -462,7 +463,19 @@ static INT scan_active(RTMP_ADAPTER *pAd, UCHAR OpMode, UCHAR ScanType,struct wi
         vht_ie_info.channel = pAd->ScanCtrl.Channel;
         vht_ie_info.phy_mode = wdev->PhyMode;
 
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+        ucETxBfCap = pAd->CommonCfg.ETxBfEnCond;
+        if (HcIsBfCapSupport(wdev) == FALSE)
+        {
+            pAd->CommonCfg.ETxBfEnCond = 0;
+        }
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
+
         FrameLen += build_vht_ies(pAd, &vht_ie_info);
+
+#if defined(TXBF_SUPPORT) && defined(VHT_TXBF_SUPPORT)
+		pAd->CommonCfg.ETxBfEnCond = ucETxBfCap;
+#endif /* TXBF_SUPPORT && VHT_TXBF_SUPPORT */ 
 #endif /* DOT11_VHT_AC */
 	}
 #endif /* DOT11_N_SUPPORT */
