@@ -598,14 +598,26 @@ INT build_vht_op_ie(RTMP_ADAPTER *pAd, UCHAR bw, UCHAR Channel, UCHAR *buf)
 			vht_op.vht_op_info.center_freq_2 = 0;
 			break;
 		case VHT_BW_160:
+#ifdef DOT11_VHT_R2
+			vht_op.vht_op_info.ch_width = 1;
+			vht_op.vht_op_info.center_freq_1 = (cent_ch - 8);
+			vht_op.vht_op_info.center_freq_2 = cent_ch;
+#else
 			vht_op.vht_op_info.ch_width = 2;
 			vht_op.vht_op_info.center_freq_1 = cent_ch;
 			vht_op.vht_op_info.center_freq_2 = 0;
+#endif /* DOT11_VHT_R2 */
 			break;
 		case VHT_BW_8080:
+#ifdef DOT11_VHT_R2 
+			vht_op.vht_op_info.ch_width = 1;
+			vht_op.vht_op_info.center_freq_1 = cent_ch;
+			vht_op.vht_op_info.center_freq_2 = pAd->CommonCfg.vht_cent_ch2;
+#else
 			vht_op.vht_op_info.ch_width = 3;
 			vht_op.vht_op_info.center_freq_1 = cent_ch;
 			vht_op.vht_op_info.center_freq_2 = pAd->CommonCfg.vht_cent_ch2;
+#endif /* DOT11_VHT_R2 */
 			break;
 	}
 
@@ -662,6 +674,8 @@ INT build_vht_op_ie(RTMP_ADAPTER *pAd, UCHAR bw, UCHAR Channel, UCHAR *buf)
 	NdisCopyMemory(&vht_op.basic_mcs_set,&tmp, sizeof(VHT_MCS_MAP));
 #endif /* RT_BIG_ENDIAN */
 	NdisMoveMemory((UCHAR *)buf, (UCHAR *)&vht_op, sizeof(VHT_OP_IE));
+
+	//hex_dump("vht op info", (UCHAR *)&vht_op, sizeof(VHT_OP_IE));
 	
 	return sizeof(VHT_OP_IE);
 }
@@ -736,13 +750,19 @@ INT build_vht_cap_ie(RTMP_ADAPTER *pAd, UCHAR *buf)
 #ifdef VHT_TXBF_SUPPORT
 	if (pAd->chipCap.FlgHwTxBfCap) 
 	{
-		vht_cap_ie.vht_cap.num_snd_dimension = pAd->CommonCfg.vht_cap_ie.vht_cap.num_snd_dimension;
-   		vht_cap_ie.vht_cap.bfee_sts_cap      = pAd->CommonCfg.vht_cap_ie.vht_cap.bfee_sts_cap;
-		vht_cap_ie.vht_cap.bfee_cap_su       = pAd->CommonCfg.vht_cap_ie.vht_cap.bfee_cap_su;
-		vht_cap_ie.vht_cap.bfer_cap_su       = pAd->CommonCfg.vht_cap_ie.vht_cap.bfer_cap_su;
+	    VHT_CAP_INFO vht_cap;
+
+	    NdisCopyMemory(&vht_cap, &pAd->CommonCfg.vht_cap_ie.vht_cap, sizeof(VHT_CAP_INFO));
+	    
+	    mt_WrapSetVHTETxBFCap(pAd, &vht_cap);
+	    
+		vht_cap_ie.vht_cap.num_snd_dimension = vht_cap.num_snd_dimension;
+   		vht_cap_ie.vht_cap.bfee_sts_cap      = vht_cap.bfee_sts_cap;
+		vht_cap_ie.vht_cap.bfee_cap_su       = vht_cap.bfee_cap_su;
+		vht_cap_ie.vht_cap.bfer_cap_su       = vht_cap.bfer_cap_su;
 #ifdef MT_MAC
-		vht_cap_ie.vht_cap.bfee_cap_mu       = pAd->CommonCfg.vht_cap_ie.vht_cap.bfee_cap_mu;
-		vht_cap_ie.vht_cap.bfer_cap_mu       = pAd->CommonCfg.vht_cap_ie.vht_cap.bfer_cap_mu;
+		vht_cap_ie.vht_cap.bfee_cap_mu       = vht_cap.bfee_cap_mu;
+		vht_cap_ie.vht_cap.bfer_cap_mu       = vht_cap.bfer_cap_mu;
 #endif
 	} 
 #endif
@@ -903,6 +923,97 @@ INT build_vht_ies(RTMP_ADAPTER *pAd, struct _build_ie_info *info)
     return len;
 }
 
+
+static long ch_offset_abs(long x) {
+	long y;
+	y = x >> 31;
+	return (x ^ y) - y;
+}
+
+UCHAR check_vht_op_bw (VHT_OP_INFO *vht_op_info)
+{
+	UCHAR bw = 0;
+	UINT8 p80ccf = vht_op_info->center_freq_1;
+	UINT8 s80160ccf = vht_op_info->center_freq_2;
+
+	switch (vht_op_info->ch_width) {
+		case VHT_BW_2040:
+			bw = VHT_BW_2040;
+			break;
+		case VHT_BW_80:
+			if (s80160ccf == 0) {
+				bw = VHT_BW_80;
+			} else if (ch_offset_abs(s80160ccf-p80ccf) == 8) {
+				bw = VHT_BW_160;
+			} else if (ch_offset_abs(s80160ccf-p80ccf) > 16) {
+				bw = VHT_BW_8080;
+			}
+			break;
+		case VHT_BW_160:
+			bw = VHT_BW_160;
+			break;
+		case VHT_BW_8080:
+			bw = VHT_BW_8080;
+			break;
+		default:
+			break;
+	}
+
+	return bw;
+}
+
+void update_vht_op_info(UINT8 cap_bw, VHT_OP_INFO *vht_op_info, struct _op_info *op_info)
+{
+	UINT8 p80ccf = vht_op_info->center_freq_1;
+	UINT8 s80160ccf = vht_op_info->center_freq_2;
+
+	if (op_info == NULL)
+		return;
+
+	op_info->bw = vht_op_info->ch_width;
+
+	switch (vht_op_info->ch_width) {
+		case VHT_BW_2040:
+			break;
+		case VHT_BW_80:
+			if (cap_bw == VHT_BW_80) {
+				op_info->cent_ch = p80ccf;
+			} else if (cap_bw > VHT_BW_80) {
+				if (s80160ccf == 0) {
+					op_info->cent_ch = p80ccf;
+				} else if (ch_offset_abs(s80160ccf-p80ccf) == 8) {
+					op_info->bw = VHT_BW_160;
+					op_info->cent_ch = s80160ccf;
+				} else if (ch_offset_abs(s80160ccf-p80ccf) > 16) {
+					op_info->bw = VHT_BW_8080;
+					op_info->cent_ch = p80ccf;
+					op_info->cent_ch2 = s80160ccf;
+				}
+			}
+			break;
+		case VHT_BW_160:
+			if (cap_bw == VHT_BW_80) {
+				op_info->bw = VHT_BW_80;
+			}
+			op_info->cent_ch = p80ccf;
+			break;
+		case VHT_BW_8080:
+			if (cap_bw == VHT_BW_80) {
+				op_info->bw = VHT_BW_80;
+			}
+			op_info->cent_ch = p80ccf;
+			op_info->cent_ch2 = s80160ccf;
+			break;
+		default:
+			op_info->bw = VHT_BW_80;
+			op_info->cent_ch = p80ccf;
+			break;
+	}
+	printk("%s, bw=%u, cent_ch=%u\n", __FUNCTION__, op_info->bw, op_info->cent_ch);
+
+	return ;
+}	
+
 BOOLEAN vht80_channel_group( RTMP_ADAPTER *pAd, UCHAR channel)
 {
 	INT idx = 0;
