@@ -391,12 +391,16 @@ VOID WifiSysUpdatePortSecur(RTMP_ADAPTER *pAd,MAC_TABLE_ENTRY *pEntry)
 		sta_ctrl->ConnectionType = pEntry->ConnectionType;
 		sta_ctrl->EnableFeature = STA_REC_BASIC_STA_RECORD_FEATURE;
 		sta_ctrl->WlanIdx = pEntry->wcid;
+		sta_ctrl->IsNewSTARec = FALSE;
 
 		WifiSysUpdateStaRec(pAd,tr_entry,&wifi_sys_ctrl.StaRecCtrl);
 
 		wifi_sys_ctrl.wdev = pEntry->wdev;
 
 		HW_WIFISYS_PEER_UPDATE(pAd,wifi_sys_ctrl);
+#ifdef CONFIG_AP_SUPPORT
+		CheckBMCPortSecured(pAd, pEntry, TRUE);
+#endif /* CONFIG_AP_SUPPORT */
 	}
 }
 
@@ -428,6 +432,11 @@ VOID WifiSysPeerLinkDown(RTMP_ADAPTER *pAd,MAC_TABLE_ENTRY *pEntry)
 	
 	wifi_sys_ctrl.wdev = wdev;
 	HW_WIFISYS_PEER_LINKDOWN(pAd,wifi_sys_ctrl);
+
+#ifdef CONFIG_AP_SUPPORT
+	CheckBMCPortSecured(pAd, pEntry, FALSE);
+#endif /* CONFIG_AP_SUPPORT */
+
 }
 
 
@@ -526,8 +535,8 @@ VOID WifiSysApLinkUp(RTMP_ADAPTER *pAd,struct wifi_dev *wdev)
 		tr_entry = &pAd->MacTab.tr_entry[wdev->tr_tb_idx];
 		sta_rec = &tr_entry->StaRec;
 
-		
-		StaState = wdev->PortSecured ? STATE_PORT_SECURE: STATE_CONNECTED;
+		/* BC sta record should always set STATE_PORT_SECURE*/
+		StaState = STATE_PORT_SECURE;
 
 		if (pAd->chipCap.SupportAMSDU == TRUE) 
 		{
@@ -539,6 +548,7 @@ VOID WifiSysApLinkUp(RTMP_ADAPTER *pAd,struct wifi_dev *wdev)
 		sta_rec->ConnectionState = StaState;
 		sta_rec->ConnectionType = CONNECTION_INFRA_STA;
 		sta_rec->EnableFeature = enableFeature;
+		sta_rec->IsNewSTARec = TRUE;
 		WifiSysUpdateStaRec(pAd,tr_entry,&wifi_sys_ctrl.StaRecCtrl);
 		
 	    MTWF_LOG(DBG_CAT_CLIENT, CATCLIENT_APCLI, DBG_LVL_TRACE,
@@ -670,6 +680,7 @@ VOID WifiSysApPeerLinkUp(RTMP_ADAPTER *pAd,MAC_TABLE_ENTRY *pEntry, IE_LISTS *ie
 	if (tr_entry->PortSecured == WPA_802_1X_PORT_SECURED)
 	{
 		PortSecured = STATE_PORT_SECURE;
+		CheckBMCPortSecured(pAd, pEntry, TRUE);
 	}
 
 	sta_rec->BssIndex = wdev->bss_info_argument.ucBssIndex;
@@ -677,6 +688,7 @@ VOID WifiSysApPeerLinkUp(RTMP_ADAPTER *pAd,MAC_TABLE_ENTRY *pEntry, IE_LISTS *ie
 	sta_rec->ConnectionType = pEntry->ConnectionType;
 	sta_rec->ConnectionState = PortSecured;
 	sta_rec->EnableFeature = enableFeature;
+	sta_rec->IsNewSTARec = TRUE;
 
 	WifiSysUpdateStaRec(pAd,tr_entry,&wifi_sys_ctrl.StaRecCtrl);
 	
@@ -684,17 +696,17 @@ VOID WifiSysApPeerLinkUp(RTMP_ADAPTER *pAd,MAC_TABLE_ENTRY *pEntry, IE_LISTS *ie
 		("===> AsicStaRecUpdate called by (%s), wcid=%d, PortSecured=%d\n",
 		__FUNCTION__, pEntry->wcid, PortSecured));
 
-    MTWF_LOG(DBG_CAT_CLIENT, CATCLIENT_APCLI, DBG_LVL_TRACE,
+	MTWF_LOG(DBG_CAT_CLIENT, CATCLIENT_APCLI, DBG_LVL_TRACE,
 		("===> AsicStaRecUpdate called by (%s), AKM=0x%x\n", __FUNCTION__, pEntry->SecConfig.AKMMap));
 
 	os_alloc_mem(NULL,(UCHAR**)&lu_ctrl,sizeof(PEER_LINKUP_HWCTRL));
 	os_zero_mem(lu_ctrl,sizeof(PEER_LINKUP_HWCTRL));
  
 #ifdef DOT11_N_SUPPORT
-    if (CLIENT_STATUS_TEST_FLAG(pEntry, fCLIENT_STATUS_RDG_CAPABLE))
-    {
-    	lu_ctrl->bRdgCap = TRUE;
-    }
+	if (CLIENT_STATUS_TEST_FLAG(pEntry, fCLIENT_STATUS_RDG_CAPABLE))
+	{
+		lu_ctrl->bRdgCap = TRUE;
+	}
 #endif /*DOT11_N_SUPPORT*/
 
 #ifdef TXBF_SUPPORT	
@@ -764,6 +776,7 @@ VOID WifiSysWdsLinkUp(RTMP_ADAPTER *pAd,struct wifi_dev *wdev, UCHAR wcid)
 	sta_rec->ConnectionType = CONNECTION_WDS;
 	sta_rec->ConnectionState = STATE_PORT_SECURE;
 	sta_rec->EnableFeature = enableFeature;
+	sta_rec->IsNewSTARec = TRUE;
 	
 	WifiSysUpdateStaRec(pAd,tr_entry,&wifi_sys_ctrl.StaRecCtrl);
 
@@ -892,6 +905,7 @@ VOID WifiSysApCliLinkUp(RTMP_ADAPTER *pAd,APCLI_STRUCT *pApCliEntry,UCHAR CliIdx
 		sta_ctrl->ConnectionType = CONNECTION_INFRA_AP;
 		sta_ctrl->ConnectionState = PortSecured;
 		sta_ctrl->EnableFeature = enableFeature;
+		sta_ctrl->IsNewSTARec = TRUE;
 		
 		WifiSysUpdateStaRec(pAd,tr_entry,&wifi_sys_ctrl.StaRecCtrl);
 		
@@ -961,6 +975,7 @@ VOID WifiSysApCliLinkUp(RTMP_ADAPTER *pAd,APCLI_STRUCT *pApCliEntry,UCHAR CliIdx
 		sta_ctrl->ConnectionType = CONNECTION_INFRA_AP;
 		sta_ctrl->ConnectionState = PortSecured;
 		sta_ctrl->EnableFeature = enableFeature;
+		sta_ctrl->IsNewSTARec = TRUE;
 		
 		WifiSysUpdateStaRec(pAd,tr_entry,&wifi_sys_ctrl.StaRecCtrl);
 
