@@ -290,12 +290,29 @@ static INT32 rcUpdateBandForRepeater(HD_CFG *pHdCfg,BCTRL_INFO_T *pBInfo)
 {
 	INT32 i;
 	BCTRL_ENTRY_T *pEntry;
-	for(i=0; i<pHdCfg->chipCap.MaxRepeaterNum; i++)
-	{
-		pEntry = &pBInfo->BctrlEntries[pBInfo->TotalNum];
-		/*fix to bind band 0 currently*/
-		rcFillEntry(pEntry,DBDC_TYPE_REPEATER,0,i);
-		pBInfo->TotalNum++;
+	HD_DEV *dev;
+	HD_DEV_OBJ *pObj;
+	HD_REPT_ENRTY *pReptEntry = NULL,*tmp=NULL;
+	if (pHdCfg->HwResourceCfg.concurrent_bands == 2) { /* DBDC mode */
+		for(i=0;i< pHdCfg->HwResourceCfg.concurrent_bands;i++) {
+			/* search repeater entry from all pObj */
+			dev = &pHdCfg->Hdev[i];
+			DlListForEach(pObj,&dev->DevObjList,struct _HD_DEV_OBJ,list) {
+				DlListForEachSafe(pReptEntry,tmp,&pObj->RepeaterList, struct _HD_REPT_ENRTY, list) {
+					pEntry = &pBInfo->BctrlEntries[pBInfo->TotalNum];
+					rcFillEntry(pEntry,DBDC_TYPE_REPEATER,dev->pRadioCtrl->BandIdx,pReptEntry->CliIdx);
+					pBInfo->TotalNum++;
+				}
+			}
+		}
+	} else {
+		for(i=0; i<pHdCfg->chipCap.MaxRepeaterNum; i++)
+		{
+			pEntry = &pBInfo->BctrlEntries[pBInfo->TotalNum];
+			/*always bind band 0*/
+			rcFillEntry(pEntry,DBDC_TYPE_REPEATER,0,i);
+			pBInfo->TotalNum++;
+		}
 	}
 	return 0;
 }
@@ -945,7 +962,24 @@ UCHAR RcUpdateBw(HD_DEV *pHdev,UCHAR Bw)
 	return Bw;
 }
 
+/*
+*
+*/
+INT32 RcUpdateExtCha(HD_DEV *pHdev,UCHAR ExtCha)
+{
+	RADIO_CTRL *pRadioCtrl = pHdev->pRadioCtrl;
+	pRadioCtrl->ExtCha = ExtCha;
+	return -1;
+}
 
+/*
+*
+*/
+UCHAR RcGetExtCha(HD_DEV *pHdev)
+{
+	RADIO_CTRL *pRadioCtrl = pHdev->pRadioCtrl;
+	return pRadioCtrl->ExtCha;
+}
 
 /*
 *
