@@ -446,7 +446,61 @@ INT32 HcUpdateRadio(RTMP_ADAPTER *pAd,UCHAR bw,UCHAR central_ch1,UCHAR control_c
 	return ret;
 }
 
+/*
+*
+*/
+INT32 HcUpdateExtCha(RTMP_ADAPTER *pAd,UCHAR Channel,UCHAR ExtCha)
+{
+	INT32 ret=0;
+	HD_DEV *pHdev=NULL;
+	HD_CFG *pHdCfg = (HD_CFG*)pAd->pHdevCfg;
+	INT32 i=0;
+	struct wifi_dev *wdev;
 
+	pHdev = RcGetHdevByChannel(pHdCfg,Channel);
+
+	if(!pHdev)
+	{
+		MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("%s(): Get Hdev by Channel %d faild, not support this RF\n",
+		__FUNCTION__,Channel));
+		return -1;
+	}
+
+    /*Update ExtCha*/
+	ret  = RcUpdateExtCha(pHdev,ExtCha);
+
+	if(ret < 0){
+	/*Update all wdev for this band*/
+		for(i=0;i<WDEV_NUM_MAX;i++){
+			wdev = pAd->wdev_list[i];
+			if(wdev){
+				if(wmode_band_equal(wdev->PhyMode,RcGetPhyMode(pHdev))){
+					wdev->extcha = RcGetExtCha(pHdev);
+				}
+			}
+		}
+	}
+	return ret;
+}
+
+/*
+*
+*/
+UCHAR HcGetExtCha(RTMP_ADAPTER *pAd,UCHAR Channel)
+{
+	HD_DEV *pHdev=NULL;
+	HD_CFG *pHdCfg = (HD_CFG*)pAd->pHdevCfg;
+
+	pHdev = RcGetHdevByChannel(pHdCfg,Channel);
+
+	if(!pHdev)
+	{
+		MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("%s(): Get Hdev by Channel %d faild, not support this RF\n",
+		__FUNCTION__,Channel));
+		return 0;
+	}
+	return RcGetExtCha(pHdev);
+}
 
 /*
 *
