@@ -139,6 +139,18 @@ static NTSTATUS HwCtrlSetClientMACEntry(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 }
 
 
+#ifdef TXBF_SUPPORT
+static NTSTATUS HwCtrlSetClientBfCap(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
+{
+    PMAC_TABLE_ENTRY pMacEntry;
+
+    pMacEntry = (PMAC_TABLE_ENTRY)CMDQelmt->buffer;
+    AsicUpdateClientBfCap(pAd, pMacEntry);
+    return NDIS_STATUS_SUCCESS;
+}
+#endif /* TXBF_SUPPORT */
+
+
 static NTSTATUS HwCtrlDelAsicWcid(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 {
 	RT_SET_ASIC_WCID SetAsicWcid;
@@ -466,25 +478,6 @@ static NTSTATUS HwCtrlSetTREntry(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
         return NDIS_STATUS_SUCCESS;
 }
 
-
-static NTSTATUS HwCtrlSetStaRec(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
-{
-	PMT_SET_STA_REC pSetStaRec = (PMT_SET_STA_REC)CMDQelmt->buffer;
-
-	IF_DEV_CONFIG_OPMODE_ON_AP(pAd)
-	{
-		MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("%s::CmdThread\n", __FUNCTION__));
-		AsicStaRecUpdate(pAd,
-			pSetStaRec->BssIndex,
-			pSetStaRec->WlanIdx,
-			pSetStaRec->ConnectionType,
-			pSetStaRec->ConnectionState,
-			pSetStaRec->EnableFeature);
-	}
-
-	return NDIS_STATUS_SUCCESS;
-}
-
 static NTSTATUS HwCtrlUpdateBssInfo(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 {
 	BSS_INFO_ARGUMENT_T *pBssInfoArgs = (BSS_INFO_ARGUMENT_T*)CMDQelmt->buffer;
@@ -617,24 +610,67 @@ static INT ErrRecoveryDone(ERR_RECOVERY_CTRL_T *pErrRecoveryCtl)
     return TRUE;
 }
 
-static ERR_RECOVERY_STATE ErrRecoveryCurStat(ERR_RECOVERY_CTRL_T *pErrRecoveryCtl)
+static UINT32 ErrRecoveryTimeDiff(UINT32 time1, UINT32 time2)
 {
-    if (pErrRecoveryCtl == NULL)
-        return FALSE;
+	UINT32 timeDiff = 0;
 
-    return pErrRecoveryCtl->errRecovState;
+	if (time1 > time2)
+	{
+		timeDiff = (0xFFFFFFFF - time1 + 1) + time2;
+	}
+	else
+	{
+		timeDiff = time2 - time1;
+	}
+	return timeDiff;
 }
 
+static void ErrRecoveryLogDump(RTMP_ADAPTER *pAd)
+{
+	UINT32 idx = 0;
+	UINT32 *pSerTimes = NULL;
 
-static NTSTATUS HwRecoveryFromError(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
+	if (pAd == NULL)
+	    return;
+
+	pSerTimes = &pAd->HwCtrl.ser_times[0];
+
+	for (idx = SER_TIME_ID_T0; idx < SER_TIME_ID_END; idx++)
+	{
+		MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, 
+		("%s,::E  R  , Time[%d](us)=%u\n", __FUNCTION__, idx,
+		pSerTimes[idx]));
+	}
+
+	for (idx = SER_TIME_ID_T0; idx < (SER_TIME_ID_END - 1); idx++)
+	{
+		MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, 
+		("%s,::E  R  , T%d - T%d(us)=%u\n", __FUNCTION__, 
+			idx + 1, idx, ErrRecoveryTimeDiff(pSerTimes[idx], 
+					    pSerTimes[idx + 1])));
+	}
+
+	MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, 
+	("%s,::E  R  , Total Time(us)=%u\n", __FUNCTION__, 
+	        ErrRecoveryTimeDiff(pSerTimes[SER_TIME_ID_T0], 
+	                            pSerTimes[SER_TIME_ID_T5])));
+}
+
+NTSTATUS HwRecoveryFromError(RTMP_ADAPTER *pAd)
 {
     UINT32 Status;
+    UINT32 Stat;
     ERR_RECOVERY_CTRL_T *pErrRecoveryCtrl;
 
+    UINT32 Highpart,Lowpart;
+    UINT32 *pSerTimes = NULL;
+    POS_COOKIE pObj;
+
 #ifdef CONFIG_ATE
     if (ATE_ON(pAd))
     {
-        MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("The driver is in ATE mode now\n"));
+        MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_ERROR,
+		("%s():The driver is in ATE mode now\n", __FUNCTION__));
         return NDIS_STATUS_SUCCESS;
     }
 #endif /* CONFIG_ATE */
@@ -643,13 +679,24 @@ static NTSTATUS HwRecoveryFromError(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
         return NDIS_STATUS_INVALID_DATA;
 
     pErrRecoveryCtrl = &pAd->ErrRecoveryCtl;
-    Status = (UINT32)*((UINT_32 *)CMDQelmt->buffer);
+    Status = pAd->HwCtrl.ser_status;
+    Stat = ErrRecoveryCurStat(pErrRecoveryCtrl);
+    pSerTimes = &pAd->HwCtrl.ser_times[0];
 
-    switch (ErrRecoveryCurStat(pErrRecoveryCtrl))
+
+    MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s,::E  R  , stat=0x%08X\n", __FUNCTION__, Stat));
+
+    switch (Stat)
     {
         case ERR_RECOV_STOP_IDLE:
             if ((Status & ERROR_DETECT_STOP_PDMA) == ERROR_DETECT_STOP_PDMA)
             {
+		os_zero_mem(pSerTimes,
+			(sizeof(pSerTimes[SER_TIME_ID_T0]) * SER_TIME_ID_END));
+
+		MtAsicGetTsfTimeByDriver(pAd, &Highpart, &Lowpart, HW_BSSID_0);
+		pSerTimes[SER_TIME_ID_T0] = Lowpart;
+
                 /* Stop access PDMA. */
                 ErrRecoveryStopPdmaAccess(pErrRecoveryCtrl);
                 /* send PDMA0 stop to N9 through interrupt. */
@@ -658,6 +705,8 @@ static NTSTATUS HwRecoveryFromError(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
                 pAd->bPCIclkOff = TRUE;
         		RtmpusecDelay(100*1000); /* delay for 100 ms to wait reset done. */
                 pAd->bPCIclkOff = FALSE;
+		MtAsicGetTsfTimeByDriver(pAd, &Highpart, &Lowpart, HW_BSSID_0);
+		pSerTimes[SER_TIME_ID_T1] = Lowpart;
             }
             else
             {
@@ -670,17 +719,22 @@ static NTSTATUS HwRecoveryFromError(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
             if ((Status & ERROR_DETECT_RESET_DONE) == ERROR_DETECT_RESET_DONE)
             {
                 WPDMA_GLO_CFG_STRUC GloCfg;
+		MtAsicGetTsfTimeByDriver(pAd, &Highpart, &Lowpart, HW_BSSID_0);
+		pSerTimes[SER_TIME_ID_T2] = Lowpart;
+
                 HIF_IO_READ32(pAd, MT_WPDMA_GLO_CFG, &GloCfg.word);
 
                 /* clean Tx/Rx rings and cut-through queue. */
                 RTMPResetTxRxRingMemory(pAd);
                 WfHifInit(pAd);
-                RT28XXDMAEnable(pAd);
+		RT28XXDMAEnable(pAd);
                 RTMP_IO_WRITE32(pAd, MT_WPDMA_MEM_RNG_ERR, 0);
                 ErrRecoveryReinitPdma(pErrRecoveryCtrl);
 
                 /* send PDMA0 reinit done to N9 through interrupt. */
                 ErrRecoveryMcuIntEvent(pAd, MCU_INT_PDMA0_INIT_DONE);
+		MtAsicGetTsfTimeByDriver(pAd, &Highpart, &Lowpart, HW_BSSID_0);
+		pSerTimes[SER_TIME_ID_T3] = Lowpart;
             }
             else
             {
@@ -692,6 +746,9 @@ static NTSTATUS HwRecoveryFromError(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
         case ERR_RECOV_RESET_PDMA0:
             if ((Status & ERROR_DETECT_RECOVERY_DONE) == ERROR_DETECT_RECOVERY_DONE)
             {
+		MtAsicGetTsfTimeByDriver(pAd, &Highpart, &Lowpart, HW_BSSID_0);
+		pSerTimes[SER_TIME_ID_T4] = Lowpart;
+
                 ErrRecoveryDone(pErrRecoveryCtrl);
 
                 /* update Beacon frame if operating in AP mode. */
@@ -699,6 +756,20 @@ static NTSTATUS HwRecoveryFromError(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
                 pAd,
                 NULL,
                 AP_RENEW);
+                ErrRecoveryMcuIntEvent(pAd, MCU_INT_PDMA0_RECOVERY_DONE);
+
+		MtAsicGetTsfTimeByDriver(pAd, &Highpart, &Lowpart, HW_BSSID_0);
+		pSerTimes[SER_TIME_ID_T5] = Lowpart;
+
+		/*print out ser log timing*/
+		ErrRecoveryLogDump(pAd);
+		pObj = (POS_COOKIE) pAd->OS_Cookie;
+#ifdef CONFIG_ANDES_SUPPORT
+		RTMP_OS_TASKLET_SCHE(&pObj->rx1_done_task);
+#endif /* CONFIG_ANDES_SUPPORT */
+#if defined(RTMP_MAC_PCI) || defined(RTMP_MAC_USB)
+		RTMP_OS_TASKLET_SCHE(&pObj->rx_done_task);
+#endif //defined(RTMP_MAC_PCI) || defined(RTMP_MAC_USB)
             }
             else
             {
@@ -859,7 +930,8 @@ static VOID hwRunWifiSysCtrl(RTMP_ADAPTER *pAd,WIFI_SYS_CTRL *wifi_sys_ctrl)
 			sta_rec_ctrl->WlanIdx,
 			sta_rec_ctrl->ConnectionType,
 			sta_rec_ctrl->ConnectionState,
-			sta_rec_ctrl->EnableFeature
+			sta_rec_ctrl->EnableFeature,
+			sta_rec_ctrl->IsNewSTARec
 		);
 	}
 }
@@ -898,9 +970,15 @@ static NTSTATUS HwCtrlWifiSysClose(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 static NTSTATUS HwCtrlWifiSysLinkUp(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 {
 	WIFI_SYS_CTRL *wifi_sys_ctrl =(WIFI_SYS_CTRL*)CMDQelmt->buffer;
+#ifdef TXBF_SUPPORT	
+    UCHAR wcid = wifi_sys_ctrl->StaRecCtrl.WlanIdx;
+    LINKUP_HWCTRL lu_ctrl;
+	PMAC_TABLE_ENTRY pEntry;   
+#endif /* TXBF_SUPPORT */
 	struct wifi_dev *wdev = wifi_sys_ctrl->wdev;
 	UINT16 txop_level = TXOP_0;
 
+	
 	hwRunWifiSysCtrl(pAd,wifi_sys_ctrl);
 
 	if(wifi_sys_ctrl->BssInfoCtrl.u4BssInfoFeature)
@@ -917,17 +995,120 @@ static NTSTATUS HwCtrlWifiSysLinkUp(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 	}
 	hw_set_tx_burst(pAd, wdev, AC_BE, PRIO_DEFAULT, txop_level, 1);
 
+#ifdef TXBF_SUPPORT
+    pAd->fgBfProfAlloc  = FALSE;
+#ifdef APCLI_SUPPORT
+    pAd->fgApClientMode = FALSE;
+#endif /* APCLI_SUPPORT */
+#endif /* TXBF_SUPPORT */
+
 	switch(wdev->wdev_type){
 #ifdef CONFIG_AP_SUPPORT
 	case WDEV_TYPE_AP:
 	{
 		wdev->bcn_buf.bBcnSntReq = TRUE;
 		LeadTimeForBcn(pAd,wdev);
+#ifdef TXBF_SUPPORT		
+		pAd->fgBfProfAlloc = FALSE;
+#endif
 	}
 	break;
 #endif /*CONFIG_AP_SUPPORT*/
+
+#if defined(APCLI_SUPPORT) && defined(TXBF_SUPPORT)
+	case WDEV_TYPE_APCLI:
+	{
+	    pAd->fgApClientMode = TRUE;	    
+	    pAd->fgBfProfAlloc  = FALSE;
+	    //if (pAd->ApCli_idx == 0xFF)
+	    //{
+		//    fgBfProfAlloc  = TRUE;
+		//    pAd->ApCli_idx = 0; // Clear index
+	    //}
+
+	    if (pAd->fgApCliBfStaRecRegister == FALSE)
+	    {
+	        pAd->ApCliWcidBfStaRecRegister = wcid;
+	        pAd->fgApCliBfStaRecRegister   = TRUE;
+	        pAd->fgBfProfAlloc             = TRUE;
+	    }    
+	}
+	break;
+#endif /*APCLI_SUPPORT && TXBF_SUPPORT*/
 	}
 
+#ifdef TXBF_SUPPORT
+    if (HcIsBfCapSupport(wdev))
+    {           
+        if (pAd->fgBfProfAlloc == TRUE)
+        {
+            /* BF StaRec Update */
+		    AsicBfStaRecUpdate(pAd,
+			                   wdev->PhyMode,
+			                   wdev->BssIdx,
+			                   wcid);
+            		        
+            /* Check BF Status and update it into WTBL */
+            pEntry         = &pAd->MacTab.Content[wcid];
+		    lu_ctrl.bMu    = pEntry->rStaRecBf.fgSU_MU;
+            lu_ctrl.bETxBf = (pEntry->rStaRecBf.fgETxBfCap && pAd->CommonCfg.ETxBfEnCond) ? TRUE : FALSE;
+            MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("%s: fgETxBfCap = %d, ETxBfEnCond = %ld\n", 
+                                                                 __FUNCTION__, 
+                                                                 pEntry->rStaRecBf.fgETxBfCap, 
+                                                                 pAd->CommonCfg.ETxBfEnCond));
+            lu_ctrl.bMuTxBf = (pEntry->rStaRecBf.fgSU_MU && pAd->CommonCfg.ETxBfEnCond) ? TRUE : FALSE;
+            lu_ctrl.bITxBf  = ((!lu_ctrl.bETxBf) && pAd->CommonCfg.RegTransmitSetting.field.ITxBfEn);
+
+            /* If Peer has eBf capability or iBF HW is enabled, SW will allocate a PFMU memory for it */
+            /* But the peer has no eBf capability and iBF HW is disabled, then SW will bypass memory allocation */
+            if ((lu_ctrl.bETxBf == TRUE) || (pAd->CommonCfg.RegTransmitSetting.field.ITxBfEn))
+            {
+		        /* Pfmu memory allocation */
+		        AsicBfPfmuMemAlloc(pAd,
+			        		       lu_ctrl.bMu,
+				        	       wcid);
+
+		        AsicTxBfTxApplyCtrl(pAd,
+			        		        wcid,
+				        		    lu_ctrl.bETxBf,
+					        	    lu_ctrl.bITxBf,
+						    	    lu_ctrl.bMuTxBf,
+							        FALSE);
+		
+		        /* update the global BF StaRec */
+		        CmdETxBfStaRecRead(pAd,wcid);
+
+#ifdef APCLI_SUPPORT
+		        if (pAd->fgApClientMode == TRUE)
+		        {
+		            mdelay(100);
+		            pAd->ApCli_CmmPfmuId = pAd->rStaRecBf.u2PfmuId;
+		            pAd->fgApCli_iBF     = lu_ctrl.bITxBf;
+                    pAd->fgApCli_eBF     = lu_ctrl.bETxBf;
+                    pAd->fgApCli_MuBF    = lu_ctrl.bMuTxBf;
+		        }
+#endif /* APCLI_SUPPORT */		
+            }
+            else
+            {
+                AsicBfStaRecRelease(pAd,
+			                        wdev->BssIdx,
+			                        wcid);
+            }
+        }   
+    }
+    else
+    {
+        /* Clear the BF bits in WTBL per entry */
+        AsicTxBfTxApplyCtrl(pAd,
+			    		    wcid,
+				    		FALSE,
+					    	FALSE,
+							FALSE,
+							FALSE);
+    }
+#endif /* TXBF_SUPPORT */                
+            
 	return NDIS_STATUS_SUCCESS;
 }
 
@@ -1025,42 +1206,65 @@ static NTSTATUS HwCtrlWifiSysPeerLinkUp(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 	        AsicSetRDG(pAd, wcid, HcGetBandByWdev(wdev), 1, 1);
 	    }
 #endif
+	}
 
 #ifdef TXBF_SUPPORT
-		/* BF StaRec Update */
+    if (HcIsBfCapSupport(wdev))
+    {
+        /* BF StaRec Update */
 		AsicBfStaRecUpdate(pAd,
-			wdev->PhyMode,
-			wdev->BssIdx,
-			&lu_ctrl->ie_list,
-			wcid);
-
+			               wdev->PhyMode,
+			               wdev->BssIdx,
+			               wcid);
+        
         /* Check BF Status and update it into WTBL */
         pEntry = &pAd->MacTab.Content[wcid];
 		lu_ctrl->bMu = pEntry->rStaRecBf.fgSU_MU;
         lu_ctrl->bETxBf  = (pEntry->rStaRecBf.fgETxBfCap && pAd->CommonCfg.ETxBfEnCond) ? TRUE : FALSE;
         MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("%s: fgETxBfCap = %d, ETxBfEnCond = %ld\n", 
-                                                            __FUNCTION__, 
-                                                            pEntry->rStaRecBf.fgETxBfCap, 
-                                                            pAd->CommonCfg.ETxBfEnCond));
+                                                             __FUNCTION__, 
+                                                             pEntry->rStaRecBf.fgETxBfCap, 
+                                                             pAd->CommonCfg.ETxBfEnCond));
         lu_ctrl->bMuTxBf = (pEntry->rStaRecBf.fgSU_MU && pAd->CommonCfg.ETxBfEnCond) ? TRUE : FALSE;
         lu_ctrl->bITxBf  = ((!lu_ctrl->bETxBf) && pAd->CommonCfg.RegTransmitSetting.field.ITxBfEn);
 
-		/* Pfmu memory allocation */
-		AsicBfPfmuMemAlloc(pAd,
-						   lu_ctrl->bMu,
-						   wcid);
+        /* If Peer has eBf capability or iBF HW is enabled, SW will allocate a PFMU memory for it */
+        /* But the peer has no eBf capability and iBF HW is disabled, then SW will bypass memory allocation */
+        if ((lu_ctrl->bETxBf == TRUE) || (pAd->CommonCfg.RegTransmitSetting.field.ITxBfEn))
+        {
+		    /* Pfmu memory allocation */
+		    AsicBfPfmuMemAlloc(pAd,
+			        		   lu_ctrl->bMu,
+				        	   wcid);
 
-		AsicTxBfTxApplyCtrl(pAd,
-							wcid,
-							lu_ctrl->bETxBf,
-							lu_ctrl->bITxBf,
-							lu_ctrl->bMuTxBf,
-							FALSE);
+		    AsicTxBfTxApplyCtrl(pAd,
+			    		        wcid,
+				    		    lu_ctrl->bETxBf,
+					    	    lu_ctrl->bITxBf,
+							    lu_ctrl->bMuTxBf,
+							    FALSE);
 		
-		/* update the global BF StaRec */
-		CmdETxBfStaRecRead(pAd,wcid);
+		    /* update the global BF StaRec */
+		    CmdETxBfStaRecRead(pAd,wcid);
+        }
+        else
+        {
+            AsicBfStaRecRelease(pAd,
+			                    wdev->BssIdx,
+			                    wcid);
+        }
+    }
+    else
+    {
+        /* Clear the BF bits in WTBL per entry */
+        AsicTxBfTxApplyCtrl(pAd,
+			    		    wcid,
+				    		FALSE,
+					    	FALSE,
+							FALSE,
+							FALSE);
+    }
 #endif /* TXBF_SUPPORT */
-	}
 	
 	if(lu_ctrl)
 	{
@@ -1107,7 +1311,8 @@ static NTSTATUS HwCtrlWifiSysPeerUpdate(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 			sta_rec_ctrl->WlanIdx,
 			sta_rec_ctrl->ConnectionType,
 			sta_rec_ctrl->ConnectionState,
-			featues
+			featues,
+			sta_rec_ctrl->IsNewSTARec
 		);
 
 		sta_rec_ctrl->EnableFeature &= (~STA_REC_RA_COMMON_INFO_FEATURE);
@@ -1122,7 +1327,8 @@ static NTSTATUS HwCtrlWifiSysPeerUpdate(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 			sta_rec_ctrl->WlanIdx,
 			sta_rec_ctrl->ConnectionType,
 			sta_rec_ctrl->ConnectionState,
-			featues
+			featues,
+			sta_rec_ctrl->IsNewSTARec
 		);
 		
 		sta_rec_ctrl->EnableFeature &= (~STA_REC_RA_FEATURE);
@@ -1137,7 +1343,8 @@ static NTSTATUS HwCtrlWifiSysPeerUpdate(RTMP_ADAPTER *pAd, HwCmdQElmt *CMDQelmt)
 			sta_rec_ctrl->WlanIdx,
 			sta_rec_ctrl->ConnectionType,
 			sta_rec_ctrl->ConnectionState,
-			sta_rec_ctrl->EnableFeature
+			sta_rec_ctrl->EnableFeature,
+			sta_rec_ctrl->IsNewSTARec
 		);
 	}
 	
@@ -1151,8 +1358,10 @@ static HW_CMD_TABLE_T HwCmdRadioTable[] = {
 	{HWCMD_ID_UPDATE_DAW_COUNTER,HwCtrlUpdateRawCounters,0},
 #ifdef MT_MAC
 	{HWCMD_ID_SET_CLIENT_MAC_ENTRY,HwCtrlSetClientMACEntry,0},
+#ifdef TXBF_SUPPORT	
+    {HWCMD_ID_SET_APCLI_BF_CAP,HwCtrlSetClientBfCap,0},
+#endif    
 	{HWCMD_ID_SET_TR_ENTRY,HwCtrlSetTREntry,0},
-	{HWCMD_ID_SET_STA_REC, HwCtrlSetStaRec,0},
 	{HWCMD_ID_SET_BA_REC, HwCtrlSetBaRec,0},
 	{HWCMD_ID_UPDATE_BSSINFO,HwCtrlUpdateBssInfo,0},
 	{HWCMD_ID_UPDATE_BEACON,HwCtrlHandleUpdateBeacon,0},
@@ -1203,10 +1412,6 @@ static HW_CMD_TABLE_T HwCmdPeripheralTable[]={
 #ifdef LED_CONTROL_SUPPORT
 	{HWCMD_ID_SET_LED_STATUS,HwCtrlSetLEDStatus,0},
 #endif
-#ifdef ERR_RECOVERY
-    {HWCMD_ID_MAC_ERROR_DETECT, HwRecoveryFromError,0},
-#endif /* ERR_RECOVERY */
-
 	{HWCMD_ID_END,NULL,0}
 };
 
