@@ -29,6 +29,7 @@ static INT32 HW_CTRL_BASIC_ENQ(RTMP_ADAPTER *pAd,INT32 CmdType,INT32 CmdId,UINT3
 	HwCtrlTxd.CmdType = CmdType;
 	HwCtrlTxd.CmdId = CmdId;
 	HwCtrlTxd.NeedWait = FALSE;
+	HwCtrlTxd.wait_time = 0;
 	HwCtrlTxd.InformationBufferLength = Len;
 	HwCtrlTxd.pInformationBuffer = pBuffer;
 	HwCtrlTxd.pRespBuffer = NULL;
@@ -44,6 +45,7 @@ static INT32 HW_CTRL_BASIC_ENQ(RTMP_ADAPTER *pAd,INT32 CmdType,INT32 CmdId,UINT3
 	_HwCtrlTxd.CmdType = _CmdType; \
 	_HwCtrlTxd.CmdId = _CmdId; \
 	_HwCtrlTxd.NeedWait = FALSE; \
+	_HwCtrlTxd.wait_time = 0;\
 	_HwCtrlTxd.InformationBufferLength = _Len; \
 	_HwCtrlTxd.pInformationBuffer = _pBuffer; \
 	_HwCtrlTxd.pRespBuffer = NULL; \
@@ -52,9 +54,10 @@ static INT32 HW_CTRL_BASIC_ENQ(RTMP_ADAPTER *pAd,INT32 CmdType,INT32 CmdId,UINT3
 	_HwCtrlTxd.CallbackArgs = NULL; \
 }
 
-#define HW_CTRL_TXD_RSP(_pAd,_RspLen,_RspBuffer,_HwCtrlTxd) \
+#define HW_CTRL_TXD_RSP(_pAd,_RspLen,_RspBuffer,_wait_time,_HwCtrlTxd) \
 { \
 	_HwCtrlTxd.NeedWait = TRUE; \
+	_HwCtrlTxd.wait_time = _wait_time;\
 	_HwCtrlTxd.pRespBuffer = _RspBuffer; \
 	_HwCtrlTxd.RespBufferLength = _RspLen; \
 }
@@ -158,7 +161,7 @@ VOID RTMP_GET_TEMPERATURE(RTMP_ADAPTER *pAd,UINT32 *pTemperature)
 	os_zero_mem(&HwCtrlTxd,sizeof(HW_CTRL_TXD));
 
 	HW_CTRL_TXD_BASIC(pAd,HWCMD_TYPE_RADIO,HWCMD_ID_GET_TEMPERATURE,0,NULL,HwCtrlTxd);
-	HW_CTRL_TXD_RSP(pAd,sizeof(UINT32),pTemperature,HwCtrlTxd);
+	HW_CTRL_TXD_RSP(pAd,sizeof(UINT32),pTemperature,0,HwCtrlTxd);
 	ret = HwCtrlEnqueueCmd(pAd,HwCtrlTxd);
 }
 
@@ -234,29 +237,6 @@ VOID HW_UPDATE_BSSINFO(RTMP_ADAPTER *pAd,BSS_INFO_ARGUMENT_T BssInfoArgs)
  	ret = HW_CTRL_BASIC_ENQ(pAd, HWCMD_TYPE_RADIO, HWCMD_ID_UPDATE_BSSINFO, sizeof(BSS_INFO_ARGUMENT_T), &BssInfoArgs);
 }
 
-
-VOID HW_SET_STA_REC(RTMP_ADAPTER *pAd,
-	UINT8 BssIndex,
-	UINT8 WlanIdx,
-	UINT32 ConnectionType,
-	UINT8 ConnectionState,
-	UINT32 EnableFeature)
-{
-	UINT32 ret;
-	MT_SET_STA_REC rMtSetStaRec;
-
-	os_zero_mem(&rMtSetStaRec,sizeof(MT_SET_STA_REC));
-
-	rMtSetStaRec.BssIndex = BssIndex;
-	rMtSetStaRec.WlanIdx = WlanIdx;
-	rMtSetStaRec.ConnectionType = ConnectionType;
-	rMtSetStaRec.ConnectionState = ConnectionState;
-	rMtSetStaRec.EnableFeature = EnableFeature;
-
- 	ret = HW_CTRL_BASIC_ENQ(pAd, HWCMD_TYPE_RADIO, HWCMD_ID_SET_STA_REC,sizeof(MT_SET_STA_REC), &rMtSetStaRec);
-}
-
-
 VOID HW_SET_BA_REC(RTMP_ADAPTER *pAd, UCHAR wcid, UCHAR tid, UINT16 sn, UCHAR basize, BOOLEAN isAdd, INT ses_type)
 {
 	UINT32 ret;
@@ -289,16 +269,28 @@ VOID RTMP_PS_RETRIVE_START(PRTMP_ADAPTER pAd, UCHAR Wcid)
 #ifdef ERR_RECOVERY
 VOID RTMP_MAC_RECOVERY(struct _RTMP_ADAPTER *pAd, UINT32 Status)
 {
-	UINT32 ret;
-    UINT32 value;
+	UINT32 value;
 
-    value = Status & ERROR_DETECT_MASK;
-    /* Trigger error recovery process with fw reload. */
-    ret = HW_CTRL_BASIC_ENQ(pAd, HWCMD_TYPE_PERIPHERAL,
-        HWCMD_ID_MAC_ERROR_DETECT,sizeof(value), &value);
+	value = Status & ERROR_DETECT_MASK;
+	/* Trigger error recovery process with fw reload. */
 
-	if (ret != NDIS_STATUS_SUCCESS)
-		ASSERT(0);
+	if (pAd->HwCtrl.ser_func_state != RTMP_TASK_STAT_RUNNING) {
+		MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
+			("%s::ERR! SER func not ready(%d)\n", 
+			__FUNCTION__, pAd->HwCtrl.ser_func_state));
+		// TODO: do we may hit this case?
+		return;
+	}
+
+	if (value != pAd->HwCtrl.ser_status) {
+		pAd->HwCtrl.ser_status = value;
+		RTCMDUp(&pAd->HwCtrl.ser_task);
+	} else {
+		// TODO: do we may hit this case?
+		MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_ERROR, 
+			("%s::ERR! prev state=%x, new stat=%x\n", 
+			__FUNCTION__, pAd->HwCtrl.ser_status, value));
+	}
 }
 
 INT IsStopingPdma(ERR_RECOVERY_CTRL_T *pErrRecoveryCtl)
@@ -306,6 +298,30 @@ INT IsStopingPdma(ERR_RECOVERY_CTRL_T *pErrRecoveryCtl)
     return (pErrRecoveryCtl->errRecovState == ERR_RECOV_STOP_IDLE) ?
         FALSE : TRUE;
 }
+
+BOOLEAN IsErrRecoveryInIdleStat(RTMP_ADAPTER *pAd)
+{
+    UINT32 Stat;
+        
+    if (pAd == NULL)
+        return TRUE;
+
+
+    Stat = ErrRecoveryCurStat(&pAd->ErrRecoveryCtl);
+
+    if (Stat == ERR_RECOV_STOP_IDLE)
+        return TRUE;
+    else
+        return FALSE;
+}
+
+ERR_RECOVERY_STATE ErrRecoveryCurStat(ERR_RECOVERY_CTRL_T *pErrRecoveryCtl)
+{
+    if (pErrRecoveryCtl == NULL)
+        return ERR_RECOV_STOP_IDLE;
+
+    return pErrRecoveryCtl->errRecovState;
+}
 #endif /* ERR_RECOVERY */
 #endif /*MT_MAC*/
 
@@ -532,6 +548,21 @@ VOID RTMP_STA_ENTRY_ADD(PRTMP_ADAPTER pAd, ULONG Wcid,UCHAR *pAddr,BOOLEAN IsBMC
 }
 
 
+#ifdef TXBF_SUPPORT
+/*
+*
+*/
+VOID HW_APCLI_BF_CAP_CONFIG(PRTMP_ADAPTER pAd, PMAC_TABLE_ENTRY pMacEntry)
+{
+	INT32 ret;
+	ret = HW_CTRL_BASIC_ENQ(pAd, 
+	                        HWCMD_TYPE_RADIO,
+	                        HWCMD_ID_SET_APCLI_BF_CAP,
+	                        sizeof(MAC_TABLE_ENTRY),
+	                        pMacEntry);
+}
+#endif /* TXBF_SUPPORT */
+
 
 /*WIFI_SYS related HwCtrl CMD*/
 /*
@@ -542,11 +573,18 @@ VOID HW_WIFISYS_OPEN(
     WIFI_SYS_CTRL wifi_sys_ctrl)
 {
     UINT32 ret;
-    ret = HW_CTRL_BASIC_ENQ(pAd,
-        HWCMD_TYPE_WIFISYS,
-        HWCMD_ID_WIFISYS_OPEN,
-        sizeof(WIFI_SYS_CTRL),
-        &wifi_sys_ctrl);
+	UINT32 wait_time = 2000;
+	HW_CTRL_TXD HwCtrlTxd;
+	HW_CTRL_TXD_BASIC(
+		pAd,
+		HWCMD_TYPE_WIFISYS,
+		HWCMD_ID_WIFISYS_OPEN,
+		sizeof(WIFI_SYS_CTRL),
+		&wifi_sys_ctrl,
+		HwCtrlTxd
+	);
+	HW_CTRL_TXD_RSP(pAd,0,NULL,wait_time,HwCtrlTxd);	
+	ret = HwCtrlEnqueueCmd(pAd,HwCtrlTxd);
 }
 
 
@@ -558,11 +596,17 @@ VOID HW_WIFISYS_CLOSE(
     WIFI_SYS_CTRL wifi_sys_ctrl)
 {
     UINT32 ret;
-    ret = HW_CTRL_BASIC_ENQ(pAd,
-        HWCMD_TYPE_WIFISYS,
-        HWCMD_ID_WIFISYS_CLOSE,
-        sizeof(WIFI_SYS_CTRL),
-        &wifi_sys_ctrl);
+	UINT32 wait_time = 2000;
+	HW_CTRL_TXD HwCtrlTxd;
+	HW_CTRL_TXD_BASIC(
+		pAd,
+		HWCMD_TYPE_WIFISYS,
+		HWCMD_ID_WIFISYS_CLOSE,
+		sizeof(WIFI_SYS_CTRL),
+		&wifi_sys_ctrl,HwCtrlTxd
+	);
+	HW_CTRL_TXD_RSP(pAd,0,NULL,wait_time,HwCtrlTxd);	
+	ret = HwCtrlEnqueueCmd(pAd,HwCtrlTxd);
 }
 
 
