@@ -900,6 +900,7 @@ typedef	struct _CIPHER_SUITE {
 struct _vendor_ie_cap {
     ULONG ra_cap;
     ULONG mtk_cap;
+    ULONG brcm_cap;
     BOOLEAN ldpc;
     BOOLEAN sgi;
 };
@@ -938,6 +939,7 @@ enum _tx_burst_prio {
     PRIO_DEFAULT=0,
     PRIO_RDG,
     PRIO_2G_INFRA,
+    PRIO_MU_MIMO,
     PRIO_APCLI_REPEATER,
     PRIO_CCI,
     PRIO_WMM,
@@ -965,6 +967,7 @@ struct _tx_burst_cfg {
 #define TXOP_60         (0x60)
 #define TXOP_80         (0x80)
 #define TXOP_A0         (0xA0)
+#define TXOP_C0         (0xC0)
 #define TXOP_FE         (0xFE)
 #define TXOP_138        (0x138)
 #define TXOP_177        (0x177)
@@ -1151,6 +1154,7 @@ typedef struct {
 struct raw_rssi_info{
 	CHAR raw_rssi[4];
 	UCHAR raw_snr;
+	UCHAR Channel;
 };
 
 typedef struct _MLME_QUEUE_ELEM {
@@ -1470,6 +1474,13 @@ struct _build_ie_info {
 };
 
 
+struct _op_info {
+	UINT8 bw;
+	UINT8 cent_ch;
+	UINT8 cent_ch2;
+};
+
+
 typedef struct _IE_lists {
 	UCHAR Addr2[MAC_ADDR_LEN];
 	UCHAR ApAddr[MAC_ADDR_LEN];
