@@ -943,10 +943,10 @@ do {	\
 		_pAd->chipOps.AsicWOWEnable(_pAd, _pStaCfg);	\
 } while (0)
 
-#define ASIC_WOW_DISABLE(_pAd)	\
+#define ASIC_WOW_DISABLE(_pAd, _pStaCfg)	\
 do {	\
 	if (_pAd->chipOps.AsicWOWDisable != NULL)	\
-		_pAd->chipOps.AsicWOWDisable(_pAd);	\
+		_pAd->chipOps.AsicWOWDisable(_pAd, _pStaCfg);	\
 } while(0)
 
 #define ASIC_WOW_INIT(_pAd) \
