@@ -965,6 +965,10 @@ enum {
 
 #define DOT11_H_STATE_MACHINE	45
 
+#ifdef CONFIG_AP_SUPPORT
+#define AUTO_CH_SEL_STATE_MACHINE   46
+#endif/* CONFIG_AP_SUPPORT */
+
 /*
 	STA's CONTROL/CONNECT state machine: states, events, total function #
 */
@@ -1413,6 +1417,24 @@ enum {
 #endif
 #define APCLI_CTRL_FUNC_SIZE              (APCLI_MAX_CTRL_STATE * APCLI_MAX_CTRL_MSG)
 
+/*ApCli,Repeater Link Down reason */
+#define APCLI_LINKDOWN_NONE				  0
+#define APCLI_LINKDOWN_DEAUTH_REQ		  1	
+#define APCLI_LINKDOWN_DEASSOC_REQ		  2
+#define APCLI_LINKDOWN_PEER_DEASSOC_REQ	  3
+#define APCLI_LINKDOWN_DISCONNECT_REQ	  4
+#define APCLI_LINKDOWN_PEER_DEASSOC_RSP	  5
+
+#define APCLI_DISCONNECT_SUB_REASON_NONE	                0
+#define APCLI_DISCONNECT_SUB_REASON_REPTLM_TRIGGER_TOO_LONG	1
+#define APCLI_DISCONNECT_SUB_REASON_MTM_IDLE_TOO_LONG	    2
+#define APCLI_DISCONNECT_SUB_REASON_MTM_REMOVE_STA          3
+#define APCLI_DISCONNECT_SUB_REASON_APCLI_IF_DOWN           4
+#define APCLI_DISCONNECT_SUB_REASON_MNT_NO_BEACON           5
+#define APCLI_DISCONNECT_SUB_REASON_AP_PEER_DISASSOC_REQ    6
+#define APCLI_DISCONNECT_SUB_REASON_AP_PEER_DEAUTH_REQ      7
+#define APCLI_DISCONNECT_SUB_REASON_APCFG_DEL_MAC_ENTRY     8
+
 
 #endif /* APCLI_SUPPORT */
 
@@ -2281,4 +2303,5 @@ do{									\
 #define RTMP_OS_TASK_INIT(__pTask, __pTaskName, __pAd)		\
 	RtmpOSTaskInit(__pTask, __pTaskName, __pAd, &(__pAd)->RscTaskMemList, &(__pAd)->RscSemMemList);
 
+#define BandOffset		0x200
 #endif /* __RTMP_DEF_H__ */
