@@ -536,7 +536,7 @@ VOID ExtEventWifiSpectrumRawDataDumpHandler(RTMP_ADAPTER *pAd, UINT8 *Data, UINT
 {
     PUINT32 pData = NULL;
     SHORT I_0, Q_0, LNA, LPF;
-    UINT32 i;
+    UINT32 i, CaptureNode;
     UCHAR msg[64] , retval;
     RTMP_OS_FS_INFO osFSInfo;
     EVENT_WIFI_ICAP_DUMP_RAW_DATA_T *prIcapGetEvent = (EVENT_WIFI_ICAP_DUMP_RAW_DATA_T *)Data;
@@ -547,32 +547,69 @@ VOID ExtEventWifiSpectrumRawDataDumpHandler(RTMP_ADAPTER *pAd, UINT8 *Data, UINT
     os_alloc_mem(pAd, (UCHAR **)&pData, WIFISPECTRUM_EVENT_DATA_SAMPLE * sizeof(UINT32));
     NdisZeroMemory(pData, WIFISPECTRUM_EVENT_DATA_SAMPLE*sizeof(UINT32));
   
+    CaptureNode = Get_Icap_WifiSpec_Capture_Node_Info(pAd);
+  
     RtmpOSFSInfoChange(&osFSInfo, TRUE); // Change limits of authority in order to read/write file
 
     for (i=0;i < WIFISPECTRUM_EVENT_DATA_SAMPLE; i++)
     {        
         pData[i] = prIcapGetEvent->Data[i];
                 
-        //Parse and dump I/Q data
-        Q_0 = (pData[i] & 0x3FF);
-        I_0 = ((pData[i] & (0x3FF<<10))>>10);
-        if (Q_0 >= 512)
+        if ((CaptureNode == WF0_ADC) || (CaptureNode == WF1_ADC) 
+            || (CaptureNode == WF2_ADC) || (CaptureNode == WF3_ADC))//Dump 1-way RXADC
         {
-            Q_0 -= 1024;
-        }    
-        if (I_0 >= 512)
+            MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
+            ("%s : Dump 1-way RXADC\n", __FUNCTION__));      
+            
+            //Parse and dump I/Q data
+            Q_0 = (pData[i] & 0x3FF);
+            I_0 = ((pData[i] & (0x3FF<<10))>>10);
+            if (Q_0 >= 512)
+            {
+                Q_0 -= 1024;
+            }    
+            if (I_0 >= 512)
+            {
+                I_0 -= 1024;
+            }    
+            sprintf(msg, "%+04d\t%+04d\n", I_0, Q_0);
+            retval = RtmpOSFileWrite(pAd->Srcf_IQ, (RTMP_STRING *)msg, strlen(msg));
+
+            //Parse and dump LNA/LPF data
+            LNA = ((pData[i] & (0x3<<28))>>28);
+            LPF = ((pData[i] & (0xF<<24))>>24);
+            sprintf(msg, "%+04d\t%+04d\n", LNA, LPF);
+            retval = RtmpOSFileWrite(pAd->Srcf_Gain, (RTMP_STRING *)msg, strlen(msg));
+        }          
+        else if ((CaptureNode == WF0_FIIQ) || (CaptureNode == WF1_FIIQ) 
+                 || (CaptureNode == WF2_FIIQ) || (CaptureNode == WF3_FIIQ)
+                 || (CaptureNode == WF0_FDIQ) || (CaptureNode == WF1_FDIQ)
+                 || (CaptureNode == WF2_FDIQ) || (CaptureNode == WF3_FDIQ))//Dump 1-way RXIQC
         {
-            I_0 -= 1024;
+            MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
+            ("%s : Dump 1-way RXIQC\n", __FUNCTION__));
+            
+            //Parse and dump I/Q data
+            Q_0 = (pData[i] & 0xFFF);
+            I_0 = ((pData[i] & (0xFFF<<12))>>12);
+            if (Q_0 >= 2048)
+            {
+                Q_0 -= 4096;
+            }    
+            if (I_0 >= 2048)
+            {
+                I_0 -= 4096;
+            }    
+            sprintf(msg, "%+04d\t%+04d\n", I_0, Q_0);
+            retval = RtmpOSFileWrite(pAd->Srcf_IQ, (RTMP_STRING *)msg, strlen(msg));
+
+            //Parse and dump LNA/LPF data
+            LNA = ((pData[i] & (0x3<<28))>>28);
+            LPF = ((pData[i] & (0xF<<24))>>24);
+            sprintf(msg, "%+04d\t%+04d\n", LNA, LPF);
+            retval = RtmpOSFileWrite(pAd->Srcf_Gain, (RTMP_STRING *)msg, strlen(msg));
         }    
-        sprintf(msg, "%+04d\t%+04d\n", I_0, Q_0);
-        retval = RtmpOSFileWrite(pAd->Srcf_IQ, (RTMP_STRING *)msg, strlen(msg));
-
-        //Parse and dump LNA/LPF data
-        LNA = ((pData[i] & (0x3<<28))>>28);
-        LPF = ((pData[i] & (0xF<<24))>>24);
-        sprintf(msg, "%+04d\t%+04d\n", LNA, LPF);
-        retval = RtmpOSFileWrite(pAd->Srcf_Gain, (RTMP_STRING *)msg, strlen(msg));
-
+   
         MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
         ("%s : 0x%08x\n", __FUNCTION__, *(pData + i)));   
     }
@@ -972,7 +1009,7 @@ VOID RT28xx_UpdateBcnAndTimToMcu(
         pkt = tim_buf->TimPkt;
         bSntReq = tim_buf->bTimSntReq;
 
-        TimIELocation = bcn_buf->TimIELocationInBeacon;
+        TimIELocation = bcn_buf->TimIELocationInTim;
         CsaIELocation = bcn_buf->CsaIELocationInBeacon;
     }
 #endif /* CONFIG_AP_SUPPORT */
@@ -1181,6 +1218,43 @@ static VOID ExtEventCswNotifyHandler(
     }
 }
 
+static VOID ExtEventBssAcQPktNumHandler(RTMP_ADAPTER *pAd,
+                                UINT8 *Data, UINT32 Length)
+{
+	P_EVENT_BSS_ACQ_PKT_NUM_T prEventBssAcQPktNum =
+				(P_EVENT_BSS_ACQ_PKT_NUM_T)Data;
+	UINT8 i = 0;
+	UINT32 sum = 0;
+
+	P_EVENT_PER_BSS_ACQ_PKT_NUM_T prPerBssInfo = NULL;
+
+
+	MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_INFO,
+        	("RX ExtEventBssAcQPktNumHandler: u4BssMap=0x%08X\n",
+		prEventBssAcQPktNum->u4BssMap));
+
+	for (i = 0; (i < CR4_CFG_BSS_NUM) && (prEventBssAcQPktNum->u4BssMap & (1 << i)) ; i++)
+	{
+        	prPerBssInfo = &prEventBssAcQPktNum->bssPktInfo[i];
+
+		sum =   prPerBssInfo->au4AcqPktCnt[WMM_AC_VI] 
+        		+ prPerBssInfo->au4AcqPktCnt[WMM_AC_VO];
+    
+	        if (sum)
+        	{
+			MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_INFO,
+			("BSS[%d], AC_BK = %d, AC_BE = %d, AC_VI = %d, AC_VO = %d\n",
+			i, 
+			prPerBssInfo->au4AcqPktCnt[WMM_AC_BK],
+			prPerBssInfo->au4AcqPktCnt[WMM_AC_BE],
+			prPerBssInfo->au4AcqPktCnt[WMM_AC_VI],
+			prPerBssInfo->au4AcqPktCnt[WMM_AC_VO]));
+			pAd->OneSecondnonBEpackets += sum;
+		}
+	}
+	mt_dynamic_wmm_be_tx_op(pAd, ONE_SECOND_NON_BE_PACKETS_THRESHOLD);
+
+}
 
 static BOOLEAN IsUnsolicitedEvent(EVENT_RXD *event_rxd)
 {
@@ -1275,6 +1349,10 @@ static VOID EventExtEventHandler(RTMP_ADAPTER *pAd, UINT8 ExtEID, UINT8 *Data,
             ExtEventTmrCalcuInfoHandler(pAd, Data, Length);
             break;
 
+        case EXT_EVENT_ID_BSS_ACQ_PKT_NUM:
+            ExtEventBssAcQPktNumHandler(pAd, Data, Length);
+            break;
+
 		default:
 			MTWF_LOG(DBG_CAT_FW, DBG_SUBCAT_ALL, DBG_LVL_OFF,
 				("%s: Unknown Ext Event(%x)\n", __FUNCTION__, ExtEID));
