@@ -285,6 +285,7 @@ VOID MacTableSetEntryRaCap(RTMP_ADAPTER *pAd, MAC_TABLE_ENTRY *ent, struct _vend
 {
     ULONG ra_ie = vendor_ie->ra_cap;
     ULONG mtk_ie = vendor_ie->mtk_cap;
+    ULONG brcm_ie = vendor_ie->brcm_cap;
 
     MTWF_LOG(DBG_CAT_MLME, DBG_SUBCAT_ALL, DBG_LVL_INFO,
             ("vendor_ie_cap for ra_cap=%08x, mtk_cap=%08x\n", (UINT32)ra_ie, (UINT32)mtk_ie));
@@ -352,6 +353,12 @@ VOID MacTableSetEntryRaCap(RTMP_ADAPTER *pAd, MAC_TABLE_ENTRY *ent, struct _vend
                 ("MEDIATEK_256QAM_CAP for 2.4G\n"));
         ent->fgGband256QAMSupport = TRUE;
     }
+    else if ((brcm_ie & BROADCOM_256QAM_CAP) 
+            && (pAd->chipCap.g_band_256_qam)) {
+        MTWF_LOG(DBG_CAT_MLME, DBG_SUBCAT_ALL, DBG_LVL_INFO,
+                ("BROADCOM_256QAM_CAP for 2.4G\n"));
+        ent->fgGband256QAMSupport = TRUE;
+    }
     else {}
 }
 
@@ -440,6 +447,33 @@ static INT MacTableResetEntry(RTMP_ADAPTER *pAd, MAC_TABLE_ENTRY *pEntry, BOOLEA
 	return 0;
 }
 
+#ifdef OUI_CHECK_SUPPORT
+static VOID oui_mgroup_update(MAC_TABLE *mtb,UCHAR *addr,UCHAR act)
+{
+	UCHAR tmp_cnt=0;
+	UCHAR i=0;
+	MAC_TABLE_ENTRY *entry;
+    MTWF_LOG(DBG_CAT_MLME, DBG_SUBCAT_ALL, DBG_LVL_INFO,
+            ("%s(): update mgroup before num:%d. act=%d\n",__FUNCTION__,mtb->oui_mgroup_cnt,act));
+
+	for(i=0;i<=mtb->Size;i++){
+		entry = &mtb->Content[i];
+		if(MAC_OUI_EQUAL(entry->Addr,addr) && !MAC_ADDR_EQUAL(entry->Addr,addr)){
+			tmp_cnt++;
+			if(tmp_cnt > 2)
+				break;
+		}
+	}
+	/*only 1  oui equal means new match group*/
+	if(tmp_cnt==1){
+		mtb->oui_mgroup_cnt = (act==OUI_MGROUP_ACT_JOIN) ? 
+			(mtb->oui_mgroup_cnt+1):(mtb->oui_mgroup_cnt-1);
+	}
+
+	MTWF_LOG(DBG_CAT_MLME, DBG_SUBCAT_ALL, DBG_LVL_INFO,
+            ("%s(): update mgroup after num:%d\n",__FUNCTION__,mtb->oui_mgroup_cnt));
+}
+#endif /*OUI_CHECK_SUPPORT*/
 
 MAC_TABLE_ENTRY *MacTableInsertEntry(
 	IN RTMP_ADAPTER *pAd,
@@ -755,6 +789,9 @@ MAC_TABLE_ENTRY *MacTableInsertEntry(
 
 	}
 
+#ifdef OUI_CHECK_SUPPORT
+	oui_mgroup_update(&pAd->MacTab,pAddr,OUI_MGROUP_ACT_JOIN);
+#endif
 	NdisReleaseSpinLock(&pAd->MacTabLock);
 
 	/* Add this entry into ASIC RX WCID search table */
@@ -767,7 +804,6 @@ MAC_TABLE_ENTRY *MacTableInsertEntry(
 	/*update tx burst, must after unlock pAd->MacTabLock*/
     //if (pEntry)
         //rtmp_tx_burst_set(pAd, pEntry->wdev);
-
 	return pEntry;
 }
 
@@ -1031,7 +1067,9 @@ BOOLEAN MacTableDeleteEntry(RTMP_ADAPTER *pAd, USHORT wcid, UCHAR *pAddr)
 			MTWF_LOG(DBG_CAT_MLME, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("\n%s: Impossible Wcid = %d !!!!!\n", __FUNCTION__, wcid));
 		}
 	}
-
+#ifdef OUI_CHECK_SUPPORT
+	oui_mgroup_update(&pAd->MacTab,pAddr,OUI_MGROUP_ACT_LEAVE);
+#endif
 	NdisReleaseSpinLock(&pAd->MacTabLock);
 
 	/*update tx burst, must after unlock pAd->MacTabLock*/
@@ -1053,7 +1091,7 @@ BOOLEAN MacTableDeleteEntry(RTMP_ADAPTER *pAd, USHORT wcid, UCHAR *pAddr)
 #endif /* MT_MAC */
 
 		/*Release Ucast wcid*/
-	WifiSysPeerLinkDown(pAd,pEntry); 
+	WifiSysPeerLinkDown(pAd,pEntry);
     }
 
     /*Reset operating mode when no Sta.*/
@@ -1074,7 +1112,6 @@ BOOLEAN MacTableDeleteEntry(RTMP_ADAPTER *pAd, USHORT wcid, UCHAR *pAddr)
 #ifdef CONFIG_AP_SUPPORT
     APUpdateCapabilityAndErpIe(pAd);
 #endif /* CONFIG_AP_SUPPORT */
-
 	return TRUE;
 }
 
@@ -1197,7 +1234,7 @@ static VOID SetCommonHtVhtByRf(RTMP_ADAPTER *pAd,UCHAR RfIC)
 		return ;
 
 #ifdef DOT11_N_SUPPORT
-	SetCommonHT(pAd,PhyMode,Channel);
+	SetCommonHT(pAd,PhyMode,Channel,NULL);
 #ifdef DOT11_VHT_AC
 	if(WMODE_CAP_AC(PhyMode))
 		SetCommonVHT(pAd,PhyMode,Channel);
@@ -1205,10 +1242,25 @@ static VOID SetCommonHtVhtByRf(RTMP_ADAPTER *pAd,UCHAR RfIC)
 #endif /* DOT11_N_SUPPORT */
 }
 
-INT	SetCommonHtVht(RTMP_ADAPTER *pAd)
+static VOID SetHtVhtForWdev(RTMP_ADAPTER *pAd,struct wifi_dev *wdev)
 {
-	SetCommonHtVhtByRf(pAd,RFIC_24GHZ);
-	SetCommonHtVhtByRf(pAd,RFIC_5GHZ);
+#ifdef DOT11_N_SUPPORT
+		SetCommonHT(pAd,wdev->PhyMode,wdev->channel,wdev);
+#ifdef DOT11_VHT_AC
+		if(WMODE_CAP_AC(wdev->PhyMode))
+			SetCommonVHT(pAd,wdev->PhyMode,wdev->channel);
+#endif /* DOT11_VHT_AC */
+#endif /* DOT11_N_SUPPORT */
+}
+
+INT	SetCommonHtVht(RTMP_ADAPTER *pAd,struct wifi_dev *wdev)
+{
+	if(wdev){
+		SetHtVhtForWdev(pAd,wdev);
+	}else{
+		SetCommonHtVhtByRf(pAd,RFIC_24GHZ);
+		SetCommonHtVhtByRf(pAd,RFIC_5GHZ);
+	}
 	return TRUE;
 }
 
