@@ -37,10 +37,10 @@ INT ht_support_bw_by_channel_boundary(RTMP_ADAPTER *pAd, MAC_TABLE_ENTRY *pEntry
 	if (!peer_ie_list)
 		return supported_bw;
 
-	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_INFO,("%s(), Channel = %d, supported_bw = %d, EXTCHA = %d, PeerSupChlLen = %d\n", __FUNCTION__, 
+	MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_INFO,("%s(), Channel = %d, supported_bw = %d, extcha = %d, PeerSupChlLen = %d\n", __FUNCTION__, 
 		pEntry->wdev->channel, 
 		supported_bw,  
-		pAd->CommonCfg.RegTransmitSetting.field.EXTCHA, 
+		pEntry->wdev->extcha, 
 		peer_ie_list->SupportedChlLen));
 
 	/* So far, only check 2.4G band */
@@ -64,10 +64,10 @@ INT ht_support_bw_by_channel_boundary(RTMP_ADAPTER *pAd, MAC_TABLE_ENTRY *pEntry
 		}
 
 		/* Check Min/Max 2.4G channel boundary in BW40 case */
-		if ((supported_bw == BW_40) && (pAd->CommonCfg.RegTransmitSetting.field.EXTCHA != EXTCHA_NONE))
+		if ((supported_bw == BW_40) && (pEntry->wdev->extcha != EXTCHA_NONE))
 		{
-			if (((pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE) && ((pEntry->wdev->channel + 4) > peer_sup_chl_max_2g)) ||
-				((pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_BELOW) && ((pEntry->wdev->channel - 4) < peer_sup_chl_min_2g)))
+			if (((pEntry->wdev->extcha == EXTCHA_ABOVE) && ((pEntry->wdev->channel + 4) > peer_sup_chl_max_2g)) ||
+				((pEntry->wdev->extcha == EXTCHA_BELOW) && ((pEntry->wdev->channel - 4) < peer_sup_chl_min_2g)))
 				supported_bw = BW_20;
 		}
 	}
@@ -161,8 +161,10 @@ INT get_ht_max_mcs(RTMP_ADAPTER *pAd, UCHAR *desire_mcs, UCHAR *cap_mcs)
 
 INT get_ht_cent_ch(RTMP_ADAPTER *pAd, UINT8 *rf_bw, UINT8 *ext_ch, UCHAR Channel)
 {
+	UCHAR ExtCha = HcGetExtCha(pAd,Channel);
+
 	if ((pAd->CommonCfg.HtCapability.HtCapInfo.ChannelWidth  == BW_40) && 
-		(pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+		(ExtCha== EXTCHA_ABOVE)
 	)
 	{
 		*rf_bw = BW_40;
@@ -171,7 +173,7 @@ INT get_ht_cent_ch(RTMP_ADAPTER *pAd, UINT8 *rf_bw, UINT8 *ext_ch, UCHAR Channel
 	}
 	else if ((Channel > 2) && 
 			(pAd->CommonCfg.HtCapability.HtCapInfo.ChannelWidth  == BW_40) && 
-			(pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_BELOW))
+			(ExtCha== EXTCHA_BELOW))
 	{
 		*rf_bw = BW_40;
 		*ext_ch = EXTCHA_BELOW;
@@ -272,7 +274,8 @@ VOID set_sta_ht_cap(RTMP_ADAPTER *pAd, MAC_TABLE_ENTRY *entry, HT_CAPABILITY_IE
 */
 VOID RTMPSetHT(
 	IN RTMP_ADAPTER *pAd,
-	IN OID_SET_HT_PHYMODE *pHTPhyMode)
+	IN OID_SET_HT_PHYMODE *pHTPhyMode,
+	IN struct wifi_dev *wdev)
 {
 	UCHAR RxStream = pAd->CommonCfg.RxStream;
 	INT idx;
@@ -722,8 +725,6 @@ VOID RTMPSetIndividualHT(RTMP_ADAPTER *pAd, UCHAR apidx)
 			
 	pDesired_ht_phy->bHtEnable = TRUE;
 
-	MTWF_LOG(DBG_CAT_MLME, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("%s(): Desired_ht Enable=%d,pAd->CommonCfg.HT_Disable=%d\n",__FUNCTION__,pDesired_ht_phy->bHtEnable,pAd->CommonCfg.HT_Disable));
-					 
 	/* Decide desired Tx MCS*/
 	switch (TxStream)
 	{
@@ -862,7 +863,7 @@ VOID RTMPDisableDesiredHtInfo(RTMP_ADAPTER *pAd, UCHAR Channel)
 }
 
 
-INT	SetCommonHT(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
+INT	SetCommonHT(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel,struct wifi_dev *wdev)
 {
 	OID_SET_HT_PHYMODE SetHT;
 
@@ -876,7 +877,7 @@ INT	SetCommonHT(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 	SetHT.PhyMode = (RT_802_11_PHY_MODE)PhyMode;
 	SetHT.TransmitNo = ((UCHAR)pAd->Antenna.field.TxPath);
 	SetHT.HtMode = (UCHAR)pAd->CommonCfg.RegTransmitSetting.field.HTMODE;
-	SetHT.ExtOffset = (UCHAR)pAd->CommonCfg.RegTransmitSetting.field.EXTCHA;
+	SetHT.ExtOffset = HcGetExtCha(pAd,Channel);
 	SetHT.MCS = MCS_AUTO;
 	SetHT.BW = (UCHAR)pAd->CommonCfg.RegTransmitSetting.field.BW;
 	SetHT.STBC = (UCHAR)pAd->CommonCfg.RegTransmitSetting.field.STBC;
@@ -884,7 +885,7 @@ INT	SetCommonHT(RTMP_ADAPTER *pAd, UCHAR PhyMode, UCHAR Channel)
 	SetHT.Channel = Channel;
 	SetHT.BandIdx=0;
 
-	RTMPSetHT(pAd, &SetHT);
+	RTMPSetHT(pAd, &SetHT,wdev);
 
 #ifdef DOT11N_DRAFT3
 	if(pAd->CommonCfg.bBssCoexEnable && pAd->CommonCfg.Bss2040NeedFallBack)
