@@ -283,14 +283,27 @@ INT	Set_SecWPAPSK_Proc (
 #ifdef WSC_AP_SUPPORT
     IF_DEV_CONFIG_OPMODE_ON_AP(pAd)
     {
-        UCHAR apidx = pObj->ioctl_if;
-        BSS_STRUCT *pMBSSStruct;
+	WSC_CTRL *pWscControl = NULL;
+	
+  	if ((pObj->ioctl_if_type == INT_MAIN || pObj->ioctl_if_type == INT_MBSSID)) {
+	    UCHAR apidx = pObj->ioctl_if;
 
-        pMBSSStruct = &pAd->ApCfg.MBSSID[apidx];
-        NdisZeroMemory(pMBSSStruct->WscControl.WpaPsk, 64);
-        pMBSSStruct->WscControl.WpaPskLen = 0;
-        pMBSSStruct->WscControl.WpaPskLen = strlen(arg);
-        NdisMoveMemory(pMBSSStruct->WscControl.WpaPsk, arg, pMBSSStruct->WscControl.WpaPskLen);
+            pWscControl = &pAd->ApCfg.MBSSID[apidx].WscControl;
+        }
+#ifdef APCLI_SUPPORT
+	else if (pObj->ioctl_if_type == INT_APCLI){
+        UCHAR    apcli_idx = pObj->ioctl_if;
+
+        pWscControl = &pAd->ApCfg.ApCliTab[apcli_idx].WscControl;
+        }
+#endif /* APCLI_SUPPORT */
+
+        if (pWscControl) {
+            NdisZeroMemory(pWscControl->WpaPsk, 64);
+            pWscControl->WpaPskLen = 0;
+            pWscControl->WpaPskLen = strlen(arg);
+            NdisMoveMemory(pWscControl->WpaPsk, arg, pWscControl->WpaPskLen);
+        }
     }
 #endif /* WSC_AP_SUPPORT */
 #endif /* CONFIG_AP_SUPPORT */
@@ -1315,6 +1328,10 @@ VOID ReadApcliSecParameterFromFile (
 {
     RTMP_STRING *macptr;
     INT i, idx;
+#ifdef DBDC_MODE
+	INT apcli_idx;
+	RTMP_STRING tok_str[16];
+#endif
     struct _SECURITY_CONFIG *pSecConfig = NULL;
 
     /*ApCliAuthMode*/
@@ -1340,8 +1357,26 @@ VOID ReadApcliSecParameterFromFile (
             SetWdevEncrypMode(pSecConfig, macptr);
         }
     }
-	
-    /*ApCliWPAPSK*/
+
+#ifdef DBDC_MODE
+    for (apcli_idx = 0; apcli_idx < MAX_APCLI_NUM; apcli_idx++) {
+		pSecConfig = &pAd->ApCfg.ApCliTab[apcli_idx].wdev.SecConfig;
+		if (apcli_idx == 0)
+			snprintf(tok_str, sizeof(tok_str), "ApCliWPAPSK");
+		else
+			snprintf(tok_str, sizeof(tok_str), "ApCliWPAPSK%d", apcli_idx);
+		if(RTMPGetKeyParameter(tok_str, tmpbuf, 65, pBuffer, FALSE))
+		{
+			if (strlen(tmpbuf) < 65)
+            {
+                os_move_mem(pSecConfig->PSK, tmpbuf, strlen(tmpbuf));
+                pSecConfig->PSK[strlen(tmpbuf)] = '\0';
+            } else
+				pSecConfig->PSK[0] = '\0';
+		}
+	}
+#else
+	/*ApCliWPAPSK*/
     if (RTMPGetKeyParameter("ApCliWPAPSK", tmpbuf, 255, pBuffer, FALSE))
     {
         for (i = 0, macptr = rstrtok(tmpbuf,";"); (macptr && i < MAX_APCLI_NUM); macptr = rstrtok(NULL,";"), i++)
@@ -1357,7 +1392,7 @@ VOID ReadApcliSecParameterFromFile (
 	
         }
     }
-	
+#endif
     /*ApCliDefaultKeyID*/
     if (RTMPGetKeyParameter("ApCliDefaultKeyID", tmpbuf, 255, pBuffer, TRUE))
     {
@@ -1394,7 +1429,23 @@ VOID ReadApcliSecParameterFromFile (
             {
                 KeyType[i] = simple_strtol(macptr, 0, 10);
             }
-	
+#ifdef DBDC_MODE
+			IF_DEV_CONFIG_OPMODE_ON_AP(pAd)
+			{			
+				for (i = 0; i < MAX_APCLI_NUM; i++)
+				{
+					if (i == 0)
+						snprintf(tok_str, sizeof(tok_str), "ApCliKey%dStr", idx + 1);
+					else
+						snprintf(tok_str, sizeof(tok_str), "ApCliKey%dStr%d", idx + 1, i);
+					if (RTMPGetKeyParameter(tok_str, tmpbuf, 128, pBuffer, FALSE))
+					{
+						pSecConfig = &pAd->ApCfg.ApCliTab[i].wdev.SecConfig;
+						ParseWebKey(pSecConfig, tmpbuf, idx, 0);
+					}
+				}
+			}
+#else
             snprintf(tok_str, sizeof(tok_str), "ApCliKey%dStr", idx+1);
             /*ApCliKey1Str*/
             if(RTMPGetKeyParameter(tok_str, tmpbuf, 512, pBuffer, FALSE))
@@ -1405,6 +1456,7 @@ VOID ReadApcliSecParameterFromFile (
                     ParseWebKey(pSecConfig, macptr, idx, 0);
                 }
             }
+#endif
         }
     }
 
