@@ -262,7 +262,13 @@ VOID Announce_or_Forward_802_3_Packet(
 		to_os = wdev->rx_pkt_foward(pAd, wdev, pPacket);
 
 	if (to_os == TRUE)
+	{
+#if defined (CONFIG_WIFI_PKT_FWD)
+		if (wf_fwd_needed_hook != NULL && wf_fwd_needed_hook() == TRUE)
+			set_wf_fwd_cb(pAd, pPacket, wdev);
+#endif /* CONFIG_WIFI_PKT_FWD */	
 		announce_802_3_packet(pAd, pPacket,op_mode);
+	}
 	else {
 MTWF_LOG(DBG_CAT_RX, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("%s(): No need to send to OS!\n", __FUNCTION__));
 		RELEASE_NDIS_PACKET(pAd, pPacket, NDIS_STATUS_FAILURE);
@@ -901,6 +907,22 @@ VOID rx_eapol_frm_handle(
 	{
 		pTmpBuf = pRxBlk->pData - LENGTH_802_11;
 		NdisMoveMemory(pTmpBuf, pRxBlk->FC, LENGTH_802_11);
+#ifdef DBDC_MODE
+		{
+			RXD_BASE_STRUCT *rxd_base = (RXD_BASE_STRUCT *)pRxBlk->rmac_info;
+			REPORT_MGMT_FRAME_TO_MLME_DBDC(pAd, pRxBlk->wcid,
+										pTmpBuf,
+										pRxBlk->DataSize + LENGTH_802_11,
+										pRxBlk->rx_signal.raw_rssi[0],
+										pRxBlk->rx_signal.raw_rssi[1],
+										pRxBlk->rx_signal.raw_rssi[2],
+										pRxBlk->rx_signal.raw_rssi[3],
+										0,
+										(rxd_base != NULL) ? rxd_base->RxD1.ChFreq : 0,
+										pRxBlk->OpMode,
+										wdev);
+		}
+#else
 		REPORT_MGMT_FRAME_TO_MLME(pAd, pRxBlk->wcid,
 							pTmpBuf,
 							pRxBlk->DataSize + LENGTH_802_11,
@@ -911,6 +933,7 @@ VOID rx_eapol_frm_handle(
 							0,
                             pRxBlk->OpMode,
                             wdev);
+#endif //DBDC_MODE
 
 		MTWF_LOG(DBG_CAT_RX, DBG_SUBCAT_ALL, DBG_LVL_TRACE,
 			     ("!!! report EAPOL DATA to MLME (len=%d) !!!\n",
@@ -991,16 +1014,19 @@ MTWF_LOG(DBG_CAT_FPGA, DBG_SUBCAT_ALL, DBG_LVL_NOISY, ("-->%s()\n", __FUNCTION__
 		goto done;
 #endif /* DOT11W_PMF_SUPPORT */
 
-	if (VALID_UCAST_ENTRY_WCID(pAd, pRxBlk->wcid))
-	{
+	if (VALID_UCAST_ENTRY_WCID(pAd, pRxBlk->wcid)) {
 		pEntry = &pAd->MacTab.Content[pRxBlk->wcid];
-	}
-	else
-	{
-#ifdef CONFIG_AP_SUPPORT			
+		if (NdisCmpMemory(pEntry->Addr, pRxBlk->Addr2, MAC_ADDR_LEN)) {
+#ifdef CONFIG_AP_SUPPORT	
+			pEntry = MacTableLookup(pAd, pRxBlk->Addr2);
+#endif
+			if (pEntry)
+				pRxBlk->wcid = pEntry->wcid;
+		}
+	} else {
+#ifdef CONFIG_AP_SUPPORT	
 		pEntry = MacTableLookup(pAd, pRxBlk->Addr2);
 #endif
-
 		if (pEntry)
 			pRxBlk->wcid = pEntry->wcid;
 	}
@@ -1153,6 +1179,22 @@ MTWF_LOG(DBG_CAT_FPGA, DBG_SUBCAT_ALL, DBG_LVL_NOISY, ("-->%s()\n", __FUNCTION__
 
 #ifdef CONFIG_AP_SUPPORT
 	/* Signal in MLME_QUEUE isn't used, therefore take this item to save min SNR. */
+#ifdef DBDC_MODE
+{
+	RXD_BASE_STRUCT *rxd_base = (RXD_BASE_STRUCT *)pRxBlk->rmac_info;
+	REPORT_MGMT_FRAME_TO_MLME_DBDC(pAd, pRxBlk->wcid,
+						  FC,
+						  pRxBlk->DataSize,
+						  pRxBlk->rx_signal.raw_rssi[0],
+						  pRxBlk->rx_signal.raw_rssi[1],
+						  pRxBlk->rx_signal.raw_rssi[2],
+						  pRxBlk->rx_signal.raw_rssi[3],
+						  min(pRxBlk->rx_signal.raw_snr[0], pRxBlk->rx_signal.raw_snr[1]),
+						  (rxd_base != NULL) ? rxd_base->RxD1.ChFreq : 0,
+						  op_mode,
+						  pAd->wdev_list[0]);
+}
+#else
 	REPORT_MGMT_FRAME_TO_MLME(pAd, pRxBlk->wcid,
 						FC,
 						pRxBlk->DataSize,
@@ -1163,6 +1205,7 @@ MTWF_LOG(DBG_CAT_FPGA, DBG_SUBCAT_ALL, DBG_LVL_NOISY, ("-->%s()\n", __FUNCTION__
 						min(pRxBlk->rx_signal.raw_snr[0], pRxBlk->rx_signal.raw_snr[1]),
                         op_mode,
                         pAd->wdev_list[0]);
+#endif //DBDC_MODE
 #endif
 
 
@@ -1407,6 +1450,7 @@ static INT rtmp_chk_rx_err(RTMP_ADAPTER *pAd, RX_BLK *pRxBlk)
 		RXD_BASE_STRUCT *rxd_base = (RXD_BASE_STRUCT *)pRxBlk->rmac_info;
 
 		if (rxd_base->RxD2.IcvErr) {
+			UCHAR band_idx = (rxd_base->RxD1.ChFreq > 14) ? 1 : 0; 
 			MTWF_LOG(DBG_CAT_RX, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("ICV Error\n"));
 			if(pRxBlk->Addr1 != NULL)
 			    MTWF_LOG(DBG_CAT_RX, DBG_SUBCAT_ALL, DBG_LVL_ERROR,
@@ -1414,6 +1458,7 @@ static INT rtmp_chk_rx_err(RTMP_ADAPTER *pAd, RX_BLK *pRxBlk)
 			if(pRxBlk->Addr2 != NULL)
 			    MTWF_LOG(DBG_CAT_RX, DBG_SUBCAT_ALL, DBG_LVL_ERROR,
 					    ("Addr2 = %02x:%02x:%02x:%02x:%02x:%02x\n",PRINT_MAC(pRxBlk->Addr2)));
+			INC_COUNTER64(pAd->WlanCounters[band_idx].RxICVErrorCount);
 			dump_rxblk(pAd, pRxBlk);
 			return NDIS_STATUS_FAILURE;
 		}
@@ -1719,6 +1764,14 @@ MTWF_LOG(DBG_CAT_RX, DBG_SUBCAT_ALL, DBG_LVL_INFO, ("%s():  No DATA!\n", __FUNCT
 					return FALSE;
 				}
 			}
+
+			/* drop received packet which come from apcli */
+			/* check if sa is to apcli */
+			if (MAC_ADDR_EQUAL(pEntry->wdev->if_addr, pRxBlk->Addr3))
+			{
+				MTWF_LOG(DBG_CAT_RX, DBG_SUBCAT_ALL, DBG_LVL_TRACE,("%s[%d]SA is from APCLI=%d\n\r",__func__,__LINE__,pEntry->wdev->func_idx));
+				return FALSE;	/* give up this frame */
+			}
 		}
         RX_BLK_SET_FLAG(pRxBlk, fRX_AP);
         goto ret;
@@ -2015,6 +2068,18 @@ VOID dev_rx_802_3_data_frm(RTMP_ADAPTER *pAd, RX_BLK *pRxBlk)
 	if (VALID_UCAST_ENTRY_WCID(pAd, pRxBlk->wcid))
 	{
 		pEntry = &pAd->MacTab.Content[pRxBlk->wcid];
+		/*double confirm wcid lookup result*/
+#ifdef OUI_CHECK_SUPPORT
+		if(pAd->MacTab.oui_mgroup_cnt > 0){
+			if (NdisCmpMemory(pEntry->Addr, pRxBlk->Addr2, MAC_ADDR_LEN)) {
+#ifdef CONFIG_AP_SUPPORT	
+				pEntry = MacTableLookup(pAd, pRxBlk->Addr2);
+#endif
+				if (pEntry)
+					pRxBlk->wcid = pEntry->wcid;
+			}
+		}
+#endif /*OUI_CHECK_SUPPORT*/
 	}
 	else
 	{
@@ -2232,20 +2297,17 @@ VOID dev_rx_data_frm(RTMP_ADAPTER *pAd, RX_BLK *pRxBlk)
 //---Add by shiangf for debug
 
 	// TODO: shiang-usw, check wcid if we are repeater mode! when in Repeater mode, wcid is get by "A2" + "A1"
-	if (VALID_UCAST_ENTRY_WCID(pAd, pRxBlk->wcid))
-	{
+	if (VALID_UCAST_ENTRY_WCID(pAd, pRxBlk->wcid)) {
 		pEntry = &pAd->MacTab.Content[pRxBlk->wcid];
-        //Carter, the below segment shall apply to unify mac chipset. 2014-0411
-        if (NdisCmpMemory(pEntry->Addr, pRxBlk->Addr2, MAC_ADDR_LEN))
-        {
+		//Carter, the below segment shall apply to unify mac chipset. 2014-0411
+		if (NdisCmpMemory(pEntry->Addr, pRxBlk->Addr2, MAC_ADDR_LEN)) {
 #ifdef CONFIG_AP_SUPPORT	
-            pEntry = MacTableLookup(pAd, pRxBlk->Addr2);
+			pEntry = MacTableLookup(pAd, pRxBlk->Addr2);
 #endif
-            if (pEntry)
-                pRxBlk->wcid = pEntry->wcid;
-        }
-	}
-	else {
+			if (pEntry)
+				pRxBlk->wcid = pEntry->wcid;
+		}
+	} else {
 		/* IOT issue with Marvell test bed AP
 		    Marvell AP ResetToOOB and do wps.
 		    Because of AP send EAP Request too fast and without retransmit.
@@ -2254,7 +2316,7 @@ VOID dev_rx_data_frm(RTMP_ADAPTER *pAd, RX_BLK *pRxBlk)
 		    The patch lookup pEntry from MacTable.
 		*/
 #ifdef CONFIG_AP_SUPPORT	
-        pEntry = MacTableLookup(pAd, pRxBlk->Addr2);
+		pEntry = MacTableLookup(pAd, pRxBlk->Addr2);
 #endif
 		if (pEntry)
 			pRxBlk->wcid = pEntry->wcid;
@@ -2668,11 +2730,6 @@ NDIS_STATUS header_packet_process(
     PNDIS_PACKET pRxPacket,
     RX_BLK *pRxBlk)
 {
-#ifdef SMART_CARRIER_SENSE_SUPPORT
-       struct wifi_dev *wdev;
-        UCHAR wdev_idx=0;
-        UCHAR BandIdx=0;
-#endif /* SMART_CARRIER_SENSE_SUPPORT */
 
 #ifdef MT_MAC
     if (pAd->chipCap.hif_type == HIF_MT) {
@@ -2818,14 +2875,7 @@ NDIS_STATUS header_packet_process(
     pAd->RalinkCounters.OneSecReceivedByteCount += pRxBlk->DataSize;
     pAd->RalinkCounters.RxCount++;
     pAd->RalinkCounters.OneSecRxCount++;
-#ifdef SMART_CARRIER_SENSE_SUPPORT
-    wdev_idx = RTMP_GET_PACKET_WDEV(pRxBlk->pRxPacket);
 
-	wdev = pAd->wdev_list[wdev_idx];
-    
-    BandIdx = HcGetBandByWdev(wdev);
-    pAd->SCSCtrl.OneSecRxByteCount[BandIdx] += pRxBlk->DataSize;;
-#endif /* SMART_CARRIER_SENSE_SUPPORT */    
 #ifdef CONFIG_ATE
 	if (ATE_ON(pAd)) {
 		MT_ATERxDoneHandle(pAd, pRxBlk);
