@@ -1575,6 +1575,25 @@ VOID AsicUpdateRxWCIDTable(RTMP_ADAPTER *pAd, USHORT WCID, UCHAR *pAddr, BOOLEAN
 }
 
 
+#ifdef TXBF_SUPPORT
+VOID AsicUpdateClientBfCap(RTMP_ADAPTER *pAd, PMAC_TABLE_ENTRY pMacEntry)
+{
+#ifdef MT_MAC
+	if (pAd->archOps.archUpdateRxWCIDTable)
+	{
+		return pAd->archOps.archUpdateClientBfCap(pAd, pMacEntry);
+	}
+	else
+	{
+		AsicNotSupportFunc(pAd, __FUNCTION__);
+		return;
+	}
+#endif
+	AsicNotSupportFunc(pAd, __FUNCTION__);
+}
+#endif /* TXBF_SUPPORT */
+
+
 /*
 	========================================================================
 	Description:
@@ -2554,7 +2573,8 @@ INT32 AsicStaRecUpdate(
 	UINT8 WlanIdx,
 	UINT32 ConnectionType,
 	UINT8 ConnectionState,
-	UINT32 EnableFeature)
+	UINT32 EnableFeature,
+	UINT8 IsNewSTARec)
 {
 	if (pAd->archOps.archSetStaRec)
 	{
@@ -2600,6 +2620,7 @@ INT32 AsicStaRecUpdate(
 		StaCfg.ucBssIndex = BssIndex;
 		StaCfg.ucWlanIdx = WlanIdx;
 		StaCfg.pEntry = pEntry;
+		StaCfg.IsNewSTARec = IsNewSTARec;
 
         /*tracking the starec input history*/
         if (pAd->StaRecTracking.RecordLoop >= MAX_STA_REC_HISTORY_RECORD)
@@ -2712,7 +2733,6 @@ INT32 AsicBfStaRecUpdate(
 	RTMP_ADAPTER *pAd,
     UCHAR        ucPhyMode,
     UCHAR        ucBssIdx,
-    IE_LISTS     *pIeList,
 	UCHAR        ucWlanIdx)
 {
     if (pAd->archOps.archSetStaRec)
@@ -2721,7 +2741,6 @@ INT32 AsicBfStaRecUpdate(
 	                                   pAd,
 	                                   ucPhyMode,
 	                                   ucBssIdx,
-	                                   pIeList,
 	                                   ucWlanIdx);
     }
     else
@@ -2738,7 +2757,7 @@ INT32 AsicBfStaRecRelease(
     UCHAR        ucBssIdx,
 	UCHAR        ucWlanIdx)
 {
-    if (pAd->archOps.archSetStaRec)
+    if (pAd->chipOps.BfStaRecRelease)
 	{
 	    return pAd->chipOps.BfStaRecRelease(
 	                                   pAd,
@@ -2759,7 +2778,7 @@ INT32 AsicBfPfmuMemAlloc(
     UCHAR ucSu_Mu,
     UCHAR ucWlanId)
 {
-    if (pAd->archOps.archSetStaRec)
+    if (pAd->chipOps.BfPfmuMemAlloc)
 	{
 	    return pAd->chipOps.BfPfmuMemAlloc(
 	                                   pAd,
@@ -2779,7 +2798,7 @@ INT32 AsicBfPfmuMemRelease(
 	RTMP_ADAPTER *pAd,
     UCHAR ucWlanId)
 {
-    if (pAd->archOps.archSetStaRec)
+    if (pAd->chipOps.BfPfmuMemRelease)
 	{
 	    return pAd->chipOps.BfPfmuMemRelease(
 	                                   pAd,
@@ -2802,7 +2821,7 @@ INT32 AsicTxBfTxApplyCtrl(
     BOOLEAN fgMuTxBf,
     BOOLEAN fgPhaseCali)
 {
-    if (pAd->archOps.archSetStaRec)
+    if (pAd->chipOps.TxBfTxApplyCtrl)
 	{
 	    return pAd->chipOps.TxBfTxApplyCtrl(
 	                                   pAd,
@@ -2819,6 +2838,48 @@ INT32 AsicTxBfTxApplyCtrl(
 
     }
 }
+
+
+INT32 AsicTxBfApClientCluster(
+	RTMP_ADAPTER *pAd,
+    UCHAR   ucWlanId,
+    UCHAR   ucPfmuId)
+{
+    if (pAd->chipOps.BfApClientCluster)
+	{
+	    return pAd->chipOps.BfApClientCluster(
+	                                   pAd,
+	                                   ucWlanId,
+	                                   ucPfmuId);
+    }
+    else
+    {
+        AsicNotSupportFunc(pAd, __FUNCTION__);
+		return FALSE;
+
+    }
+}
+
+
+INT32 AsicTxBfHwEnStatusUpdate(
+	RTMP_ADAPTER *pAd,
+    BOOLEAN   fgETxBf,
+    BOOLEAN   fgITxBf)
+{
+    if (pAd->chipOps.BfHwEnStatusUpdate)
+	{
+	    return pAd->chipOps.BfHwEnStatusUpdate(
+	                                   pAd,
+	                                   fgETxBf,
+	                                   fgITxBf);
+    }
+    else
+    {
+        AsicNotSupportFunc(pAd, __FUNCTION__);
+		return FALSE;
+
+    }
+}
 #endif /* MT_MAC && TXBF_SUPPORT */
 
 INT32 AsicRadioOnOffCtrl(RTMP_ADAPTER *pAd, UINT8 ucDbdcIdx, UINT8 ucRadio)
@@ -2997,6 +3058,32 @@ AsicThermalProtect(
 
     return 0;
 }
+
+INT32
+AsicThermalProtectAdmitDuty(
+	RTMP_ADAPTER *pAd,
+	UINT32 u4Lv0Duty,
+	UINT32 u4Lv1Duty,
+	UINT32 u4Lv2Duty,
+	UINT32 u4Lv3Duty
+	)
+{
+
+#ifdef MT_MAC
+	if (pAd->chipCap.hif_type == HIF_MT)
+	{
+
+		INT32 ret = 0;
+		ret = MtCmdThermalProtectAdmitDuty(pAd,u4Lv0Duty, u4Lv1Duty, u4Lv2Duty, u4Lv3Duty);
+
+		return ret;
+	}
+#endif
+
+	AsicNotSupportFunc(pAd, __FUNCTION__);
+
+    return 0;
+}
 #endif /* THERMAL_PROTECT_SUPPORT */
 
 
