@@ -301,7 +301,7 @@ VOID MtAsicUpdateRxWCIDTableByFw(
 			/* Tag = 0 */
 			rWtblGeneric.ucQos = 1;
 #if defined(MT7615) || defined(MT7622)
-			rWtblGeneric.ucBafEn = 1;
+			rWtblGeneric.ucBafEn = 0;
 #endif /* defined(MT7615) || defined(MT7622) */
 
 			/* Tag = 2 */
@@ -432,13 +432,20 @@ VOID MtAsicUpdateRxWCIDTableByFw(
 		&u4TotalTlvLen,
 		&ucTotalTlvNumber);
 #ifdef TXBF_SUPPORT
-    pTempBuffer = pTlvAppend(
+    MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_ERROR,
+			("%s: u2PfmuId = 0x%x\n",
+			__FUNCTION__, pAd->rStaRecBf.u2PfmuId));
+
+    if (pAd->rStaRecBf.u2PfmuId != 0xFFFF)
+    {
+        pTempBuffer = pTlvAppend(
 		pTempBuffer,
 		cpu_to_le16(WTBL_BF),
 		cpu_to_le16(sizeof(CMD_WTBL_BF_T)),
 		&rWtblBf,
 		&u4TotalTlvLen,
 		&ucTotalTlvNumber);
+    }
 #endif /* TXBF_SUPPORT */
     pTempBuffer = pTlvAppend(
 		pTempBuffer,
