@@ -3357,10 +3357,11 @@ VOID RtmpPrepareHwNullFrame(
 		}
 		else
 		{
-			mac_info.PID = (UCHAR)pAd->CommonCfg.MlmeTransmit.field.MCS;
-			mac_info.TxRate = (UCHAR)pAd->CommonCfg.MlmeTransmit.field.MCS;
+			struct dev_rate_info *rate = &pEntry->wdev.rate;
+			mac_info.PID = (UCHAR)rate->MlmeTransmit.field.MCS;
+			mac_info.TxRate = (UCHAR)rate->MlmeTransmit.field.MCS;
 			write_tmac_info(pAd, tmac_info, &mac_info,
-							&pAd->CommonCfg.MlmeTransmit);
+							&rate->MlmeTransmit);
 		}
 
 		if (bWaitACK) {
