@@ -48,8 +48,10 @@
 
 #ifdef ERR_RECOVERY
 #define MT_MCU_INT_EVENT    (MT_HIF_BASE + 0x01f8)
-#define MCU_INT_PDMA0_STOP_DONE	BIT(0)
-#define MCU_INT_PDMA0_INIT_DONE	BIT(1)
+#define MCU_INT_PDMA0_STOP_DONE	        BIT(0)
+#define MCU_INT_PDMA0_INIT_DONE	        BIT(1)
+#define MCU_INT_SER_TRIGGER_FROM_HOST   BIT(2)
+#define MCU_INT_PDMA0_RECOVERY_DONE	    BIT(3)
 #endif /* ERR_RECOVERY .*/
 
 #define MT_INT_SOURCE_CSR	(MT_HIF_BASE + 0x0200)
