@@ -142,6 +142,10 @@ VOID AsicUpdateRxWCIDTable(struct _RTMP_ADAPTER *pAd, USHORT WCID, UCHAR *pAddr,
 VOID AsicUpdateBASession(struct _RTMP_ADAPTER *pAd, UCHAR wcid, UCHAR tid, UINT16 sn, UCHAR basize, BOOLEAN isAdd, INT ses_type);
 UINT16 AsicGetTidSn(struct _RTMP_ADAPTER *pAd, UCHAR wcid, UCHAR tid);
 
+#ifdef TXBF_SUPPORT
+VOID AsicUpdateClientBfCap(struct _RTMP_ADAPTER *pAd, struct _MAC_TABLE_ENTRY *pEntry);
+#endif /* TXBF_SUPPORT */
+
 VOID AsicUpdateWcidAttributeEntry(
 	struct _RTMP_ADAPTER *pAd,
 	IN	UCHAR BssIdx,
@@ -187,7 +191,8 @@ INT32 AsicStaRecUpdate(
 	UINT8 ucWlanIdx,
 	UINT32 ConnectionType,
 	UINT8 ConnectionState,
-	UINT32 u4EnableFeature);
+	UINT32 u4EnableFeature,
+	UINT8 IsNewSTARec);
 
 INT32 AsicRaParamStaRecUpdate(
 	struct _RTMP_ADAPTER *pAd,
@@ -204,7 +209,6 @@ INT32 AsicBfStaRecUpdate(
 	struct _RTMP_ADAPTER *pAd,
     UCHAR                ucPhyMode,
     UCHAR                ucBssIdx,
-    IE_LISTS             *pIeList,
 	UCHAR                ucWlanIdx);
 
 INT32 AsicBfStaRecRelease(
@@ -228,6 +232,16 @@ INT32 AsicTxBfTxApplyCtrl(
     BOOLEAN              fgITxBf, 
     BOOLEAN              fgMuTxBf,
     BOOLEAN              fgPhaseCali);
+
+INT32 AsicTxBfApClientCluster(
+	struct _RTMP_ADAPTER *pAd,
+    UCHAR                ucWlanId,
+    UCHAR                ucPfmuId);
+
+INT32 AsicTxBfHwEnStatusUpdate(
+	struct _RTMP_ADAPTER *pAd,
+    BOOLEAN              fgETxBf,
+    BOOLEAN              fgITxBf);
 #endif /* MT_MAC && TXBF_SUPPORT */
 
 #define AsicBssInfoReNew(pAd, bss_info_argument) AsicBssInfoUpdate(pAd, bss_info_argument)
@@ -267,6 +281,15 @@ AsicThermalProtect(
     UINT8 RFOffEn,
     CHAR RFOffTh,
     UINT8 ucType);
+
+INT32
+AsicThermalProtectAdmitDuty(
+	RTMP_ADAPTER *pAd,
+	UINT32 u4Lv0Duty,
+	UINT32 u4Lv1Duty,
+	UINT32 u4Lv2Duty,
+	UINT32 u4Lv3Duty
+	);
 #endif /* THERMAL_PROTECT_SUPPORT */
 
 
@@ -540,6 +563,11 @@ typedef struct _RTMP_ARCH_OP {
 VOID (*archUpdateBcnToAsicMethod)(struct _RTMP_ADAPTER *pAd, INT apidx, ULONG FrameLen, ULONG UpdatePos);
 
 	VOID (*archUpdateRxWCIDTable)(struct _RTMP_ADAPTER *pAd, MT_WCID_TABLE_INFO_T WtblInfo);
+
+#ifdef TXBF_SUPPORT
+    VOID (*archUpdateClientBfCap)(struct _RTMP_ADAPTER *pAd, struct _MAC_TABLE_ENTRY *pEntry);
+#endif
+
 #if defined(RTMP_MAC) || defined(RLT_MAC)
 	INT32 (* archUpdateBASession)(struct _RTMP_ADAPTER *pAd, UCHAR wcid, UCHAR tid, UCHAR basize, BOOLEAN isAdd, INT ses_type);
 #endif /* RTMP_MAC || RLT_MAC */
