@@ -66,6 +66,7 @@ enum ASIC_CAP {
 	fASIC_CAP_BA_OFFLOAD = 0x100,
 	fASIC_CAP_HW_DAMSDU = 0x200,
 	fASIC_CAP_IGMP_SNOOP_OFFLOAD = 0x400,
+	fASIC_CAP_WMM_PKTDETECT_OFFLOAD = 0x00000800,
 };
 
 enum PHY_CAP {
@@ -330,7 +331,7 @@ typedef struct _RTMP_CHIP_OP {
 
 #if (defined(WOW_SUPPORT) && defined(RTMP_MAC_USB)) || defined(NEW_WOW_SUPPORT) || defined(MT_WOW_SUPPORT)
 	VOID (*AsicWOWEnable)(struct _RTMP_ADAPTER *ad, struct _STA_ADMIN_CONFIG *pStaCfg);
-	VOID (*AsicWOWDisable)(struct _RTMP_ADAPTER *ad);
+	VOID (*AsicWOWDisable)(struct _RTMP_ADAPTER *ad, struct _STA_ADMIN_CONFIG *pStaCfg);
 	VOID (*AsicWOWInit)(struct _RTMP_ADAPTER *ad);
 #endif /* (defined(WOW_SUPPORT) && defined(RTMP_MAC_USB)) || defined(NEW_WOW_SUPPORT) */
 
@@ -352,12 +353,14 @@ typedef struct _RTMP_CHIP_OP {
     VOID (*setETxBFCap)(struct _RTMP_ADAPTER *ad, struct _TXBF_STATUS_INFO  *pTxBfInfo);
     VOID (*setVHTETxBFCap)(struct _RTMP_ADAPTER *ad, struct _TXBF_STATUS_INFO  *pTxBfInfo);
 #ifdef MT_MAC
-    INT32 (*BfStaRecUpdate)(struct _RTMP_ADAPTER *ad, UCHAR ucPhyMode, UCHAR ucBssIdx, IE_LISTS *pIeList, UINT8 ucWlanIdx);
+    INT32 (*BfStaRecUpdate)(struct _RTMP_ADAPTER *ad, UCHAR ucPhyMode, UCHAR ucBssIdx, UINT8 ucWlanIdx);
     INT32 (*BfStaRecRelease)(struct _RTMP_ADAPTER *ad, UCHAR ucBssIdx, UINT8 ucWlanIdx);
     INT32 (*BfPfmuMemAlloc)(struct _RTMP_ADAPTER *ad, UCHAR ucSu_Mu, UCHAR ucWlanId);
     INT32 (*BfPfmuMemRelease)(struct _RTMP_ADAPTER *ad, UCHAR ucWlanId);
+    INT32 (*BfHwEnStatusUpdate)(struct _RTMP_ADAPTER *ad, BOOLEAN fgETxBf, BOOLEAN fgITxBf);
     INT32 (*TxBfTxApplyCtrl)(struct _RTMP_ADAPTER *pAd, UCHAR ucWlanId, BOOLEAN fgETxBf, BOOLEAN fgITxBf, BOOLEAN fgMuTxBf, BOOLEAN fgPhaseCali);
     INT32 (*archSetAid)(struct _RTMP_ADAPTER *pAd, UINT16 Aid);
+    INT32 (*BfApClientCluster)(struct _RTMP_ADAPTER *pAd, UCHAR ucWlanId, UCHAR ucPfmuId);
 #endif /* MT_MAC */
 #endif /* TXBF_SUPPORT */
 #ifdef SMART_CARRIER_SENSE_SUPPORT
@@ -715,6 +718,10 @@ typedef struct _RTMP_CHIP_CAP {
 
     UINT8 APPSMode;
 
+#if (defined(WOW_SUPPORT) && defined(RTMP_MAC_USB)) || defined(NEW_WOW_SUPPORT) || defined(MT_WOW_SUPPORT)
+    UINT8   nWakeupInterface;
+#endif /* MT_WOW_SUPPORT */
+
 #ifdef RX_SCATTER
 	BOOLEAN RxDMAScatter;
 #endif /* RX_SCATTER */
