@@ -1965,6 +1965,7 @@ typedef struct wtbl_entry {
 
 // TODO: shiang-MT7615, move to top.h!
 #define MCU_CFG_BASE		0x2000
+#define MCU_COM_REG1	    (MCU_CFG_BASE + 0x204)
 #define MCU_PCIE_REMAP_1	(MCU_CFG_BASE + 0x500)
 #define REMAP_1_OFFSET_MASK (0x3ffff)
 #define GET_REMAP_1_OFFSET(p) (((p) & REMAP_1_OFFSET_MASK))
@@ -2098,12 +2099,16 @@ typedef struct __DMAC_TX_CNT_INFO {
 #define DBDC_BAND_NUM		1
 #endif /* DBDC_MODE */
 
+#define DBDC_BAND0		0
+#define DBDC_BAND1		1
+
 struct _RTMP_ADAPTER;
 
 VOID dump_mt_mac_cr(struct _RTMP_ADAPTER *pAd);
 INT dump_dmac_mib_info(struct _RTMP_ADAPTER *pAd, RTMP_STRING *arg);
 INT dump_dmac_pse_info(struct _RTMP_ADAPTER *pAd);
 INT dump_dmac_pse_data(struct _RTMP_ADAPTER *pAd, UINT32 StartFID, UINT32 FrameNums);
+VOID Update_Mib_Bucket_One_Sec(struct _RTMP_ADAPTER *pAd);
 
 INT mt_wtbl_init_ByFw(struct _RTMP_ADAPTER *pAd);
 INT mt_wtbl_init_ByDriver(struct _RTMP_ADAPTER *pAd);
