@@ -134,7 +134,7 @@ typedef struct usb_ctrlrequest devctrlrequest;
 #endif
 
 
-#define AP_DRIVER_VERSION			"4.4.0.1"
+#define AP_DRIVER_VERSION			"4.4.0.2"
 #ifdef MULTIPLE_CARD_SUPPORT
 #define CARD_INFO_PATH			"/etc/Wireless/RT2860AP/RT2860APCard.dat"
 #endif /* MULTIPLE_CARD_SUPPORT */
@@ -816,8 +816,6 @@ void linux_pci_unmap_single(void *handle, ra_dma_addr_t dma_addr, size_t size, i
 #else
 #define CONFIG_WIFI_PAGE_ALLOC_SKB
 #endif
-#else
-#define CONFIG_WIFI_PAGE_ALLOC_SKB
 #endif
 
 #ifdef  CONFIG_WIFI_PAGE_ALLOC_SKB
@@ -1307,11 +1305,12 @@ extern int (*wf_fwd_rx_hook) (struct sk_buff *skb);
 extern unsigned char (*wf_fwd_entry_insert_hook) (struct net_device *src, struct net_device *dest, void *adapter);
 extern unsigned char (*wf_fwd_entry_delete_hook) (struct net_device *src, struct net_device *dest, unsigned char link_down);
 extern void (*wf_fwd_set_cb_num) (unsigned int band_cb_num, unsigned int receive_cb_num);
-extern bool (*wf_fwd_tx_check_hook) (void *adapter, struct net_device *tx_dev, struct sk_buff *skb);
 
 extern void (*wf_fwd_get_rep_hook) (unsigned char idx);
 extern void (*wf_fwd_pro_active_hook) (void);
 extern void (*wf_fwd_pro_halt_hook) (void);
+extern void (*wf_fwd_pro_enabled_hook) (void);
+extern void (*wf_fwd_pro_disabled_hook) (void);
 extern void (*wf_fwd_access_schedule_active_hook) (void);
 extern void (*wf_fwd_access_schedule_halt_hook) (void);
 extern void (*wf_fwd_hijack_active_hook) (void);
@@ -1326,6 +1325,7 @@ extern void (*wf_fwd_insert_bridge_mapping_hook) (struct sk_buff *skb);
 extern void (*wf_fwd_insert_repeater_mapping_hook) (void *adapter, void *lock, void *cli_mapping, void *map_mapping, void *ifAddr_mapping);
 extern int (*wf_fwd_search_mapping_table_hook) (struct sk_buff *skb, struct APCLI_BRIDGE_LEARNING_MAPPING_STRUCT **tbl_entry);
 extern void (*wf_fwd_delete_entry_inform_hook) (unsigned char *addr);
+extern bool (*wf_fwd_needed_hook) (void);
 #endif /* CONFIG_WIFI_PKT_FWD */
 
 void RTMP_GetCurrentSystemTime(LARGE_INTEGER *time);
