@@ -376,7 +376,7 @@ typedef enum _BF_SOUNDING_MODE
     MU_SOUNDING,
     SU_PERIODIC_SOUNDING,
     MU_PERIODIC_SOUNDING,
-    AUTO_SU_PERIODIC_SOUNDING,
+    BF_PROCESSING,
 } BF_SOUNDING_MODE;
 #endif /* #ifdef MT_MAC */
 
@@ -400,8 +400,9 @@ typedef enum _BF_ACTION_CATEGORY
     BF_IBF_PHASE_COMP,
     BF_LNA_GAIN_CONFIG,
     BF_PROFILE_WRITE_20M_ALL,
-    BF_TX_POWER_BACK_OFF,
-    BF_BFEE_SOUNDING_REQ_POLLING
+    BF_APCLIENT_CLUSTER,
+    BF_AWARE_CTRL,
+    BF_HW_ENABLE_STATUS_UPDATE
 } BF_ACTION_CATEGORY;   
 
 // Indices of valid rows in Implicit and Explicit profiles for 20 and 40 MHz
@@ -684,7 +685,6 @@ INT32 mt_AsicBfStaRecUpdate(
    IN struct _RTMP_ADAPTER *pAd, 
    IN UCHAR                ucPhyMode,
    IN UCHAR                ucBssIdx,
-   IN IE_LISTS             *pIeList,
    IN UCHAR                ucWlanIdx);
 
 INT32 mt_AsicBfStaRecRelease(
@@ -692,6 +692,10 @@ INT32 mt_AsicBfStaRecRelease(
    IN UCHAR                ucBssIdx,
    IN UCHAR                ucWlanIdx);
 
+VOID mt_AsicClientBfCap(
+   IN struct _RTMP_ADAPTER *pAd, 
+   IN struct _MAC_TABLE_ENTRY *pEntry);
+
 UCHAR AsicTxBfEnCondProc(
 	IN	struct _RTMP_ADAPTER *pAd, 
 	IN	TXBF_STATUS_INFO     *pTxBfInfo);
