@@ -646,15 +646,15 @@ BOOLEAN CFG80211DRV_OpsBeaconAdd(VOID *pAdOrg, VOID *pData)
 	pAd->MacTab.Content[0].HTPhyMode.field.MCS = 3;
 #ifdef RT_CFG80211_P2P_MULTI_CHAN_SUPPORT
 #ifdef DOT11_N_SUPPORT
-	SetCommonHtVht(pAd);
+	SetCommonHtVht(pAd,NULL);
 #endif /* DOT11_N_SUPPORT */
 
 	/*In MCC  & p2p GO not support VHT now, */
 	/*change here for support P2P GO 40 BW*/
 	/*	pAd->CommonCfg.vht_bw = 0;*/
-	if(pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_BELOW)
+	if(wdev->extcha== EXTCHA_BELOW)
 		pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel - 2;
-	else if (pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+	else if (wdev->extcha == EXTCHA_ABOVE)
 		pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel + 2;
 	else
 	pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel;
@@ -971,10 +971,11 @@ BOOLEAN CFG80211DRV_ApKeyAdd(
 				//MTWF_LOG(DBG_CAT_AP, DBG_SUBCAT_ALL, DBG_LVL_TRACE, ("iversontest pWdev->bw %d  \n",pWdev->bw));
 				if (pWdev->bw == 0)
 				{
-					pAd->CommonCfg.RegTransmitSetting.field.EXTCHA  = EXTCHA_BELOW;
+					wdev->extcha = EXTCHA_BELOW;					
+					HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
 					pAd->CommonCfg.RegTransmitSetting.field.BW = BW_40;
 					pAd->CommonCfg.HT_Disable = 0;
-					SetCommonHtVht(pAd);
+					SetCommonHtVht(pAd,NULL);
 				}
 
 #endif /* RT_CFG80211_P2P_MULTI_CHAN_SUPPORT */
