@@ -674,10 +674,11 @@ VOID RTMP_CFG80211_VirtualIF_Remove(
 			pwdev->extcha = EXTCHA_NONE;
 
 /*after p2p cli connect , neet to change to default configure*/
-			pAd->CommonCfg.RegTransmitSetting.field.EXTCHA  = EXTCHA_BELOW;
+			wdev->extcha  = EXTCHA_BELOW;
+			HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
 			pAd->CommonCfg.RegTransmitSetting.field.BW = BW_40;
 			pAd->CommonCfg.HT_Disable = 0;
-			SetCommonHtVht(pAd);
+			SetCommonHtVht(pAd,NULL);
 
 
 #endif /* RT_CFG80211_P2P_MULTI_CHAN_SUPPORT */
@@ -715,7 +716,7 @@ VOID RTMP_CFG80211_VirtualIF_Remove(
 			wdev->channel= 0;
 			wdev->bw = HT_BW_20;
 			wdev->extcha = EXTCHA_NONE;
-
+			HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
 #endif /* RT_CFG80211_P2P_MULTI_CHAN_SUPPORT */
 
 			OPSTATUS_CLEAR_FLAG(pAd, fOP_AP_STATUS_MEDIA_STATE_CONNECTED);
