@@ -26,6 +26,7 @@
 #ifdef RT_CFG80211_SUPPORT
 
 #include "rt_config.h"
+#define BSSID_WCID_TO_REMOVE 1 //Pat:TODO
 
 extern struct notifier_block cfg80211_netdev_notifier;
 
@@ -563,13 +564,8 @@ BOOLEAN CFG80211DRV_OpsSetChannel(RTMP_ADAPTER *pAd, VOID *pData)
 #ifdef DOT11_N_SUPPORT
 	BOOLEAN FlgIsChanged;
 #endif /* DOT11_N_SUPPORT */
-
-#ifdef RT_CFG80211_P2P_MULTI_CHAN_SUPPORT
 	BSS_STRUCT *pMbss = &pAd->ApCfg.MBSSID[CFG_GO_BSSID_IDX];
 	struct wifi_dev *wdev = &pMbss->wdev;
-#endif /* RT_CFG80211_P2P_MULTI_CHAN_SUPPORT */
-
-
 /*
  *  enum nl80211_channel_type {
  *	NL80211_CHAN_NO_HT,
@@ -603,13 +599,15 @@ BOOLEAN CFG80211DRV_OpsSetChannel(RTMP_ADAPTER *pAd, VOID *pData)
 #endif /* RT_CFG80211_P2P_MULTI_CHAN_SUPPORT */
 
 			pAd->CommonCfg.RegTransmitSetting.field.BW = BW_20;
-			pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_NONE;
+			wdev->extcha= EXTCHA_NONE;			
+			HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
 			pAd->CommonCfg.HT_Disable = 0;
 		}
 		else if (ChannelType == RT_CMD_80211_CHANTYPE_HT40MINUS)
 		{
 			pAd->CommonCfg.RegTransmitSetting.field.BW = BW_40;
-			pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_BELOW;
+			wdev->extcha= EXTCHA_BELOW;
+			HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
 			pAd->CommonCfg.HT_Disable = 0;
 #ifdef RT_CFG80211_P2P_MULTI_CHAN_SUPPORT
 			if(IfType == RT_CMD_80211_IFTYPE_AP ||
@@ -619,9 +617,9 @@ BOOLEAN CFG80211DRV_OpsSetChannel(RTMP_ADAPTER *pAd, VOID *pData)
 				wdev->bw = pAd->CommonCfg.RegTransmitSetting.field.BW;
 				pAd->CommonCfg.Channel = ChanId;
 
-				if(pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_BELOW)
+				if(wdev->extcha == EXTCHA_BELOW)
 					pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel - 2;
-				else if (pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+				else if (wdev->extcha == EXTCHA_ABOVE)
 					pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel + 2;
 				else
 					pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel;
@@ -637,7 +635,8 @@ BOOLEAN CFG80211DRV_OpsSetChannel(RTMP_ADAPTER *pAd, VOID *pData)
 			/* not support NL80211_CHAN_HT40MINUS or NL80211_CHAN_HT40PLUS */
 			/* i.e. primary channel = 36, secondary channel must be 40 */
 			pAd->CommonCfg.RegTransmitSetting.field.BW = BW_40;
-			pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_ABOVE;
+			wdev->extcha = EXTCHA_ABOVE;			
+			HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
 			pAd->CommonCfg.HT_Disable = 0;
 #ifdef RT_CFG80211_P2P_MULTI_CHAN_SUPPORT
 			if(IfType == RT_CMD_80211_IFTYPE_AP ||
@@ -646,9 +645,9 @@ BOOLEAN CFG80211DRV_OpsSetChannel(RTMP_ADAPTER *pAd, VOID *pData)
 				wdev->bw = pAd->CommonCfg.RegTransmitSetting.field.BW;
 				pAd->CommonCfg.Channel = ChanId;
 
-				if(pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_BELOW)
+				if(wdev->extcha == EXTCHA_BELOW)
 					pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel - 2;
-				else if (pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+				else if (wdev->extcha == EXTCHA_ABOVE)
 					pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel + 2;
 				else
 					pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel;
@@ -663,7 +662,8 @@ BOOLEAN CFG80211DRV_OpsSetChannel(RTMP_ADAPTER *pAd, VOID *pData)
 		else if  (ChannelType == RT_CMD_80211_CHANTYPE_NOHT)
 		{
 			pAd->CommonCfg.RegTransmitSetting.field.BW = BW_20;
-			pAd->CommonCfg.RegTransmitSetting.field.EXTCHA = EXTCHA_NONE;
+			wdev->extcha = EXTCHA_NONE;			
+			HcUpdateExtCha(pAd,wdev->channel,wdev->extcha);
 			pAd->CommonCfg.HT_Disable = 1;
 #ifdef RT_CFG80211_P2P_MULTI_CHAN_SUPPORT
 			if(IfType == RT_CMD_80211_IFTYPE_AP ||
@@ -688,14 +688,14 @@ BOOLEAN CFG80211DRV_OpsSetChannel(RTMP_ADAPTER *pAd, VOID *pData)
 	}
 
 	if (FlgIsChanged == TRUE)
-		SetCommonHtVht(pAd);
+		SetCommonHtVht(pAd,NULL);
 
 	/* switch to the channel with Common Channel */
 	pAd->CommonCfg.Channel = ChanId;
 
-        if(pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_BELOW)
+        if(wdev->extcha == EXTCHA_BELOW)
               pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel - 2;
-        else if (pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+        else if (wdev->extcha == EXTCHA_ABOVE)
               pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel + 2;
         else
         	pAd->CommonCfg.CentralChannel = pAd->CommonCfg.Channel;
@@ -706,7 +706,7 @@ BOOLEAN CFG80211DRV_OpsSetChannel(RTMP_ADAPTER *pAd, VOID *pData)
 
 	CFG80211DBG(DBG_LVL_TRACE, ("80211> CentralChannel = %d, New BW = %d with Ext[%d]\n",
 		pAd->CommonCfg.CentralChannel, pAd->CommonCfg.RegTransmitSetting.field.BW,
-		pAd->CommonCfg.RegTransmitSetting.field.EXTCHA));
+		wdev->extcha));
 
 	if(IfType == RT_CMD_80211_IFTYPE_AP ||
 	   IfType == RT_CMD_80211_IFTYPE_P2P_GO)
@@ -1181,14 +1181,14 @@ VOID CFG80211_RegRuleApply(
 				continue;
 			}
 
-			if (!WMODE_CAP_2G(pAd->CommonCfg.PhyMode))
+			if (!WMODE_CAP_2G(pAd->CommonCfg.cfg_wmode))
 			{
 				/* 5G-only mode */
 				if (ChanId <= CFG80211_NUM_OF_CHAN_2GHZ)
 					continue;
 			}
 
-			if (!WMODE_CAP_5G(pAd->CommonCfg.PhyMode))
+			if (!WMODE_CAP_5G(pAd->CommonCfg.cfg_wmode))
 			{
 				/* 2.4G-only mode */
 				if (ChanId > CFG80211_NUM_OF_CHAN_2GHZ)
@@ -1197,12 +1197,14 @@ VOID CFG80211_RegRuleApply(
 
 			for(IdPwr=0; IdPwr<MAX_NUM_OF_CHANNELS; IdPwr++)
 			{
-				if (ChanId == pAd->TxPower[IdPwr].Channel)
+				//sachin - TODO
+				//if (ChanId == pAd->TxPower[IdPwr].Channel)
 				{
+					//sachin - TODO
 					/* init the channel info. */
-					os_move_mem(&pAd->ChannelList[RecId],
+					/*os_move_mem(&pAd->ChannelList[RecId],
 									&pAd->TxPower[IdPwr],
-									sizeof(CHANNEL_TX_POWER));
+									sizeof(CHANNEL_TX_POWER));*/
 
 					/* keep channel number */
 					pAd->ChannelList[RecId].Channel = ChanId;
@@ -1504,7 +1506,7 @@ UCHAR CFG80211_getCenCh(RTMP_ADAPTER *pAd, UCHAR prim_ch)
 
 	if (pAd->CommonCfg.RegTransmitSetting.field.BW == BW_40)
 	{
-		if (pAd->CommonCfg.RegTransmitSetting.field.EXTCHA == EXTCHA_ABOVE)
+		if (pAd->StaCfg[0].wdev.extcha== EXTCHA_ABOVE)
 			ret_channel = prim_ch + 2;
 		else
 		{
@@ -1596,8 +1598,8 @@ VOID CFG80211_JoinIBSS(
 
 	MakeOutgoingFrame(pBeaconFrame, &FrameLen,
 			  1, &SsidIe,
-			  1, &pAd->CommonCfg.SsidLen,
-			  pAd->CommonCfg.SsidLen, pAd->CommonCfg.Ssid,
+			  1, &pAd->StaCfg[0].SsidLen,
+			  pAd->StaCfg[0].SsidLen, pAd->StaCfg[0].Ssid,
 			  1, &SupRateIe,
 			  1, &SupRateLen,
 			  SupRateLen, SupRate,
