@@ -68,6 +68,25 @@ struct dev_type_name_map{
 #define xdef_to_str(s)   def_to_str(s)
 #define def_to_str(s)    #s
 
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,18,0)) && defined (CONFIG_ANDROID)
+#define FIRST_EEPROM_FILE_PATH	"/data/router/etc_ro/Wireless/RT2860/"
+#define FIRST_AP_PROFILE_PATH		"/data/router/etc/Wireless/RT2860/RT2860.dat"
+#define FIRST_STA_PROFILE_PATH      "/data/router/etc/Wireless/RT2860/RT2860.dat"
+#define FIRST_CHIP_ID	xdef_to_str(CONFIG_RT_FIRST_CARD)
+
+#define SECOND_EEPROM_FILE_PATH	"/data/router/etc_ro/Wireless/iNIC/"
+#define SECOND_AP_PROFILE_PATH	"/data/router/etc/Wireless/iNIC/iNIC_ap.dat"
+#define SECOND_STA_PROFILE_PATH "/data/router/etc/Wireless/iNIC/iNIC_sta.dat"
+
+#define SECOND_CHIP_ID	xdef_to_str(CONFIG_RT_SECOND_CARD)
+
+#define THIRD_EEPROM_FILE_PATH	"/data/router/etc_ro/Wireless/WIFI3/"
+#define THIRD_AP_PROFILE_PATH	"/data/router/etc/Wireless/WIFI3/RT2870AP.dat"
+#define THIRD_STA_PROFILE_PATH "/data/router/etc/Wireless/WIFI3/RT2870AP.dat"
+
+#define THIRD_CHIP_ID	xdef_to_str(CONFIG_RT_THIRD_CARD)
+
+#else
 #define FIRST_EEPROM_FILE_PATH	"/etc_ro/Wireless/RT2860/"
 #define FIRST_AP_PROFILE_PATH		"/etc/Wireless/RT2860/RT2860.dat"
 #define FIRST_STA_PROFILE_PATH      "/etc/Wireless/RT2860/RT2860.dat"
@@ -84,6 +103,7 @@ struct dev_type_name_map{
 #define THIRD_STA_PROFILE_PATH "/etc/Wireless/WIFI3/RT2870AP.dat"
 
 #define THIRD_CHIP_ID	xdef_to_str(CONFIG_RT_THIRD_CARD)
+#endif /* CONFIG_ANDROID */
 
 static struct dev_type_name_map prefix_map[] =
 {
@@ -144,6 +164,17 @@ INT get_dev_config_idx(RTMP_ADAPTER *pAd)
 #endif /* MULTI_INF_SUPPORT */
 		idx = 0;
 
+
+#if defined(CONFIG_RT_SECOND_CARD)
+#if defined(CONFIG_FIRST_IF_MT7603E)
+/*
+	MT7603(ra0) + MT7615 (rai0) combination
+*/
+	if(IS_MT7615(pAd))
+		idx = 1;
+#endif /* defined(CONFIG_FIRST_IF_MT7603E) */
+#endif /* defined(CONFIG_RT_SECOND_CARD) */
+
 	pAd->dev_idx = idx;
 
 	return idx;
@@ -644,6 +675,7 @@ void announce_802_3_packet(
 #ifdef MAT_SUPPORT
 		if (RTMP_MATPktRxNeedConvert(pAd, RtmpOsPktNetDevGet(pRxPkt))) {
 #if defined (CONFIG_WIFI_PKT_FWD)
+		if ((wf_fwd_needed_hook != NULL) && (wf_fwd_needed_hook() == TRUE)) {
 			BOOLEAN	 need_drop = FALSE;
 
 			ApCliLinkCoverRxPolicy(pAd, pPacket, &need_drop);
@@ -652,6 +684,7 @@ void announce_802_3_packet(
 				RELEASE_NDIS_PACKET(pAd, pRxPkt, NDIS_STATUS_FAILURE);
 				return;
 			}
+		}
 #endif /* CONFIG_WIFI_PKT_FWD */
 			RTMP_MATEngineRxHandle(pAd, pRxPkt, 0);
 		}
@@ -806,74 +839,59 @@ void announce_802_3_packet(
 #endif
 
 #if defined (CONFIG_WIFI_PKT_FWD)
+		if ((wf_fwd_needed_hook != NULL) && (wf_fwd_needed_hook() == TRUE)) 
 		{
 			struct sk_buff *pOsRxPkt = RTPKT_TO_OSPKT(pRxPkt);
-			PNET_DEV rx_dev = RtmpOsPktNetDevGet(pRxPkt);
-			RTMP_CLEAN_PACKET_BAND(pOsRxPkt);
-			RTMP_CLEAN_PACKET_RECV_FROM(pOsRxPkt);
-
-			/* all incoming packets should set CB to mark off which net device received and in which band */
-			if ((rx_dev == rlt_dev_get_by_name("rai0")) || (rx_dev == rlt_dev_get_by_name("apclii0")) ||
-				(rx_dev == rlt_dev_get_by_name("rai1")) || (rx_dev == rlt_dev_get_by_name("apclii1"))) {
-				RTMP_SET_PACKET_BAND(pOsRxPkt, RTMP_PACKET_SPECIFIC_5G);
-
-				if (NdisEqualMemory(pOsRxPkt->dev->name, "apcli", 5))
-					RTMP_SET_PACKET_RECV_FROM(pOsRxPkt, RTMP_PACKET_RECV_FROM_5G_CLIENT);
-				else
-					RTMP_SET_PACKET_RECV_FROM(pOsRxPkt, RTMP_PACKET_RECV_FROM_5G_AP);
-			} else {
-				RTMP_SET_PACKET_BAND(pOsRxPkt, RTMP_PACKET_SPECIFIC_2G);
-			
-				if (NdisEqualMemory(pOsRxPkt->dev->name, "apcli", 5))
-					RTMP_SET_PACKET_RECV_FROM(pOsRxPkt, RTMP_PACKET_RECV_FROM_2G_CLIENT);
-				else
-					RTMP_SET_PACKET_RECV_FROM(pOsRxPkt, RTMP_PACKET_RECV_FROM_2G_AP);
-			}
-		
-
-			if (wf_fwd_rx_hook != NULL)
-			{
-				struct ethhdr *mh = eth_hdr(pRxPkt);
-				int ret = 0;
-				
-				if ((mh->h_dest[0] & 0x1) == 0x1)
+ 			if (RTMP_IS_PACKET_AP_APCLI(pOsRxPkt))
+ 			{	
+				if (wf_fwd_rx_hook != NULL)
 				{
-					if (NdisEqualMemory(pOsRxPkt->dev->name, "apcli", 5)) 
-					{
-#ifdef MAC_REPEATER_SUPPORT
-						if ((pAd->ApCfg.bMACRepeaterEn == TRUE) &&
-							(RTMPQueryLookupRepeaterCliEntryMT(pAd, mh->h_source, TRUE) == TRUE)) {
-							RELEASE_NDIS_PACKET(pAd, pRxPkt, NDIS_STATUS_FAILURE);
-							return;
-						}
-						else
-#endif /* MAC_REPEATER_SUPPORT */
+					struct ethhdr *mh = eth_hdr(pRxPkt);
+					int ret = 0;
+					
+					if ((mh->h_dest[0] & 0x1) == 0x1)
+					{ 
+						if (RTMP_IS_PACKET_APCLI(pOsRxPkt))
 						{
-							VOID *opp_band_tbl = NULL;
-							VOID *band_tbl = NULL;
-
-							if (wf_fwd_feedback_map_table)
-								wf_fwd_feedback_map_table(pAd, &band_tbl, &opp_band_tbl);
-							
-							
-							if ((opp_band_tbl != NULL) 
-								&& MAC_ADDR_EQUAL(((UCHAR *)((REPEATER_ADAPTER_DATA_TABLE *)opp_band_tbl)->Wdev_ifAddr), mh->h_source)) {
+#ifdef MAC_REPEATER_SUPPORT
+							if ((pAd->ApCfg.bMACRepeaterEn == TRUE) &&
+								(RTMPQueryLookupRepeaterCliEntryMT(pAd, mh->h_source, TRUE) == TRUE)) {
 								RELEASE_NDIS_PACKET(pAd, pRxPkt, NDIS_STATUS_FAILURE);
 								return;
 							}
+							else
+#endif /* MAC_REPEATER_SUPPORT */
+							{
+								VOID *opp_band_tbl = NULL;
+								VOID *band_tbl = NULL;
+
+								if (wf_fwd_feedback_map_table)
+									wf_fwd_feedback_map_table(pAd, &band_tbl, &opp_band_tbl);
+								
+								
+								if ((opp_band_tbl != NULL) 
+									&& MAC_ADDR_EQUAL(((UCHAR *)((REPEATER_ADAPTER_DATA_TABLE *)opp_band_tbl)->Wdev_ifAddr), mh->h_source)) {
+									RELEASE_NDIS_PACKET(pAd, pRxPkt, NDIS_STATUS_FAILURE);
+									return;
+								}
+							}
 						}
 					}
-				}
-				
-				ret = wf_fwd_rx_hook(pRxPkt);
+					
+					ret = wf_fwd_rx_hook(pRxPkt);
 
-				if (ret == 0) {
-					return;
-				} else if (ret == 2) {
-					RELEASE_NDIS_PACKET(pAd, pRxPkt, NDIS_STATUS_FAILURE);
-					return;
+					if (ret == 0) {
+						return;
+					} else if (ret == 2) {
+						RELEASE_NDIS_PACKET(pAd, pRxPkt, NDIS_STATUS_FAILURE);
+						return;
+					}
 				}
-		}
+ 			}
+			else
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_TRACE, 
+				("No CB Packet RTMP_IS_PACKET_AP_APCLI(%d)\n", RTMP_IS_PACKET_AP_APCLI(pOsRxPkt)));
+				
 	}
 #endif /* CONFIG_WIFI_PKT_FWD */
 
@@ -1086,12 +1104,14 @@ int RTMPSendPackets(
 
 
 #if defined (CONFIG_WIFI_PKT_FWD)
-	if (wf_fwd_tx_hook != NULL)
-	{
-		if (wf_fwd_tx_hook(pPacket) == 1)
+	if ((wf_fwd_needed_hook != NULL) && (wf_fwd_needed_hook() == TRUE)) {
+		if (wf_fwd_tx_hook != NULL)
 		{
-			RELEASE_NDIS_PACKET(pAd, pPacket, NDIS_STATUS_FAILURE);
-			return 0;
+			if (wf_fwd_tx_hook(pPacket) == 1)
+			{
+				RELEASE_NDIS_PACKET(pAd, pPacket, NDIS_STATUS_FAILURE);
+				return 0;
+			}
 		}
 	}
 #endif /* CONFIG_WIFI_PKT_FWD */
