@@ -1272,6 +1272,16 @@ static void bmc_dma_done_tasklet(unsigned long data)
         return;
     }
 
+#ifdef ERR_RECOVERY
+	if (IsStopingPdma(&pAd->ErrRecoveryCtl))
+	{
+		RTMP_INT_LOCK(&pAd->LockInterrupt, flags);
+		pAd->PciHif.intDisableMask &= ~(INT_BMC_DLY);
+		RTMP_INT_UNLOCK(&pAd->LockInterrupt, flags);
+		return;
+	}
+#endif /* ERR_RECOVERY */
+
 	pObj = (POS_COOKIE) pAd->OS_Cookie;
 
     RTMP_INT_LOCK(&pAd->LockInterrupt, flags);
@@ -2010,6 +2020,7 @@ redo:
 #ifdef ERR_RECOVERY
             if (value & ERROR_DETECT_MASK)
             {
+				UINT32 reg_tmp_val;
                 rt2860_int_disable(pAd, McuCommand);
 
                 /* updated ErrRecovery Status. */
@@ -2017,6 +2028,9 @@ redo:
 
                 /* Trigger error recovery process with fw reload. */
                 RTMP_OS_TASKLET_SCHE(&pObj->mac_error_recovey_task);
+                MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s,::E  R  , status=0x%08X\n", __FUNCTION__, value));
+				MAC_IO_READ32(pAd, MCU_COM_REG1, &reg_tmp_val);
+				MTWF_LOG(DBG_CAT_ALL, DBG_SUBCAT_ALL, DBG_LVL_OFF, ("%s,::E  R  , MCU_COM_REG1=0x%08X\n", __FUNCTION__, reg_tmp_val));
             }
 #endif /* ERR_RECOVERY */
 
