@@ -118,7 +118,8 @@ raWrapperEntrySet(
 
     pRaEntry->ucPhyMode = pEntry->wdev->PhyMode;
     pRaEntry->ucChannel= pEntry->wdev->channel;
-    pRaEntry->ucBBPCurrentBW = pAd->CommonCfg.BBPCurrentBW;
+	//use the maximum bw capability
+	pRaEntry->ucBBPCurrentBW = HcGetBw(pAd, pEntry->wdev);
 
     {
         pRaEntry->fgDisableCCK = FALSE;
@@ -197,6 +198,7 @@ raWrapperEntrySet(
     pRaEntry->TxPhyCfg.STBC = pEntry->HTPhyMode.field.STBC;
     pRaEntry->TxPhyCfg.ShortGI = pEntry->HTPhyMode.field.ShortGI;
     pRaEntry->TxPhyCfg.BW = pEntry->HTPhyMode.field.BW;
+
     pRaEntry->TxPhyCfg.ldpc = pEntry->HTPhyMode.field.ldpc;
 #ifdef DOT11_N_SUPPORT
 #ifdef DOT11_VHT_AC
