@@ -351,7 +351,6 @@ INT32 mt_AsicBfStaRecUpdate(
     RTMP_ADAPTER *pAd, 
     UCHAR        ucPhyMode,
     UCHAR        ucBssIdx,
-    IE_LISTS     *pIeList,
 	UCHAR        ucWlanIdx)
 {
     PMAC_TABLE_ENTRY pEntry;
@@ -368,7 +367,7 @@ INT32 mt_AsicBfStaRecUpdate(
 	{
 #ifdef DOT11_VHT_AC	
 	case MODE_VHT:
-	    fgETxBfCap                           = pAd->chipOps.ClientSupportsVhtETxBF(pAd, &pIeList->vht_cap.vht_cap);
+	    fgETxBfCap                           = pAd->chipOps.ClientSupportsVhtETxBF(pAd, &pEntry->vht_cap_ie.vht_cap);
 	    fgETxBfCap                           &= pAd->CommonCfg.ETxBfEnCond;
 	    pEntry->rStaRecBf.fgETxBfCap         = fgETxBfCap;
 	    pEntry->rStaRecBf.fgSU_MU            = FALSE;
@@ -378,9 +377,9 @@ INT32 mt_AsicBfStaRecUpdate(
 	    if (fgETxBfCap)
 	    {
 #ifdef CFG_SUPPORT_MU_MIMO	    
-	        if (pIeList->vht_cap.vht_cap.bfee_cap_mu == TRUE)
+	        if (pAd->CommonCfg.vht_cap_ie.vht_cap.bfer_cap_mu == TRUE)
 	        {
-	            pEntry->rStaRecBf.fgSU_MU    = TRUE;
+	            pEntry->rStaRecBf.fgSU_MU    = pEntry->vht_cap_ie.vht_cap.bfee_cap_mu;
 	        }
 #endif			
 	        pEntry->rStaRecBf.ucSoundingPhy  = 1; // OFDM
@@ -395,25 +394,28 @@ INT32 mt_AsicBfStaRecUpdate(
 	        pEntry->rStaRecBf.ucReptPollRate = 9; // 9 : OFDM 24M
 
 	        ucBFerMaxNr                      = pAd->CommonCfg.vht_cap_ie.vht_cap.num_snd_dimension;
-	        ucBFeeMaxNr                      = pIeList->vht_cap.vht_cap.bfee_sts_cap;
+	        ucBFeeMaxNr                      = pEntry->vht_cap_ie.vht_cap.bfee_sts_cap;
 	        ucBFerCurrNr                     = pAd->CommonCfg.TxStream - 1;
 	        pEntry->rStaRecBf.ucNr           = (ucBFerMaxNr < ucBFeeMaxNr) ? ucBFerMaxNr : ucBFeeMaxNr;
 	        pEntry->rStaRecBf.ucNr           = (pEntry->rStaRecBf.ucNr < ucBFerCurrNr) ? pEntry->rStaRecBf.ucNr : ucBFerCurrNr;
 
-	        ucPeerRxNumSupport               = (pIeList->vht_cap.mcs_set.rx_mcs_map.mcs_ss2 != 3) ? 1 : 0;
-            ucPeerRxNumSupport               = (pIeList->vht_cap.mcs_set.rx_mcs_map.mcs_ss3 != 3) ? 2 : ucPeerRxNumSupport;
+	        ucPeerRxNumSupport               = (pEntry->vht_cap_ie.mcs_set.rx_mcs_map.mcs_ss2 != 3) ? 1 : 0;
+            ucPeerRxNumSupport               = (pEntry->vht_cap_ie.mcs_set.rx_mcs_map.mcs_ss3 != 3) ? 2 : ucPeerRxNumSupport;
 	        pEntry->rStaRecBf.ucNc           = ucPeerRxNumSupport;
+	        pEntry->rStaRecBf.uciBfNcol      = pEntry->rStaRecBf.ucNc;
+	        pEntry->rStaRecBf.uciBfNrow      = pEntry->rStaRecBf.ucNr;
+	        pEntry->rStaRecBf.uciBfTimeOut   = 0xFE;
 	    }
 	    else
 	    {
 	        pEntry->rStaRecBf.ucNr           = pAd->CommonCfg.TxStream - 1;
 	        
-	        pEntry->rStaRecBf.uciBfTimeOut   = 0xFF;
+	        pEntry->rStaRecBf.uciBfTimeOut   = 0xFE;
             pEntry->rStaRecBf.uciBfDBW       = pEntry->MaxHTPhyMode.field.BW;
             pEntry->rStaRecBf.uciBfNrow      = pAd->CommonCfg.TxStream - 1;
 
-            ucPeerRxNumSupport               = (pIeList->vht_cap.mcs_set.tx_mcs_map.mcs_ss2 != 3) ? 1 : 0;
-            ucPeerRxNumSupport               = (pIeList->vht_cap.mcs_set.tx_mcs_map.mcs_ss3 != 3) ? 2 : ucPeerRxNumSupport;
+            ucPeerRxNumSupport               = (pEntry->vht_cap_ie.mcs_set.tx_mcs_map.mcs_ss2 != 3) ? 1 : 0;
+            ucPeerRxNumSupport               = (pEntry->vht_cap_ie.mcs_set.tx_mcs_map.mcs_ss3 != 3) ? 2 : ucPeerRxNumSupport;
 	        pEntry->rStaRecBf.ucNc           = ucPeerRxNumSupport;
 	        pEntry->rStaRecBf.uciBfNcol      = ucPeerRxNumSupport;
 	    }
@@ -422,7 +424,7 @@ INT32 mt_AsicBfStaRecUpdate(
 #endif /* DOT11_VHT_AC */
 	case MODE_HTMIX:
 	case MODE_HTGREENFIELD:
-	    fgETxBfCap                           = pAd->chipOps.ClientSupportsETxBF(pAd, &pIeList->HTCapability.TxBFCap);
+	    fgETxBfCap                           = pAd->chipOps.ClientSupportsETxBF(pAd, &pEntry->HTCapability.TxBFCap);
 	    fgETxBfCap                           &= pAd->CommonCfg.ETxBfEnCond;
 	    pEntry->rStaRecBf.fgETxBfCap         = fgETxBfCap;
 	    pEntry->rStaRecBf.ucTxMode           = 2; // HT mode
@@ -432,21 +434,24 @@ INT32 mt_AsicBfStaRecUpdate(
 	    {
 	        pEntry->rStaRecBf.fgSU_MU        = 0;
 	        ucBFerMaxNr                      = pAd->CommonCfg.HtCapability.TxBFCap.ChanEstimation;
-	        ucBFeeMaxNr                      = pIeList->HTCapability.TxBFCap.ComSteerBFAntSup;
+	        ucBFeeMaxNr                      = pEntry->HTCapability.TxBFCap.ComSteerBFAntSup;
 	        ucBFerCurrNr                     = pAd->CommonCfg.TxStream - 1;
 	        pEntry->rStaRecBf.ucNr           = (ucBFerMaxNr < ucBFeeMaxNr) ? ucBFerMaxNr : ucBFeeMaxNr;
 	        pEntry->rStaRecBf.ucNr           = (pEntry->rStaRecBf.ucNr < ucBFerCurrNr) ? pEntry->rStaRecBf.ucNr : ucBFerCurrNr;
 	        pEntry->rStaRecBf.ucNdpRate      = pEntry->rStaRecBf.ucNr * 8;    
 
-	        ucPeerRxNumSupport               = (pIeList->HTCapability.MCSSet[1] > 0)? 1 : 0;
-	        ucPeerRxNumSupport               = (pIeList->HTCapability.MCSSet[2] > 0)? 2 : ucPeerRxNumSupport;
+	        ucPeerRxNumSupport               = (pEntry->HTCapability.MCSSet[1] > 0)? 1 : 0;
+	        ucPeerRxNumSupport               = (pEntry->HTCapability.MCSSet[2] > 0)? 2 : ucPeerRxNumSupport;
 	        pEntry->rStaRecBf.ucNc           = ucPeerRxNumSupport;
+	        pEntry->rStaRecBf.uciBfNcol      = pEntry->rStaRecBf.ucNc;
+	        pEntry->rStaRecBf.uciBfNrow      = pEntry->rStaRecBf.ucNr;
+	        pEntry->rStaRecBf.uciBfTimeOut   = 0xFE;
 	    }
 	    else
 	    {
 	        pEntry->rStaRecBf.ucNr           = pAd->CommonCfg.TxStream - 1;
 	        
-	        pEntry->rStaRecBf.uciBfTimeOut   = 0xFF;
+	        pEntry->rStaRecBf.uciBfTimeOut   = 0xFE;
             pEntry->rStaRecBf.uciBfDBW       = pEntry->MaxHTPhyMode.field.BW;
             pEntry->rStaRecBf.uciBfNrow      = pAd->CommonCfg.TxStream - 1;
 
@@ -467,18 +472,18 @@ INT32 mt_AsicBfStaRecUpdate(
             // |The Rx MCS set  |     1     |    1    |                  |
             // |________________|___________|_________|__________________|
 
-            ucTxMCSSetdefined                = ((pIeList->HTCapability.MCSSet[12] & 
+            ucTxMCSSetdefined                = ((pEntry->HTCapability.MCSSet[12] & 
                                                TX_MCS_SET_DEFINED) >>
                                                TX_MCS_SET_DEFINED_OFFSET) ;
-            ucTxRxMCSSetNotEqual             = ((pIeList->HTCapability.MCSSet[12] & 
+            ucTxRxMCSSetNotEqual             = ((pEntry->HTCapability.MCSSet[12] & 
                                                TX_RX_MCS_SET_N_EQUAL) >>
                                                TX_RX_MCS_SET_N_EQUAL_OFFSET) ;
-            ucTxMaxNumSpatilStream           = ((pIeList->HTCapability.MCSSet[12] & 
+            ucTxMaxNumSpatilStream           = ((pEntry->HTCapability.MCSSet[12] & 
                                                TX_MAX_NUM_SPATIAL_STREAMS_SUPPORTED) >>
                                                TX_MAX_NUM_SPATIAL_STREAMS_SUPPORTED_OFFSET);
 
-            ucPeerRxNumSupport               = (pIeList->HTCapability.MCSSet[1] > 0)? 1 : 0;
-	        ucPeerRxNumSupport               = (pIeList->HTCapability.MCSSet[2] > 0)? 2 : ucPeerRxNumSupport;
+            ucPeerRxNumSupport               = (pEntry->HTCapability.MCSSet[1] > 0)? 1 : 0;
+	        ucPeerRxNumSupport               = (pEntry->HTCapability.MCSSet[2] > 0)? 2 : ucPeerRxNumSupport;
 
             if ((ucTxMCSSetdefined == 1) && (ucTxRxMCSSetNotEqual == 1))
             {
@@ -499,7 +504,7 @@ INT32 mt_AsicBfStaRecUpdate(
 	    pEntry->rStaRecBf.ucNr               = pAd->CommonCfg.TxStream - 1;
 	    pEntry->rStaRecBf.ucCBW              = pEntry->MaxHTPhyMode.field.BW;	
 	    
-	    pEntry->rStaRecBf.uciBfTimeOut       = 0xFF;
+	    pEntry->rStaRecBf.uciBfTimeOut       = 0xFE;
         pEntry->rStaRecBf.uciBfDBW           = pEntry->MaxHTPhyMode.field.BW;
         pEntry->rStaRecBf.uciBfNcol          = 0;
         pEntry->rStaRecBf.uciBfNrow          = pAd->CommonCfg.TxStream - 1;
@@ -516,8 +521,22 @@ INT32 mt_AsicBfStaRecUpdate(
 	{
 	    return ucStatus;
 	}
-	
-	pEntry->rStaRecBf.ucMemRequire20M        = g_ru2PfmuMemReq[pEntry->rStaRecBf.ucNr][pEntry->rStaRecBf.ucNc];
+
+	if (pEntry->rStaRecBf.fgETxBfCap == TRUE)
+	{
+	    if (pEntry->rStaRecBf.ucNr == (pAd->CommonCfg.TxStream - 1))
+	    {
+	        pEntry->rStaRecBf.ucMemRequire20M= g_ru2PfmuMemReq[pEntry->rStaRecBf.ucNr][pEntry->rStaRecBf.ucNc];
+	    }
+	    else
+	    {
+	        pEntry->rStaRecBf.ucMemRequire20M= g_ru2PfmuMemReq[pAd->CommonCfg.TxStream - 1][pEntry->rStaRecBf.ucNc];
+	    }
+	}
+	else
+	{
+	    pEntry->rStaRecBf.ucMemRequire20M    = g_ru2PfmuMemReq[pEntry->rStaRecBf.ucNr][pEntry->rStaRecBf.ucNc];
+	}
 	pEntry->rStaRecBf.ucTotMemRequire        = pEntry->rStaRecBf.ucMemRequire20M * g_aPfmuTimeOfMem20M[pEntry->rStaRecBf.ucCBW];
 	pEntry->rStaRecBf.ucMemRow0              = 0;
 	pEntry->rStaRecBf.ucMemCol0              = 0;
@@ -633,6 +652,64 @@ INT32 mt_AsicBfStaRecRelease(
 }
 
 
+VOID mt_AsicClientBfCap(
+    RTMP_ADAPTER *pAd, 
+    PMAC_TABLE_ENTRY pEntry)
+{
+    if (pAd->fgBfProfAlloc == FALSE)
+    {
+#ifdef APCLI_SUPPORT        
+        if (pAd->fgApClientMode == TRUE)
+        {                
+            pAd->rStaRecBf.u2PfmuId    = 0xFFFF;
+            pEntry->rStaRecBf.u2PfmuId = 0xFFFF;
+
+            {
+		        STA_REC_CFG_T StaCfg;
+		        os_zero_mem(&StaCfg,sizeof(STA_REC_CFG_T));
+			
+		        StaCfg.MuarIdx = pEntry->wdev->OmacIdx;
+		        StaCfg.ConnectionState = TRUE;
+		        StaCfg.ConnectionType = 0;
+		        StaCfg.u4EnableFeature = (1 << STA_REC_BF);
+		        StaCfg.ucBssIndex = pEntry->wdev->BssIdx;
+		        StaCfg.ucWlanIdx = pEntry->wcid;
+		        StaCfg.pEntry = pEntry;
+
+	            CmdExtStaRecUpdate(pAd,StaCfg);
+	        }
+                
+            /* Force the Pfmu ID of the other repeater cli to be the same with ApCli */
+            AsicTxBfApClientCluster(pAd, pEntry->wcid, pAd->ApCli_CmmPfmuId);
+
+            //mdelay(1000);
+            //dump_wtbl_info(pAd, pMacEntry->wcid);
+
+            /* Force the Bf capability to be the same with ApCli */
+            AsicTxBfTxApplyCtrl(pAd,
+			        		    pEntry->wcid,
+				        		pAd->fgApCli_eBF,
+					        	pAd->fgApCli_iBF,
+						    	pAd->fgApCli_MuBF,
+							    FALSE);
+
+            //mdelay(1000);
+            //dump_wtbl_info(pAd, pMacEntry->wcid);
+
+            MTWF_LOG(DBG_CAT_HW, DBG_SUBCAT_ALL, DBG_LVL_ERROR, ("%s: wcid = %d, fgApCli_eBF = %d, fgApCli_iBF = %d, fgApCli_MuBF = %d\n", 
+                                                                 __FUNCTION__, 
+                                                                 pEntry->wcid,
+                                                                 pAd->fgApCli_eBF, 
+                                                                 pAd->fgApCli_iBF,
+                                                                 pAd->fgApCli_MuBF));
+        }
+#endif /* APCLI_SUPPORT */     
+    }
+	
+    return;
+}
+
+
 /* MlmeTxBfAllowed - returns true if ETxBF or ITxBF is supported and pTxRate is a valid BF mode */
 BOOLEAN MlmeTxBfAllowed(
 	IN RTMP_ADAPTER *pAd,
