@@ -370,6 +370,14 @@ INT	MtSingleSkuLoadParam(RTMP_ADAPTER *pAd)
 						}
 					}
 
+                    if ( isSame == TRUE )
+                    {
+                        if ( StartCh->band != pwr->band )
+                        {
+                            isSame = FALSE;
+                        }
+                    }
+
 					if ( isSame == TRUE )
 					{
 						os_free_mem( pwr);
@@ -719,6 +727,7 @@ INT	MtBfBackOffLoadTable(RTMP_ADAPTER *pAd)
                         pwr->PwrMax[i] = os_str_tol(token, 0, 10) * 2;
                 }
 
+                /* check the parameters similarity, if the same then we use condensed structure to save memory size */
                 if ( StartCh == NULL )
                 {
                     StartCh = pwr;
@@ -740,6 +749,14 @@ INT	MtBfBackOffLoadTable(RTMP_ADAPTER *pAd)
                         }
                     }
 
+                    if ( isSame == TRUE )
+                    {
+                        if ( StartCh->band != pwr->band )
+                        {
+                            isSame = FALSE;
+                        }
+                    }
+                    
                     if ( isSame == TRUE )
                     {
                         os_free_mem( pwr);
